// ClientMortar.cpp: implementation of the CClientMortar class.
//
//////////////////////////////////////////////////////////////////////

#include "ClientMortar.h"
#include "colors.h"
#include "RubicsObjects.h"
#include "RubicsDirect3D.h"
#include "RubicsMain.h"
#include "rodcone.h"
#include <d3dtypes.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CClientMortar::CClientMortar()
{
	self_frame = NULL;
	parent_frame = NULL;
	lpdsb = NULL;
	lpdsb3d = NULL;
}
CClientMortar::CClientMortar(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, Position* pos, PieceType type, SoundSet *pSound) 
{
	LPDIRECT3DRMMESHBUILDER3 meshBuilder, shadowBuilder;
	LPDIRECT3DRMMESH		 mesh, shadowMesh;
	parent_frame = parent;
	parent->AddRef();
	modified = FALSE;
	pSoundSet = pSound;
	bPlayedSound = FALSE;
	LPDIRECT3DRMSHADOW2 shadow = NULL;
	LPDIRECT3DRMFACE2 face;
	lpdsb = NULL;
	lpdsb3d = NULL;

	D3DVECTOR a,b;
	a.x = 0.25f;
	a.y = 0.25f;
	a.z = 0.0f;

	b.x = 0.25f;
	b.y = -0.25f;
	b.z = 0.0f;

	//Create the frame
	self_frame = NULL;
	//Setup defaults for the info. This will make the guy
	//stand still at the origin and do nothing.
	info = new rubiks_info;

	info->head  = 0.0f;
	info->pos = *pos;
	//info->pos.y = 0.0f;
	//info->pos.z = 0.0f;
	info->framecount = 0.0f;
	info->anim = STANDING;

	// Create the Mesh Builder for the Wagon
	if (FAILED( rm->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	if (FAILED( rm->CreateMeshBuilder( &shadowBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	// Load the X file
	if(type == CONECT_MORTAR_PROJ) 
	{
		AddRod(meshBuilder, 5.0f, a, b);
		Center(meshBuilder);
		meshBuilder->SetColor(((rand() % 2) ? BLACK : RED));

		a.x = 0.25f;
		a.y = 0.0f;
		a.z = 0.0f;
		b.x = 0.25f;
		b.y = 0.1f;
		b.z = 0.0f;
		AddRod(shadowBuilder, 4.5f, a, b);
		shadowBuilder->SetColor(D3DRGBA(0.0f, 0.0f, 0.0f, 80.0f));
	} 
	else 
	{
		/* Build the block */
	  if( FAILED(GetSquare(meshBuilder, 5.0, RED,GREEN, ORANGE, YELLOW, WHITE, BLUE)))
	  {
			DISPLAYMSG("Failed to create projectile.");
			goto error;
	  }
	
	  /*Build the shadow */
		if (FAILED (shadowBuilder->CreateFace(&face)))
		{
			DISPLAYMSG("Unable to create face");
			goto error;	
		}

		if (FAILED (face->SetColor(D3DRGBA(0.0f, 0.0f, 0.0f, 127.0f))))
			goto error;	

		if (FAILED (face->AddVertex(-2.5f, 0.1f, -2.5f)))
			goto error;	
		if (FAILED (face->AddVertex(-2.5f, 0.1f, 2.5f)))
			goto error;	
		if (FAILED (face->AddVertex(2.5f,  0.1f, 2.5f)))
			goto error;	
		if (FAILED (face->AddVertex(2.5f,  0.1f, -2.5f)))
			goto error;	
	}
	/*if( FAILED( meshBuilder->Load("missile.x",NULL,D3DRMLOAD_FROMFILE, NULL,NULL) ) )
	{
		DISPLAYMSG("Failed to load .X file");
		goto error;
	}
*/
	// Create Mesh to be used for the Wagon
	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create Projectile mesh");
		goto error;
	}

	if (FAILED( shadowBuilder->CreateMesh( &shadowMesh ) ) )
	{
		DISPLAYMSG("Unable to create Projectile mesh");
		goto error;
	}

	/*if (FAILED(mesh->Translate(0,0.9f,0)))
	{
		DISPLAYMSG("Unable to translate Projectile mesh");
		goto error;
	}*/

	
	// Create frame that the outside world will see
	if (FAILED(rm->CreateFrame( parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create self frame on Projectile");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to Projectile frame" );
		goto error;
	}

	/** Shadow stuff */
	if (FAILED(rm->CreateFrame( parent, &shadowFrame ) ) )
	{
		DISPLAYMSG("Unable to create self frame on Projectile");
		goto error;
	}

	if( FAILED( shadowFrame->AddVisual( shadowMesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to Projectile frame" );
		goto error;
	}


	if (FAILED(self_frame->SetPosition(parent, pos->x, pos->y, pos->z)))
				DISPLAYMSG("Error setting Position on Projectile");

	/*if (FAILED(self_frame->SetOrientation(parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on Non-Movable Object");*/
	
error:
	RELEASE(shadow);
	RELEASE( meshBuilder );
	RELEASE( mesh );
	RELEASE( shadowBuilder );
	RELEASE( shadowMesh );
}

CClientMortar::~CClientMortar()
{
	parent_frame->DeleteChild(self_frame);
	parent_frame->DeleteChild(shadowFrame);

	RELEASE(parent_frame);
	RELEASE(shadowFrame);
	RELEASE(self_frame);

	/*Info*/
	delete info;
	info = NULL;
}

/** Updates the drawing of the cube based on
  * the information it's received
  */
void CClientMortar::Update()
{
	D3DVALUE scaleFactor = 1.0f - info->pos.y / 50.0f;

	//Only re-evaluate ourselves if a call to ReceiveMessage was called
	if (modified == FALSE) return;
	self_frame->AddRotation(D3DRMCOMBINE_REPLACE, 1.0f, 0.0f, 0.0f, 0.2f*info->framecount);
	 //Move the rubiks cube to the proper location and heading	
	self_frame->SetPosition(parent_frame, info->pos.x, info->pos.y, info->pos.z);

	shadowFrame->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0.2f*info->framecount);
	shadowFrame->SetPosition(parent_frame, info->pos.x, 0.0f, info->pos.z);
	shadowFrame->AddScale(D3DRMCOMBINE_BEFORE, scaleFactor, scaleFactor, scaleFactor);
	if( !bPlayedSound && pSoundSet )
	{
		pSoundSet->Play( SHIELD, info->pos, info->pos, &lpdsb, &lpdsb3d );
		bPlayedSound = TRUE;
	}

	//COMMENTED OUT IN HOPES OF MAKING THE MORTAR NOT CRASH - KEVIN
//	lpdsb3d->SetPosition( info->pos.x, info->pos.y, info->pos.z, DS3D_IMMEDIATE );

	  //I'M GUESSING THIS IS WHAT YOU WANT?
	  //D3DVALUE s = ((info->framecount / 30.0) * 4.0f) + 1.0f;
	  //self_frame->AddScale(D3DRMCOMBINE_BEFORE, s, s, s);

	modified = FALSE;
}
