// ClientPlayer.cpp: implementation of the CClientPlayer class.
//
//////////////////////////////////////////////////////////////////////

#include "ClientPlayer.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
int CClientPlayer::GetCenterIndex(int index)
{
	switch (index)
	{
		case 4:  return RED_SQUARE;
		case 14: return GREEN_SQUARE;
		case 22: return ORANGE_SQUARE;
		case 12: return BLUE_SQUARE;
		case 16: return YELLOW_SQUARE;
		case 10: return WHITE_SQUARE;
	}
	assert(0);	// Shouldn't get this far
	return 0;
}

int CClientPlayer::GetCubeIndex(int index)
{
	switch (index)
	{
		case RED_SQUARE:    return 4;
		case GREEN_SQUARE:  return 14;
		case ORANGE_SQUARE: return 22;
		case BLUE_SQUARE:   return 12;
		case YELLOW_SQUARE: return 16;
		case WHITE_SQUARE:  return 10;
	}
	assert(0);	// Shouldn't get this far
	return 0;
}


void CClientPlayer::PickupPiece(PieceType piece) 
{
	int index;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		centerCube[RED_SQUARE] = true;
		index = GetCubeIndex(RED_SQUARE);
		break;
	case BLUE_RUBIKS_PIECE:
		centerCube[BLUE_SQUARE] = true;
		index = GetCubeIndex(BLUE_SQUARE);
		break;
	case GREEN_RUBIKS_PIECE:
		centerCube[GREEN_SQUARE] = true;
		index = GetCubeIndex(GREEN_SQUARE);
		break;
	case YELLOW_RUBIKS_PIECE:
		centerCube[YELLOW_SQUARE] = true;
		index = GetCubeIndex(YELLOW_SQUARE);
		break;
	case WHITE_RUBIKS_PIECE:
		centerCube[WHITE_SQUARE] = true;
		index = GetCubeIndex(WHITE_SQUARE);
		break;
	case ORANGE_RUBIKS_PIECE:
		centerCube[ORANGE_SQUARE] = true;
		index = GetCubeIndex(ORANGE_SQUARE);
		break;
	case MISSILE:
		currentWeapon = Tornado;
		return;
		break;
	default:
		// This wasn't a Rubiks piece so don't do nuthin!
		return;
		break;
	}
	
	//Add the frame to the cube for the block we just picked up
//	assert(SUCCEEDED(self_frame->AddChild(blocks[index])));
}


void CClientPlayer::DropPiece(PieceType piece) 
{
	int index;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		centerCube[RED_SQUARE] = false;
		index = GetCubeIndex(RED_SQUARE);
		break;
	case BLUE_RUBIKS_PIECE:
		centerCube[BLUE_SQUARE] = false;
		index = GetCubeIndex(BLUE_SQUARE);
		break;
	case GREEN_RUBIKS_PIECE:
		centerCube[GREEN_SQUARE] = false;
		index = GetCubeIndex(GREEN_SQUARE);
		break;
	case YELLOW_RUBIKS_PIECE:
		centerCube[YELLOW_SQUARE] = false;
		index = GetCubeIndex(YELLOW_SQUARE);
		break;
	case WHITE_RUBIKS_PIECE:
		centerCube[WHITE_SQUARE] = false;
		index = GetCubeIndex(WHITE_SQUARE);
		break;
	case ORANGE_RUBIKS_PIECE:
		centerCube[ORANGE_SQUARE] = false;
		index = GetCubeIndex(ORANGE_SQUARE);
		break;
	default:
		// This wasn't a Rubiks piece so don't do nuthin!
		return;
		break;
	}
	
	//Remove the frame from the cube 
	//assert(SUCCEEDED(self_frame->DeleteChild(blocks[index])));
}

BOOL CClientPlayer::IsPlayer()
{
	return true;
}


PieceType
CClientPlayer::GetCenterCube(int index){

	PieceType type = NONE;
	switch(index) {
	case RED_SQUARE:
		if (centerCube[RED_SQUARE])
			type = RED_RUBIKS_PIECE;
		
		break;
	case GREEN_SQUARE:
		if (centerCube[GREEN_SQUARE])
			type = GREEN_RUBIKS_PIECE;
		break;
	case ORANGE_SQUARE:
		if(centerCube[ORANGE_SQUARE])
			type = ORANGE_RUBIKS_PIECE;
		break;
	case BLUE_SQUARE:
		if(centerCube[BLUE_SQUARE])
			type = BLUE_RUBIKS_PIECE;
		break;
	case YELLOW_SQUARE:
		if(centerCube[YELLOW_SQUARE])
			type = YELLOW_RUBIKS_PIECE;
		break;
	case WHITE_SQUARE:
		if(centerCube[WHITE_SQUARE])
			type = WHITE_RUBIKS_PIECE;
		break;
	default:
		break;
	}
	return type;

}

void CClientPlayer::SetHero()
{
	bHero = true;		
}

BOOL CClientPlayer::IsHero()
{
	return bHero;
}


void CClientPlayer::getPlayerIcon(LPDIRECT3DRM3 rm, LPLPDIRECT3DRMFRAME3 frame, 
							      LPDIRECT3DRMFRAME3 parent, bool isDrEvil)
{
	
	LPDIRECT3DRMMESHBUILDER3 mesh = NULL;
	LPDIRECT3DRMMATERIAL2 mat = NULL;
	LPDIRECT3DRMMESH cubeMesh = NULL;
	LPDIRECT3DRMTEXTURE3 pTex = NULL;
	LPDIRECT3DRMWRAP lpWrap = NULL;
	LPDIRECT3DRMFACE2 lpFace = NULL;

	float fSize = 6;

	//Create the mesh builder
	if (FAILED (rm->CreateMeshBuilder(&mesh)))
		goto error;
	
	//create the face
	if (FAILED (mesh->CreateFace(&lpFace)))
		goto error;
/*
	if (FAILED (lpFace->SetColor(color)))
		goto error;
*/
	//Build the face. 
	if (FAILED (lpFace->AddVertex(-fSize/2.0f, 0.0f,  0.0f)))
		goto error;
	if (FAILED (lpFace->AddVertex( fSize/2.0f, 0.0f,  0.0f)))
		goto error;
	if (FAILED (lpFace->AddVertex( fSize/2.0f, fSize, 0.0f)))
		goto error;
	if (FAILED (lpFace->AddVertex(-fSize/2.0f, fSize, 0.0f)))
		goto error;
	
	// Load the textures
	if( strcmp( name, "Barbara") == 0 )
	{
		if( FAILED( rm->LoadTexture( "Barbara256.bmp", &pTex ) ) )
			DISPLAYMSG("Failed to load Texture in Player Icon\n");
	}
	else if( strcmp( name, "Curless") == 0 )
	{
		if( FAILED( rm->LoadTexture( "Curless256.bmp", &pTex ) ) )
			DISPLAYMSG("Failed to load Texture in Player Icon\n");
	}
	else
	{
		if (isDrEvil)
		{
			if(FAILED(rm->LoadTexture("DrEvil256.bmp", &pTex ) ) )
				DISPLAYMSG("Failed to load Texture in Player Icon\n");
		}
		else
		{
			if(FAILED(rm->LoadTexture("APowers256.bmp", &pTex ) ) )
				DISPLAYMSG("Failed to load Texture in Player Icon\n");
		}  
	}

	
	    rm->CreateWrap
			(D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
			D3DVAL(-fSize), D3DVAL(fSize/2.0), D3DVAL(-fSize/2.0),	// Wrap Origin
			D3DVAL(0.0), D3DVAL(0.0), D3DVAL(1.0),	// Wrap Direction Vector
			D3DVAL(0.0), D3DVAL(-1.0), D3DVAL(0.0),	// Wrap Up Vector
   			0.5, 0.5,		// Texture Origin
			D3DVAL(1.0/fSize), D3DVAL(1.0/fSize ),// Texture Scale Factor
			&lpWrap);

	
		lpWrap->Apply( (LPDIRECT3DRMOBJECT) lpFace );

		// Apply the texture to the wall
		if( FAILED( lpFace->SetTexture( pTex ) ) )
			DISPLAYMSG("Failed to Set Wall Texture Colors");
	

	if (FAILED(rm->CreateFrame(parent, frame)))
		goto error;

	
	if (FAILED(mesh->CreateMesh(&cubeMesh)))
		goto error;

	if (FAILED((*frame)->AddVisual(cubeMesh)))
	    goto error;

	if( FAILED( (*frame)->AddTranslation( D3DRMCOMBINE_REPLACE, 0.0f, 5.0f, 0.0f ) ) )
		goto error;

	RELEASE(mesh);
	RELEASE(cubeMesh);
	RELEASE(mat);
	return;

error:
	RELEASE(mesh);
	RELEASE(cubeMesh);
	RELEASE(mat);
	DISPLAYMSG("Error creating block!");
	return;
}
