#include "ClientProjectile.h"
#include "colors.h"
#include "RubicsObjects.h"
#include "RubicsDirect3D.h"
#include <d3dtypes.h>
#include <crtdbg.h>

CClientProjectile::CClientProjectile() {
	self_frame = NULL;
	parent_frame = NULL;
}

CClientProjectile::CClientProjectile(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, Position* pos,
									 SoundSet *pSound,float rotation )
{
	LPDIRECT3DRMMESHBUILDER3 meshBuilder;
	LPDIRECT3DRMMESH		 mesh;
	parent_frame = parent;
	parent->AddRef();
	modified = FALSE;
	pSoundSet = pSound;
	bPlayedSound = FALSE;
	lpdsb = NULL;
	lpdsb3d = NULL;

	_ASSERT( pSoundSet );

	//Create the frame
	self_frame = NULL;
	//Setup defaults for the info. This will make the guy
	//stand still at the origin and do nothing.
	info = new rubiks_info;

	info->head  = 0.0f;
	info->pos = *pos;
	//info->pos.y = 0.0f;
	//info->pos.z = 0.0f;
	info->framecount = 0.0f;
	info->anim = STANDING;

	// Create the Mesh Builder for the Wagon
	if (FAILED( rm->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	// Load the X file
	if( FAILED(GetSquare(meshBuilder, 1.0, RED,GREEN, ORANGE, YELLOW, WHITE, BLUE)))
	{
		DISPLAYMSG("Failed to create projectile.");
		goto error;
	}
	/*if( FAILED( meshBuilder->Load("missile.x",NULL,D3DRMLOAD_FROMFILE, NULL,NULL) ) )
	{
		DISPLAYMSG("Failed to load .X file");
		goto error;
	}
*/
	// Create Mesh to be used for the Wagon
	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create Projectile mesh");
		goto error;
	}
	// Create Mesh to be used for the Wagon
	if (FAILED(mesh->Translate(0,0.9f,0)))
	{
		DISPLAYMSG("Unable to translate Projectile mesh");
		goto error;
	}

	
	// Create frame that the outside world will see
	if (FAILED(rm->CreateFrame( parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create self frame on Projectile");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to Projectile frame" );
		goto error;
	}

	if( FAILED( self_frame->SetRotation(NULL, 1.0f, 0.0f,0.0f,rotation) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to Projectile frame" );
		goto error;
	}

	if (FAILED(self_frame->SetPosition(parent, pos->x, pos->y, pos->z)))
				DISPLAYMSG("Error setting Position on Projectile");

	/*if (FAILED(self_frame->SetOrientation(parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on Non-Movable Object");*/
	
error:
	RELEASE( meshBuilder );
	RELEASE( mesh );
}


CClientProjectile::~CClientProjectile()
{	
	if (lpdsb != NULL ) lpdsb->Stop(); // Stop the projectile from playing
	RELEASE( lpdsb );
	RELEASE( lpdsb3d );
	/* Frames */
	RELEASE(parent_frame);
	RELEASE(self_frame);
	lpdsb = NULL;
	/*Info*/
	if( info )
	  delete info;
}



void CClientProjectile::ReceiveMessage(rubiks_info *message)
{
	info->head  = message->head;
	info->pos.x = message->pos.x;
	info->pos.y = message->pos.y;
	info->pos.z = message->pos.z;
	info->framecount = message->framecount;
	info->anim = message->anim;
	//We should re-evaluate ourselves next time Update is called
	modified = TRUE;


}

/** Updates the drawing of the cube based on
  * the information it's received
  */
void CClientProjectile::Update()
{
	//unreferenced: DWORD dwStatus;
	//Only re-evaluate ourselves if a call to ReceiveMessage was called
	if (modified == FALSE) return;
	self_frame->AddRotation(D3DRMCOMBINE_BEFORE, 1.0f, 1.0f, 1.0f, 0.8f);
	 //Move the rubiks cube to the proper location and heading	
	self_frame->SetPosition(parent_frame, info->pos.x, info->pos.y, info->pos.z);
	
	if( lpdsb == NULL ) // If we haven't started playing the sound yet, go ahead and start it
	{
		pSoundSet->Play( PROJECTILE, info->pos, info->pos, &lpdsb, &lpdsb3d );
	}
	
	lpdsb3d->SetPosition( info->pos.x, info->pos.y, info->pos.z, DS3D_IMMEDIATE );

	modified = FALSE;
}


BOOL CClientProjectile::IsProjectile()
{
	return TRUE;
}
