#include "ClientRubiksPiece.h"
#include "RubicsObjects.h"
#include "RubicsDirect3D.h"
#include "SoundTypes.h"
#include "rodcone.h"

CClientRubiksPiece ::CClientRubiksPiece (LPDIRECT3DRM3 lpD3DRM, LPDIRECT3DRMFRAME3 Parent, 
										 Position *pos, float orientation, PieceType type, PieceType PlayerType, SoundSet *pSS )
{
	LPDIRECT3DRMMESHBUILDER3 meshBuilder;
	LPDIRECT3DRMMESH mesh;
	self_frame = NULL;
	D3DCOLOR color = NULL;
	size = 2.0f;
	pSoundSet = pSS;

	D3DVECTOR a,b;
	a.x = 0.25f;
	a.y = 0.25f;
	a.z = 0.0f;

	b.x = 0.25f;
	b.y = -0.25f;
	b.z = 0.0f;


	parent_frame = Parent;
	// Create the meshbuilder
	if (FAILED( lpD3DRM->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	/* Create alphabet cube with six faces
	Get3DRectangle( meshBuilder, 
					m_fBlockSize, m_fBlockSize, m_fBlockSize,			// Cube Dimensions 
					0.0, (float)(m_fBlockSize/2.0), 0.0,	// Center of cube in relation to meshBuilder
	
			m_Color, m_Color, WHITE, WHITE, m_Color, m_Color);  // Color of the cube*/
	pType = type;
	switch (type) {
		
		case RED_RUBIKS_PIECE:
			color = RED;
			break;
		case BLUE_RUBIKS_PIECE:
			color =  BLUE;
			break;
		case GREEN_RUBIKS_PIECE:
			color = GREEN;
			break;
		case YELLOW_RUBIKS_PIECE:
			color = YELLOW;
			break;
		case WHITE_RUBIKS_PIECE:
			color =  WHITE;
			break;
		case ORANGE_RUBIKS_PIECE:
			color =  ORANGE;
			break;
		default:
			color = BLACK;
	}	

	if(CONNECT_FOUR ==PlayerType) {
		AddRod(meshBuilder, 0.8f, a, b);
		Center(meshBuilder);
		meshBuilder->SetColor(color);
	} else {
		if( FAILED( GetSquare(meshBuilder, size, color))) {
			DISPLAYMSG("Failed to create square\n");
			goto error;
		}
	}

	
	
		//GetSquare( meshBuilder, m_fBlockSize );
	// Load the textures

	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;
	}
	
	if (FAILED(lpD3DRM->CreateFrame( Parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create frame in Alphabet Block");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to frame" );
		goto error;
	}

	/* We set color above now!
	if( FAILED( self_frame->SetColor(color ))) {
		DISPLAYMSG("Failed to set color of piece");
	}*/

	// shift Y value up by 1/2 size to account for our coordinate system
	if (FAILED(self_frame->SetPosition(Parent, pos->x, (pos->y + size/2.0f), pos->z)))
				DISPLAYMSG("Error setting Position on an Alphabet Block");

	if (FAILED(self_frame->SetOrientation(Parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on an Alphabet Block");
	
error:
	RELEASE( meshBuilder );
	RELEASE( mesh );	
}

void CClientRubiksPiece ::ReceiveMessage(rubiks_info* i) {
	memcpy(&info, i, sizeof(rubiks_info));
}
		
void CClientRubiksPiece::Update()
{
	//Move the rubiks cube to the proper location and heading	
	self_frame->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, info.head);
	self_frame->SetPosition(parent_frame, info.pos.x, (info.pos.y + size/2.0f) , info.pos.z);
}

void CClientRubiksPiece::RemoveSound() 
{
	if( pSoundSet != NULL )
	{
		pSoundSet->Play( BOUNCE, info.pos, info.pos );
	}
}
