#include "ClientViewer.h"
#include "colors.h"
#include "RubicsObjects.h"
#include "RubicsDirect3D.h"
#include <d3dtypes.h>

CClientViewer::CClientViewer() {
	self_frame = NULL;
	parent_frame = NULL;
}

CClientViewer::CClientViewer(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, Position* pos, float orientation, SoundSet *pSS )
{
	parent_frame = parent;
	parent->AddRef();
	modified = FALSE;
	multi_shot = FALSE;
	ricochete = FALSE;
	pSoundSet = pSS;

	//Initialize Overlay Display stuff for the camera guy!
	for (int i = 0; i < 6; i++)
		centerCube[i] = FALSE;
	name[0]='C'; name[1]='a'; name[2]='m'; name[3]='e'; name[4]='r'; name[5]='a'; name[6]='G';
	name[7]='u'; name[8]='y'; name[9]='\0';
	team = 3;  // no such team, so don't display in overlay.

	//Create the frame
	self_frame = NULL;

	info.head  = orientation;
	info.pos = *pos;
	info.framecount = 0.0f;
	info.anim = STANDING;

	// Initial orientation vectors for the 3D Listener
	vFront.x = cos( info.head );
	vFront.y = 0.0;
	vFront.z = sin( info.head );

	vTop.x = 0.0;
	vTop.y = 1.0;
	vTop.z = 0.0;
	
	// Create frame that the outside world will see
	if (FAILED(rm->CreateFrame( parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create self frame on Viewer");
		goto error;
	}

	if (FAILED(self_frame->SetPosition(parent, pos->x, pos->y, pos->z)))
				DISPLAYMSG("Error setting Position on Viewer");
	if (FAILED(self_frame->SetOrientation(parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on Viewer");

error:
	return;
}


CClientViewer::~CClientViewer()
{	
	/* Frames */
	RELEASE(parent_frame);
	RELEASE(self_frame);
}



void CClientViewer::ReceiveMessage(rubiks_info *message)
{
	info.head  = message->head;
	info.pos.x = message->pos.x;
	info.pos.y = message->pos.y;
	info.pos.z = message->pos.z;
	info.framecount = message->framecount;
	info.anim = message->anim;
	//We should re-evaluate ourselves next time Update is called
	modified = TRUE;


}

/** Updates the drawing of the cube based on
  * the information it's received
  */
void CClientViewer::Update()
{
	//Only re-evaluate ourselves if a call to ReceiveMessage was called
	if (modified == FALSE) return;
	self_frame->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, info.head);
	 //Move the rubiks cube to the proper location and heading	
	self_frame->SetPosition(parent_frame, info.pos.x, info.pos.y, info.pos.z);
	modified = FALSE;
}

void CClientViewer::DoSound()
{
	// If this rubiks cube is the hero, then update the listener's position
	if( bHero )
		UpdateListener();

}

BOOL CClientViewer::UpdateListener()
{
	// Update the orientation vector of the listener
	vFront.x = cos( info.head );
	vFront.z = sin( info.head );

	if(pSoundSet)
	{
		pSoundSet->SetListenerPosition( info.pos );
		pSoundSet->SetListenerOrientation( vFront, vTop );
	}

	return TRUE;
}
