#include "ConnectFour.h"
#include "Messages.h"
#include "RubicsDirect3D.h"
#include "RubicsMain.h"
#include "RubicsObjects.h"
#include "rodcone.h"

#define PI 3.14159f
#define NUM_ROWS 4
#define NUM_COLUMNS  4
#define BASE_WIDTH 4.0f
#define BASE_LENGTH 3.0f
#define ARM_HEIGHT 1.5f
#define ARM_WIDTH 0.2f


extern void ConnectFourBase(LPDIRECT3DRMMESHBUILDER3 meshbuilder, double width, float length,double depth, double height);
extern void ConnectArmSectionPair(LPDIRECT3DRMMESHBUILDER3 meshbuilder, float seperation, float width, float length, float height);
extern void ConnectFourBoard(LPDIRECT3DRMMESHBUILDER3 meshbuilder, double Depth, double FaceDepth, int NumRows, int NumColumns, double width, double height, double radius, int NumSidesInCircle);


ConnectFour::~ConnectFour()
{
	/* Blocks*/
	RELEASE(board);
	RELEASE(arm2);	
	RELEASE(arm1);	
	RELEASE(base);

	/* Animations*/
	RELEASE(base_hop);
	RELEASE(block_rotate);


	/* Frames */

	for (int i=0; i<4; i++) {
		for(int j = 0 ; j<4; j++) {
			if(CheckersFrames[i][j])
				RELEASE(CheckersFrames[i][j]);
		}
	}	

	RELEASE(lpPlayerIconFrame);
	RELEASE(board);
	RELEASE(base);
	RELEASE(arm1);
	RELEASE(arm2);
	RELEASE(parent_frame);
	RELEASE(self_frame);
	RELEASE(shadow);

	/*Info*/
	//delete info;
}



ConnectFour::ConnectFour(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, BOOL centerPieces[6],
						 char playername[], SoundSet *pSound, BOOL isHero )
{
	specularPower = 20.0;
	parent_frame = parent;
	parent->AddRef();
	modified = FALSE;
	multi_shot = FALSE;
	ricochete = FALSE;
	pSoundSet = pSound;
	NumCheckers = 0;
	CheckersInWinningRow = 0;
	lp3drm	= rm;
	currentWeapon = Ram;
	lpdsb = NULL;

	// set the type of the player
	pType = CONNECT_FOUR;

	// Set the hero status (are you the player or not)
	bHero = isHero;

	ChoosenRow = rand() % 4;
	if(rand() % 2){
		OpponentColor = RED;
		ChoosenColor = BLACK;
	} else  {
		OpponentColor = BLACK;
		ChoosenColor = RED;
	}


	for (int i = 0; i < 4; i++)
		Columns[i] = 0;

	for (i=0; i<10; i++)
		name[i] = playername[i];

	for (i=0; i<4; i++) {
		for(int j = 0 ; j<4; j++){
			CheckersFrames[i][j] = NULL;
			Checkers[i][j] = 0xffffffff;
		}
	}	
	
	


	const D3DCOLOR BOARD_COLOR = D3DRGB(1.0,1.0,0.0);
	const D3DCOLOR ARM_COLOR = D3DRGB(0.0,0.0,1.0);
	const D3DCOLOR BASE_COLOR = D3DRGB(0.0,0.0,1.0);

	LPDIRECT3DRMMESHBUILDER3 builder = NULL;
	LPDIRECT3DRMMESH mesh = NULL;
	LPDIRECT3DRMMATERIAL2 mat = NULL;
	LPDIRECT3DRMFRAME3 temp = NULL;

	//const float BASE_

	//Create the frame
	if (FAILED(rm->CreateFrame(parent, &self_frame)))
	{
		DISPLAYMSG("Unable to create self_frame in ConnectFour");
		return;
	}

	if (FAILED(rm->CreateFrame(self_frame, &base)))
	{
		DISPLAYMSG("Unable to create base in ConnectFour");
		return;
	}
	
	/*if (FAILED(base->AddRotation(D3DRMCOMBINE_REPLACE, 1.0f,0.0f,0.0f, 3.0 *PI/2.0f )))
		goto generic_error;

	if (FAILED(base->AddRotation(D3DRMCOMBINE_AFTER, 0.0f,1.0f,0.0f, PI/2.0f )))
		goto generic_error;*/
	//if (FAILED(base->AddRotation(D3DRMCOMBINE_AFTER, 0.0f,0.0f,1.0f, PI/2.0f )))
	//	goto generic_error;

	if (FAILED(rm->CreateMaterial(D3DVAL(10.0), &mat)))
	goto generic_error;

	if (FAILED(rm->CreateMeshBuilder(&builder)))
	{
		DISPLAYMSG("Unable to create builder in ConnectFour");
		return;
	}
	
	if (FAILED(rm->CreateFrame(base, &arm1)))
	{
		DISPLAYMSG("Unable to create arm1 in ConnectFour");
		return;
	}
	
	if (FAILED(rm->CreateFrame(arm1, &arm2)))
	{
		DISPLAYMSG("Unable to create arm2 in ConnectFour");
		return;
	}
	
	if (FAILED(rm->CreateFrame(arm2, &board)))
	{
		DISPLAYMSG("Unable to create board in ConnectFour");
		return;
	}

	/*********** Add the player bitmap  ***************/
	/* Don't make a bitmap if you're the player */
	if (!bHero) getPlayerIcon( rm, &lpPlayerIconFrame, self_frame, false); 


	ConnectFourBase(builder, BASE_WIDTH, BASE_LENGTH, 1, 1);
	CenterBase(builder);
	//builder->Save("ConnectBase.x", D3DRMXOF_BINARY, 0);
	if (FAILED(builder->SetColor(BASE_COLOR)))
	goto generic_error;
    if (FAILED(builder->SetMaterial(mat)))
	goto generic_error;
	
	if (FAILED(rm->CreateFrame(base, &temp)))
	{
		DISPLAYMSG("Unable to create base in ConnectFour");
		return;
	}
	
	if (FAILED(temp->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f,1.0f,0.0f, PI/2.0f )))
		goto generic_error;
//	if (FAILED(temp->AddRotation(D3DRMCOMBINE_AFTER, 1.0f,0.0f,0.0f, PI)))
//		goto generic_error;

	if (FAILED(builder->CreateMesh(&mesh)))
		goto generic_error;
	if (FAILED(temp->AddVisual(mesh)))
	{
		DISPLAYMSG("Unable to AddVisual in ConnectFour");
		return;
	}
	RELEASE(mesh);
	RELEASE(temp);
	if (FAILED(builder->Empty(0)))
		goto generic_error;

	ConnectArmSectionPair(builder, BASE_WIDTH + 2.0f * ARM_WIDTH, ARM_WIDTH, 0.4f, ARM_HEIGHT);
	CenterBase(builder);
	//builder->Save("Arm1.x", D3DRMXOF_BINARY, 0);
	if (FAILED(builder->SetColor(ARM_COLOR)))
	goto generic_error;
    if (FAILED(builder->SetMaterial(mat)))
	goto generic_error;

	if (FAILED(builder->CreateMesh(&mesh)))
		goto generic_error;

	if (FAILED(arm1->AddVisual(mesh)))
	{
		DISPLAYMSG("Unable to AddVisual in ConnectFour");
		return;
	}
	RELEASE(mesh);
	if (FAILED(builder->Empty(0)))
	goto generic_error;

	if (FAILED(arm1->SetPosition(base, 0.0f, 0.5f, 0.0f)))
		goto generic_error;
	

	ConnectArmSectionPair(builder, BASE_WIDTH + ARM_WIDTH, ARM_WIDTH/2, 0.3f, ARM_HEIGHT);
	CenterBase(builder);
	if (FAILED(builder->SetColor(ARM_COLOR)))
	goto generic_error;
    if (FAILED(builder->SetMaterial(mat)))
	goto generic_error;

	if (FAILED(builder->CreateMesh(&mesh)))
		goto generic_error;
	if (FAILED(arm2->AddVisual(mesh)))
	{
		DISPLAYMSG("Unable to AddVisual in ConnectFour");
		return;
	}
	RELEASE(mesh);
	if (FAILED(builder->Empty(0)))
	goto generic_error;
	
	if (FAILED(arm2->SetPosition(arm1, 0.0f, ARM_HEIGHT/2.0f, 0.0f)))
		goto generic_error;

	ConnectFourBoard(builder, 0.5, 0.1, NUM_ROWS, NUM_COLUMNS, BASE_WIDTH / NUM_COLUMNS / 2, BASE_WIDTH / NUM_COLUMNS / 2.0f, 0.375, 4);
	Center(builder);
	//builder->Save("ConnectBoard.x", D3DRMXOF_BINARY, 0);
	if (FAILED(builder->SetColor(BOARD_COLOR)))
	goto generic_error;
    if (FAILED(builder->SetMaterial(mat)))
	goto generic_error;
	if (FAILED(builder->CreateMesh(&mesh)))
		goto generic_error;
	
	if (FAILED(rm->CreateFrame(board, &temp)))
	{
		DISPLAYMSG("Unable to create base in ConnectFour");
		return;
	}
	
	if (FAILED(temp->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f,1.0f,0.0f, PI/2.0f )))
		goto generic_error;

	
	if (FAILED(temp->AddVisual(mesh)))
	{
		DISPLAYMSG("Unable to AddVisual in ConnectFour");
		return;
	}
	RELEASE(mesh);
	RELEASE(temp);
	
	if (FAILED(board->SetPosition(arm2, 0.0f, ARM_HEIGHT, 0.0f)))
		goto generic_error;

	if (FAILED(builder->Empty(0)))
	goto generic_error;
    //if (FAILED(builder->Save("ConnectBoard.x",D3DRMXOF_BINARY, 0 )))
	//goto generic_error;

    RELEASE(mat);
/*
	for(i = 0; i<6; i++) {
		CreateChecker(rm, board, &(Checkers[i]));
	}

*/	


	for (i = 0; i < 6; i++) {
		centerCube[i] = centerPieces[i];
		if(centerPieces[i]){
			AddChecker(GetCenterCube(i), i);
		}else
			BottomCheckers[i] = NULL;
	}


	createShadow(self_frame, rm); 

	//Setup defaults for the info. This will make the guy
	//stand still at the origin and do nothing.
	//info = new rubiks_info;

	info.head  = 0.0f;
	info.pos.x = 0.0f;
	info.pos.y = 0.0f;
	info.pos.z = 0.0f;
	info.framecount = 0.0f;
	info.anim = STANDING;
	
	/** Set up the animation interfaces */

	if (FAILED(rm->CreateAnimation(&block_rotate)) || 
		FAILED(rm->CreateAnimation(&cannon_begin)) || 
		FAILED(rm->CreateAnimation(&cannon)) || 
		FAILED(rm->CreateAnimation(&cannon_end)) || 
		FAILED(rm->CreateAnimation(&base_hop)) ||  
		FAILED(rm->CreateAnimation(&base_back_hop)) ||  
		FAILED(rm->CreateAnimation(&arm1_hop)) ||
		FAILED(rm->CreateAnimation(&arm2_hop)) ||
		FAILED(rm->CreateAnimation(&arm1_ram_begin)) ||
		FAILED(rm->CreateAnimation(&arm2_ram_begin)) ||
		FAILED(rm->CreateAnimation(&arm1_ram_end)) ||
		FAILED(rm->CreateAnimation(&arm2_ram_end)) ||
		FAILED(rm->CreateAnimation(&board_ram)) ||
		FAILED(rm->CreateAnimation(&board_hop)))   
	{
		DISPLAYMSG("Unable to create block_rotate in ConnectFour");
		return;
	}

	if (FAILED(block_rotate->SetOptions(D3DRMANIMATION_OPEN | D3DRMANIMATION_SPLINEPOSITION
								       | D3DRMANIMATION_POSITION 
									   | D3DRMANIMATION_SCALEANDROTATION)))
	{
			DISPLAYMSG("Unable to set animation options in ConnectFour");
		return;
	}
	//Copy the options
	cannon_begin->SetOptions(block_rotate->GetOptions());
	cannon->SetOptions(D3DRMANIMATION_OPEN | D3DRMANIMATION_SPLINEPOSITION
								       | D3DRMANIMATION_POSITION );
	cannon_end->SetOptions(block_rotate->GetOptions());
	base_hop->SetOptions(block_rotate->GetOptions());
	base_back_hop->SetOptions(block_rotate->GetOptions());
	arm2_hop->SetOptions(block_rotate->GetOptions());
	board_hop->SetOptions(block_rotate->GetOptions());
	arm1_ram_begin->SetOptions(block_rotate->GetOptions());
	arm2_ram_begin->SetOptions(block_rotate->GetOptions());
	arm1_ram_end->SetOptions(block_rotate->GetOptions());
	arm2_ram_end->SetOptions(block_rotate->GetOptions());
	board_ram->SetOptions(block_rotate->GetOptions());
	
	createHoppingAnimation();
	createBackwardHoppingAnimation();

	//Use these variables for the rotation stuff below
	D3DVECTOR v;
		v.x = 0.0f; 
		v.y = 0.0f; 
		v.z = 1.0f;

	// Orientation vectors for the 3D Listener
	vFront.x = cos( info.head );
	vFront.y = 0.0;
	vFront.z = sin( info.head );

	vTop.x = 0.0;
	vTop.y = 1.0;
	vTop.z = 0.0;

	D3DRMQUATERNION quat;

	///////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////
	//// Animation for firing
	///////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////

	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	cannon_begin->AddRotateKey(0.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, -PI/2.0f);
	cannon_begin->AddRotateKey(6.0f, &quat);

	cannon->AddPositionKey(0.0f,  0,ARM_HEIGHT, 0.0f);
	cannon->AddPositionKey(6.0f,  -ARM_HEIGHT,ARM_HEIGHT, 0.0f);
	cannon->AddPositionKey(12.0f,  0.0f,ARM_HEIGHT, 0.0f);

	D3DRMQuaternionFromRotation(&quat, &v, -PI/2.0f);
	cannon_end->AddRotateKey(0.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	cannon_end->AddRotateKey(6.0f, &quat);



	//Set the default position for the frames
	getStandingFrame();
generic_error:
	return;
}

void ConnectFour::CreateChecker(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 *checker, D3DCOLOR color) {
	
	LPDIRECT3DRMMESHBUILDER3 builder;
	LPDIRECT3DRMMESH mesh;
	HRESULT hr;
	hr = rm->CreateFrame(board, checker);
	assert(!FAILED(hr));

	hr = rm->CreateMeshBuilder(&builder);
	assert(!FAILED(hr));
	CreateChecker(builder, color);
	//GetSquare(builder, 1.0f, color);
	//GetCylinder(builder, 0.3 ,0.5f, BLUE);
	
	hr = builder->CreateMesh(&mesh);
	assert(!FAILED(hr));
	hr = (*checker)->AddVisual(mesh);
	assert(!FAILED(hr));
	
	//hr = board->DeleteChild((*checker));
	//assert(!FAILED(hr));

	RELEASE(mesh);
	RELEASE(builder);
}

void ConnectFour::CreateChecker(LPDIRECT3DRMMESHBUILDER3 builder, D3DCOLOR color) 
{
	HRESULT hr;
	D3DVECTOR a,b;

	a.x = 0.2f;
	a.y = 0.0f;
	a.z = 0.2f;

	b.x = 0.0f;
	b.y = 0.0f;
	b.z = 0.2f;

	AddRod(builder, 0.4f, a, b);
	Center(builder);
	hr = builder->SetColor(color);
	assert(!FAILED(hr));
}

///////////////////////////////////////////////////////////
////          Animation for forward hopping           /////
///////////////////////////////////////////////////////////
void ConnectFour::createBackwardHoppingAnimation()
{
	D3DVECTOR v;
		v.x = 0.0f; 
		v.y = 0.0f; 
		v.z = 1.0f;

	D3DRMQUATERNION quat;
	
	//Before we start moving
	base_back_hop->AddPositionKey(0.0f,  0.0f,           .0f,                0.0f);
	//Start jumping
	base_back_hop->AddPositionKey(3.0f,  0.0f,           .3f,                0.0f);
	//Start jumping
	//base_hop->AddPositionKey(4.0f,  .8f,           .5f,                0.0f);
	//Suspend in mid-air
	base_back_hop->AddPositionKey(9.0f,   0.0f,           .5f + .9f,    0.0f);
	base_back_hop->AddPositionKey(16.0f, .0f,             .5f + .9f,    0.0f);
	//Reverse the jump
	//base_hop->AddPositionKey(20.0f, .8f,       .5f,                0.0f);
	//Land and crunch
	base_back_hop->AddPositionKey(27.0f, .0f,            .3f,                0.0f);
    //Prepare for next frame
	base_back_hop->AddPositionKey(30.0f, .0f,            .0f,              0.0f);
	
	
	/**Rotations of each face */
	
	//Start a bit tucked forward
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_back_hop->AddRotateKey(0.0f, &quat);
    //Jump up and rotate to normal
	D3DRMQuaternionFromRotation(&quat, &v, 0.2f);
	base_back_hop->AddRotateKey(3.0f, &quat);
	//Lean back on jump a bit as we reach the crest
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_back_hop->AddRotateKey(9.0f, &quat);
    //Lean WAY back as we start coming down!
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_back_hop->AddRotateKey(24.0f, &quat);
    //SPLAT!! Hit the ground and level out
	D3DRMQuaternionFromRotation(&quat, &v, -0.2f);
	base_back_hop->AddRotateKey(27.0f, &quat);
    //Lean forward again
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_back_hop->AddRotateKey(30.0f, &quat);
	
//	}
}


///////////////////////////////////////////////////////////
////          Animation for forward hopping           /////
///////////////////////////////////////////////////////////
void ConnectFour::createHoppingAnimation()
{
	D3DVECTOR v;
		v.x = 0.0f; 
		v.y = 0.0f; 
		v.z = 1.0f;

	D3DRMQUATERNION quat;

	//X direction slide of each slice, along with Y pull-apart.
	//for (int i = 0; i < 3; i++)
//	{
	//Before we start moving
	base_hop->AddPositionKey(0.0f,  0.0f,           .0f,                0.0f);
	//Start jumping
	base_hop->AddPositionKey(3.0f,  0.0f,           .3f,                0.0f);
	//Start jumping
	//base_hop->AddPositionKey(4.0f,  .8f,           .5f,                0.0f);
	//Suspend in mid-air
	base_hop->AddPositionKey(9.0f,   0.0f,           .5f + .9f,    0.0f);
	base_hop->AddPositionKey(16.0f, .0f,             .5f + .9f,    0.0f);
	//Reverse the jump
	//base_hop->AddPositionKey(20.0f, .8f,       .5f,                0.0f);
	//Land and crunch
	base_hop->AddPositionKey(27.0f, .0f,            .3f,                0.0f);
    //Prepare for next frame
	base_hop->AddPositionKey(30.0f, .0f,            .0f,              0.0f);
	
	/**Rotations of each face */
	
	//Start a bit tucked forward
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_hop->AddRotateKey(0.0f, &quat);
    //Jump up and rotate to normal
	D3DRMQuaternionFromRotation(&quat, &v, -0.2f);
	base_hop->AddRotateKey(3.0f, &quat);
	//Lean back on jump a bit as we reach the crest
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_hop->AddRotateKey(9.0f, &quat);
    //Lean WAY back as we start coming down!
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_hop->AddRotateKey(24.0f, &quat);
    //SPLAT!! Hit the ground and level out
	D3DRMQuaternionFromRotation(&quat, &v, 0.2f);
	base_hop->AddRotateKey(27.0f, &quat);
    //Lean forward again
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	base_hop->AddRotateKey(30.0f, &quat);
	
	//Before we start moving
	arm2_hop->AddPositionKey(0.0f,  0.0f,           ARM_HEIGHT/2,    0.0f);
	//Start jumping
	arm2_hop->AddPositionKey(2.0f,  0.0f,           ARM_HEIGHT/4.0f,    0.0f);
	//Suspend in mid-air
	arm2_hop->AddPositionKey(10.0f,   0.0f,           3.0f *ARM_HEIGHT / 4.0f,    0.0f);
	arm2_hop->AddPositionKey(20.0f, .0f,             3.0f*ARM_HEIGHT / 4.0f,    0.0f);
	//Reverse the jump
    //Prepare for next frame
	arm2_hop->AddPositionKey(30.0f, .0f,            ARM_HEIGHT/2.0f,              0.0f);
	
	board_hop->AddPositionKey(0.0f,  0.0f,0.0f,    0.0f);
	board_hop->AddPositionKey(30.0f,  0.0f,0.0f,    0.0f);
//	}
}

void ConnectFour::createRamAnimations()
{
	D3DVECTOR v;
		v.x = 0.0f; 
		v.y = 0.0f; 
		v.z = 1.0f;

	D3DRMQUATERNION quat;

	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	arm1_ram_begin->AddRotateKey(0.0f, &quat);
    
	D3DRMQuaternionFromRotation(&quat, &v, PI/4.0f);
	arm1_ram_begin->AddRotateKey(12.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, PI/4.0f);
	arm1_ram_begin->AddRotateKey(1000.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, PI/4.0f);
	arm1_ram_end->AddRotateKey(0.0f, &quat);
    
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	arm1_ram_end->AddRotateKey(12.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	arm1_ram_begin->AddRotateKey(1000.0f, &quat);


	//Before we start moving
	arm2_ram_begin->AddPositionKey(0.0f,  0.0f,           ARM_HEIGHT/2.0f,    0.0f);
	//Start jumping
	arm2_ram_begin->AddPositionKey(12.0f,  0.0f,           ARM_HEIGHT,    0.0f);
	arm2_ram_begin->AddPositionKey(200.0f,  0.0f,           ARM_HEIGHT,    0.0f);

	//Before we start moving
	arm2_ram_end->AddPositionKey(0.0f,  0.0f,           ARM_HEIGHT,    0.0f);
	//Start jumping
	arm2_ram_end->AddPositionKey(12.0f,  0.0f,           ARM_HEIGHT/2.0f,    0.0f);

	arm2_ram_end->AddPositionKey(200.0f,  0.0f,           ARM_HEIGHT/2.0f,    0.0f);

	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	board_ram->AddRotateKey(0.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, 3.14f);
	board_ram->AddRotateKey(24.0f, &quat);

	D3DRMQuaternionFromRotation(&quat, &v, 6.28f);
	board_ram->AddRotateKey(48.0f, &quat);
}








void ConnectFour::ReceiveMessage(rubiks_info *message)
{
	memcpy(&info, message, sizeof(rubiks_info));
	/*
	info.head  = message->head;
	info.pos.x = message->pos.x;
	info.pos.y = message->pos.y;
	info.pos.z = message->pos.z;
	info.framecount = message->framecount;
	info.anim = message->anim;*/
	//We should re-evaluate ourselves next time Update is called
	modified = TRUE;


}

/** Updates the drawing of the cube based on
  * the information it's received
  */
void ConnectFour::Update()
{
		//Only re-evaluate ourselves if a call to ReceiveMessage was made.
	if (modified == FALSE) return;

	switch (info.anim)
	{
	case STANDING:
		getStandingFrame();
		break;

	case HOPPING:
		getHoppingFrame();
		break;

	case BACKWARDS:
		getBackwardHoppingFrame();
		break;
	case BEGINFIRING:
		getBeginFiringFrame();
		break;
	case FIRING:
		getFiringFrame();
		break;
	case ENDFIRING:
		getEndFiringFrame();
		break;

	case BEGINACTION2:
		switch (currentWeapon)
		{
			case Tornado:
				getBeginFiringFrame();
				//getRamFrame(0);
				break;
			case Ram:
			case Stomp:
				getRamFrame(0);
				//getBeginStompFrame();
				break;
			case ArcingMortar:
				getBeginFiringFrame();
				break;
		}
		break;

	case ACTION2:
		switch (currentWeapon)
		{
			case Tornado:
				getFiringFrame();
				//getRamFrame(1);
				break;
			case Ram:
			case Stomp:
				getRamFrame(1);
				//getStompFrame();
				break;
			case ArcingMortar:
				getFiringFrame();
				break;
		}
		break;

	case ENDACTION2:
		switch (currentWeapon)
		{
			case Tornado:
				getEndFiringFrame();
				break;
			case Ram:
			case Stomp:
				getRamFrame(2);
				//getEndStompFrame();
				break;
			case ArcingMortar:
				getEndFiringFrame();
				break;
		}
		break;

	case COLLIDEWITHPROJECTILE:
	case COLLIDEWITHPLAYER:
		getHitByProjectileFrame();
		break;
	default:
		getStandingFrame();
	}

	
	/* Global map position and location */
	self_frame->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, info.head);
	self_frame->SetPosition(parent_frame, info.pos.x, info.pos.y, info.pos.z);

	/** Position of player bitmap **/
	if (!bHero)
	{
		lpPlayerIconFrame->LookAt( GetCamera(), self_frame, D3DRMCONSTRAIN_X );
		lpPlayerIconFrame->LookAt( GetCamera(), self_frame, D3DRMCONSTRAIN_Z );
	}

	/*Shadow position. Simulate it staying in place by moving it down at the
	  same rate the cube goes up */
	D3DVALUE y = info.pos.y;
	shadow->SetPosition(self_frame, 0, -y, 0);
	//shadow->AddScale(D3DRMCOMBINE_BEFORE, 1.0f - 0.6f*(y/50.0f), 1.0f, 1.0f - 0.6f*(y/50.0f));
	
	modified = FALSE;
}

BOOL ConnectFour::getBeginFiringFrame()
{
	HRESULT hr;
	hr = base->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0);
	assert(SUCCEEDED(hr));
	hr = base->SetPosition(self_frame, 0.0f, 0.0f, 0.0f);
	assert(SUCCEEDED(hr));
	hr = board->SetPosition(arm2, 0.0f,ARM_HEIGHT, 0.0f);
	assert(SUCCEEDED(hr));
	hr = cannon_begin->SetFrame(board);
	assert(SUCCEEDED(hr));
	hr = cannon_begin->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	//return getFiringFrame();
	return TRUE;
}

BOOL ConnectFour::getEndFiringFrame()
{
	HRESULT hr;
	hr = board->SetPosition(arm2, 0.0f,ARM_HEIGHT, 0.0f);
	assert(SUCCEEDED(hr));
	hr = cannon_end->SetFrame(board);
	assert(SUCCEEDED(hr));
	hr = cannon_end->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	
	return TRUE;
	//return getFiringFrame();
}

BOOL ConnectFour::getBeginStompFrame()
{
return getStandingFrame();
}

BOOL ConnectFour::getStompFrame()
{
return getStandingFrame();
}

BOOL ConnectFour::getEndStompFrame()
{
return getStandingFrame();
}

//type corresponds to BEGINTORNADO, TORNADO, ENDTORNADO
BOOL ConnectFour::getRamFrame(int type)
{
	HRESULT hr;
	float spin;

	getStandingFrame();

	if (type == 0)  
		spin = (info.framecount/MAX_BEGINTORNADO) * 6.28;
	else if (type == 1) 
	{
		spin = 3.0f * (info.framecount/MAX_TORNADO) * 6.28;
		while (spin > 6.28) spin -= 6.28f;
	}
	else 
		spin = (info.framecount/MAX_ENDTORNADO) * 6.28;
	

	hr = board->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 1.0f, spin);
	assert(!FAILED(hr));
	hr = board->SetPosition(arm2, 0.0f, ARM_HEIGHT, 0.0f);
	assert(!FAILED(hr));

	hr = arm1->SetPosition(base, 0.0f, 1.0f, 0.0f);
	assert(!FAILED(hr));

	/*hr = board_ram->SetFrame(board);
	assert(SUCCEEDED(hr));
	
	hr = board_ram->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	*/
	

	if (type == 0) { 
		hr = arm1->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 1.0f, -(2*PI - (7.5f *PI /4.0f))*info.framecount/(float)MAX_BEGINTORNADO);
		assert(!FAILED(hr));

		hr = arm2->SetPosition(arm1, 0.0f, ARM_HEIGHT/2 + info.framecount/(float)MAX_BEGINTORNADO * ARM_HEIGHT, 0.0f);
		assert(SUCCEEDED(hr));
	} else if (type == 1) 
	{
		hr = arm1->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 1.0f, 7.5f *PI /4.0f);
		assert(!FAILED(hr));

		hr = arm2->SetPosition(arm1, 0.0f, ARM_HEIGHT, 0.0f);
		assert(SUCCEEDED(hr));

	
	} else {
		hr = arm1->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 1.0f, -(2*PI - (7.5f *PI /4.0f))*(1.0f-info.framecount/(float)MAX_ENDTORNADO));
		assert(!FAILED(hr));
		hr = arm2->SetPosition(arm1, 0.0f, ARM_HEIGHT - info.framecount/(float)MAX_BEGINTORNADO * ARM_HEIGHT, 0.0f);
		assert(SUCCEEDED(hr));
		/*hr = arm1_ram_end->SetFrame(arm1);
		assert(SUCCEEDED(hr));
		hr = arm2_ram_end->SetFrame(arm2);
		assert(SUCCEEDED(hr));
		
		hr = arm1_ram_end->SetTime(D3DVALUE(info.framecount));
		assert(SUCCEEDED(hr));
		hr = arm2_ram_end->SetTime(D3DVALUE(info.framecount));
		assert(SUCCEEDED(hr));*/

	}			
			 
	return TRUE;
}

BOOL ConnectFour::getBackwardHoppingFrame()
{
	HRESULT hr;
	board->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0);
	hr = base_back_hop->SetFrame(base);
	assert(SUCCEEDED(hr));
	hr = arm2_hop->SetFrame(arm2);
	assert(SUCCEEDED(hr));
	
	hr = base_back_hop->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	hr = arm2_hop->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	
	hr = board->SetPosition(arm2, 0.0f, ARM_HEIGHT, 0.0f);
	assert(SUCCEEDED(hr));
			//Moves block to its place in the cube
			//assert(SUCCEEDED(block->AddTranslation(D3DRMCOMBINE_BEFORE, -1.55f + 1.05f*x, 1.05f*y,
																						  //-1.55f + 1.05f * z)));
		  //}


	// SOUND STUFF

/*	D3DVECTOR vOrigin;
	vOrigin.x = 0.0;
	vOrigin.y = 0.0;
	vOrigin.z = 0.0;
*/
	// Play bounce sound when cube begins to jump
	/*if( info.framecount == 3 ) 
	{
		if( pSoundSet )	// Must check to see if pSoundSet has been passed in
			pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  
		UpdateListener();	// We don't need to update the listener every single time
	}*/
	return TRUE;
}




BOOL ConnectFour::getHoppingFrame()
{
	HRESULT hr;
	board->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0);
	hr = base_hop->SetFrame(base);
	assert(SUCCEEDED(hr));
	hr = arm2_hop->SetFrame(arm2);
	assert(SUCCEEDED(hr));
	
	hr = base_hop->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	hr = arm2_hop->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	
	hr = board->SetPosition(arm2, 0.0f, ARM_HEIGHT, 0.0f);
	assert(SUCCEEDED(hr));
			//Moves block to its place in the cube
			//assert(SUCCEEDED(block->AddTranslation(D3DRMCOMBINE_BEFORE, -1.55f + 1.05f*x, 1.05f*y,
																						  //-1.55f + 1.05f * z)));
		  //}


	// SOUND STUFF

/*	D3DVECTOR vOrigin;
	vOrigin.x = 0.0;
	vOrigin.y = 0.0;
	vOrigin.z = 0.0;
*/
	// Play bounce sound when cube begins to jump
	/*if( info.framecount == 3 ) 
	{
		if( pSoundSet )	// Must check to see if pSoundSet has been passed in
			pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  
		UpdateListener();	// We don't need to update the listener every single time
	}*/
	return TRUE;
}

BOOL ConnectFour::getFiringFrame()
{
	HRESULT hr;
	hr = board->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f,0.1f, 3.0f * PI/2.0f);
	assert(SUCCEEDED(hr));
	hr = cannon->SetFrame(board);
	assert(SUCCEEDED(hr));
	hr = cannon->SetTime(D3DVALUE(info.framecount));
	assert(SUCCEEDED(hr));
	
	if( info.framecount == 3 )
	{
		pSoundSet->Play( CANNON, info.pos, info.pos );
	}
	return TRUE;
}


BOOL ConnectFour::getHitByProjectileFrame()
{
	HRESULT hr;
	float spin;

	getStandingFrame();

	spin = 3.0f * (info.framecount/MAX_COLLIDEWITHPROJECTILE) * 6.28;
	
	hr = board->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 1.0f, spin);
	assert(!FAILED(hr));
	hr = board->SetPosition(arm2, 0.0f, (8*ARM_HEIGHT) *(info.framecount/(MAX_COLLIDEWITHPROJECTILE*2.0f) )+ARM_HEIGHT, 0.0f);
	assert(!FAILED(hr));

	return TRUE;
}


BOOL ConnectFour::getStandingFrame()
{	HRESULT hr;

	hr = base->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0);
	assert(SUCCEEDED(hr));
	hr = base->SetPosition(self_frame, 0.0f, 0.0f, 0.0f);
	assert(SUCCEEDED(hr));
	hr = board->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0);
	assert(SUCCEEDED(hr));
	hr = board->SetPosition(arm2, 0.0f, ARM_HEIGHT, 0.0f);
	assert(SUCCEEDED(hr));
	hr = arm1->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, 0);
	assert(SUCCEEDED(hr));

	hr = arm2->SetPosition(arm1, 0.0f, ARM_HEIGHT/2.0f, 0.0f);
	assert(SUCCEEDED(hr));

	hr = arm1->SetPosition(base, 0.0f, 0.5f, 0.0f);
	assert(!FAILED(hr));

/*	LPDIRECT3DRMFRAME3 block;

	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			//Get the current block. 
			block = blocks[x + 3*y + 9*z];

			//Moves the block to its place in the cube. The center block is centered around the origin.
			block->AddTranslation(D3DRMCOMBINE_REPLACE, -1.55f + 1.05f*x, 1.05f*y, -1.55f + 1.05f*z);		
		  }
			
	RELEASE(block);*/
	return TRUE;
}

void ConnectFour::AddChecker(int x, int y, D3DCOLOR color) {
	LPDIRECT3DRMFRAME3 checker;
	HRESULT hr;
	
	CreateChecker(lp3drm, &(CheckersFrames[x][y]),color);
	checker = CheckersFrames[x][y];
	hr = checker->SetPosition(board, 0,((BASE_WIDTH/NUM_ROWS)/2.0f )*(1+2*y)-(BASE_WIDTH/2.0f), ((BASE_WIDTH/NUM_COLUMNS)/2.0f )*(1+2*x)-(BASE_WIDTH/2.0f));
	assert(!FAILED(hr));
}

void ConnectFour::AddCheckerInBottom(int index, D3DCOLOR color) {
	LPDIRECT3DRMFRAME3 checker;	
	HRESULT hr;
	LPDIRECT3DRMMESHBUILDER3 builder;
	LPDIRECT3DRMMESH mesh;

	hr = lp3drm->CreateFrame(base, &checker);
	assert(!FAILED(hr));

	hr = lp3drm	->CreateMeshBuilder(&builder);
	assert(!FAILED(hr));
	CreateChecker(builder, color);
	//GetSquare(builder, 1.0f, color);
	//GetCylinder(builder, 0.3 ,0.5f, BLUE);
	
	hr = builder->CreateMesh(&mesh);
	assert(!FAILED(hr));
	hr = checker->AddVisual(mesh);
	assert(!FAILED(hr));
	
	//hr = board->DeleteChild((*checker));
	//assert(!FAILED(hr));

	RELEASE(mesh);
	RELEASE(builder);

	hr = checker->AddRotation(D3DRMCOMBINE_REPLACE, 1.0f, 0.0f, 0.0f, PI/2.0f);
	assert(!FAILED(hr));
	hr = checker->SetPosition(base, 0, 0.2f, ((BASE_WIDTH/6.0f)/2.0f )*(1+2*index)-(BASE_WIDTH/2.0f));
	assert(!FAILED(hr));
	
	BottomCheckers[index] = checker;
}

void ConnectFour::PlaceCheckers(int column, D3DCOLOR color) {
	int y;
	y = Columns[column];
	AddChecker(column, y, color);
	Columns[column]++;
	pieceStack.push(column);
}

int ConnectFour::RandomRow(int BadRow, D3DCOLOR color) {
	int temp;
	int count = 0;
	do {
		temp = rand() % 4;
		count++;
	} while (Columns[temp] > 3 && temp == BadRow && !CanWin(temp, color) && count < 20);
	count = 0;
	while((temp == BadRow  || Columns[temp] > 3) && count <= 4) {
		temp = (temp+1) % 4;
		count++;
	}
	return temp;
}

bool ConnectFour::CanWin(int column,D3DCOLOR color) {
	if(Checkers[column][3] == 0)
		return false;
	for(int i = 3; i >=0; i--) {
		if(Checkers[column][i] != color)
			return false;
	}
	return true;
}

void ConnectFour::AddChecker(PieceType piece,int index) {		
	AddCheckerInBottom(index, CheckerColor(piece));

	switch((6 - NumCheckers) -  (4 - CheckersInWinningRow)) {
		case 0:
			PlaceCheckers(ChoosenRow, ChoosenColor);
			CheckersInWinningRow++;
			PlaceCheckers(RandomRow(ChoosenRow, OpponentColor), OpponentColor);
		break;
		case 1:
		if(rand()%2) {
			PlaceCheckers(ChoosenRow, ChoosenColor);
			CheckersInWinningRow++;
			PlaceCheckers(RandomRow(ChoosenRow, OpponentColor), OpponentColor);
		} else {
			PlaceCheckers(RandomRow(ChoosenRow, ChoosenColor), ChoosenColor);
			PlaceCheckers(RandomRow(ChoosenRow, OpponentColor), OpponentColor);
		}
			break;
		case 2:
		if(rand()%3) {
			PlaceCheckers(RandomRow(ChoosenRow, ChoosenColor), ChoosenColor);
			PlaceCheckers(RandomRow(ChoosenRow, OpponentColor), OpponentColor);
		} else {
			PlaceCheckers(ChoosenRow, ChoosenColor);
			CheckersInWinningRow++;
			PlaceCheckers(RandomRow(ChoosenRow, OpponentColor), OpponentColor);
		}
		break;
	}
	
	NumCheckers++;
}

D3DCOLOR  ConnectFour::CheckerColor(PieceType piece) {
	D3DCOLOR color;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		color = RED;
		break;
	case BLUE_RUBIKS_PIECE:
		color = BLUE;
		break;
	case GREEN_RUBIKS_PIECE:
		color = GREEN;
		break;
	case YELLOW_RUBIKS_PIECE:
		color = YELLOW;
		break;
	case WHITE_RUBIKS_PIECE:
		color = WHITE;
		break;
	case ORANGE_RUBIKS_PIECE:
		color = ORANGE;
		break;
	default:
		color = BLACK;
		break;
	}
	return color;
}
void ConnectFour::PickupPiece(PieceType piece) 
{
	int index;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		centerCube[RED_SQUARE] = true;
		index = RED_SQUARE;
		DoSound( PICKUP_PIECE );
		break;
	case BLUE_RUBIKS_PIECE:
		centerCube[BLUE_SQUARE] = true;
		index = BLUE_SQUARE;
		DoSound( PICKUP_PIECE );
		break;
	case GREEN_RUBIKS_PIECE:
		centerCube[GREEN_SQUARE] = true;
		index = GREEN_SQUARE;
		DoSound( PICKUP_PIECE );
		break;
	case YELLOW_RUBIKS_PIECE:
		centerCube[YELLOW_SQUARE] = true;
		index = YELLOW_SQUARE;
		DoSound( PICKUP_PIECE );
		break;
	case WHITE_RUBIKS_PIECE:
		centerCube[WHITE_SQUARE] = true;
		index = WHITE_SQUARE;
		DoSound( PICKUP_PIECE );
		break;
	case ORANGE_RUBIKS_PIECE:
		centerCube[ORANGE_SQUARE] = true;
		index = ORANGE_SQUARE;
		DoSound( PICKUP_PIECE );
		break;
	case MISSILE:
		currentWeapon = Tornado;
		DoSound( PICKUP_ICON );
		return;
		break;
	case RAM_OBJECT:
		currentWeapon = Ram;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case TORNADO_OBJECT:
		currentWeapon = Tornado;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case ARCING_OBJECT:
		currentWeapon = ArcingMortar;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case MULTI_SHOT:
		multi_shot = true;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case RICOCHETE:
		ricochete = true;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	default:
		// This wasn't a Rubiks piece so don't do nuthin!
		return;
		break;
	}
	
	//Add the frame to the cube for the block we just picked up
	AddChecker(piece, index);
}


BOOL ConnectFour::UpdateListener()
{
	// Update the orientation vector of the listener
	vFront.x = cos( info.head );
	vFront.z = sin( info.head );

	if(pSoundSet)
	{
		pSoundSet->SetListenerPosition( info.pos );
		pSoundSet->SetListenerOrientation( vFront, vTop );
	}
	return TRUE;
}

void ConnectFour::createShadow(LPDIRECT3DRMFRAME3 parent, LPDIRECT3DRM3 rm)
{
	LPDIRECT3DRMMESHBUILDER3 meshb;
	LPDIRECT3DRMMESH mesh;
	LPDIRECT3DRMFACE2 face;
	//Create the mesh builder
	if (FAILED (rm->CreateMeshBuilder(&meshb)))
	{
		DISPLAYMSG("Unable to create mesh_builder");
		goto error;	
	}

	//create the mesh builder
	if (FAILED (meshb->CreateFace(&face)))
	{
		DISPLAYMSG("Unable to create face");
		goto error;	
	}

	if (FAILED (face->SetColor(D3DRGBA(0.0f, 0.0f, 0.0f, 127.0f))))
		goto error;	

	if (FAILED (face->AddVertex(-1.075f, 0.001f, -1.075f)))
		goto error;	
	if (FAILED (face->AddVertex(-1.075f, 0.001f, 1.075f)))
		goto error;	
	if (FAILED (face->AddVertex(1.075f,  0.001f, 1.075f)))
		goto error;	
	if (FAILED (face->AddVertex(1.075f,  0.001f, -1.075f)))
		goto error;	

	
	if (FAILED(meshb->CreateMesh(&mesh)))
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;	
	}

	if (FAILED(rm->CreateFrame(parent, &shadow)))
	{
		DISPLAYMSG("Unable to create shadow frame");
		goto error;	
	}

	if (FAILED(shadow->AddVisual(mesh)))
	{
		DISPLAYMSG("Unable to add mesh to shadow frame");
		goto error;	
	}

	RELEASE(meshb);
	RELEASE(mesh);
	RELEASE(face);
	return;

error:
	DISPLAYMSG("Error creating shadow!");
	RELEASE(meshb);
	RELEASE(mesh);
	RELEASE(face);
}

void ConnectFour::DoSound( SOUND_STATE eState )
{
	// If the connect 4 board is the hero, then update the listener's position
	if( bHero )
		UpdateListener();
	
	switch( eState )
	{
	case ANIM:	// We have a sound associated with an enum

		switch( info.anim )
		{
		case HOPPING:
			// Play bounce sound when cube begins to jump
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now
			}
			break;

		case BACKWARDS:
			// Play bounce sound when cube begins to jump
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now
			}
			break;

		case FIRING:
			// Play firing sound when cube begins to fire
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( BANG, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BANG, info.pos, info.pos ); // Orientation just set to pos for now
			}
			break;

		case ACTION2:
			// Play tornado sound when cube begins to tornado attack
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				switch( GetCurrentWeapon() )
				{
				  case Tornado:
					if( bHero )
						pSoundSet->Play( HERO_CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
					else
						pSoundSet->Play( CANNON, info.pos, info.pos ); // Orientation just set to pos for now
					break;

				case ArcingMortar:
					if( bHero )
						pSoundSet->Play( HERO_CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
					else
						pSoundSet->Play( CANNON, info.pos, info.pos ); // Orientation just set to pos for now
					break;

				case Ram:
					if( lpdsb == NULL )
					{
						if( bHero )
							pSoundSet->Play( HERO_SCREAM, info.pos, info.pos, &lpdsb ); // Orientation just set to pos for now  	
						else
							pSoundSet->Play( SCREAM, info.pos, info.pos, &lpdsb ); // Orientation just set to pos for now
					}
					break;
				}			
			}
			break;

		case ENDACTION2:
			if( info.framecount == ANIMATION_STEP_SIZE )
			{
			  if( GetCurrentWeapon() == Ram )
			  {
				if( lpdsb )
				{
				  lpdsb->Stop();
				  RELEASE( lpdsb );

				  lpdsb = NULL;
				}
			  }

			}
			
			break;


		case COLLIDEWITHPROJECTILE:
			// Play collision with projectile sound
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
			}
			break;
		
		case COLLIDEWITHPLAYER:
			// Play collision sound when cube collides with other player
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_SBOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
			}
			break;

		default:
			break;
		}
		break;

	// The rest of the sounds are not animation sounds...
	case PICKUP_ICON:
		if( bHero )
			pSoundSet->Play( HERO_GREATBABY, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( GREATBABY, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	case DROP_ICON:
		if( bHero )
			pSoundSet->Play( HERO_LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	case PICKUP_PIECE:
		if( bHero )
			pSoundSet->Play( HERO_WIN, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( WIN, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	case DROP_PIECE:
		if( bHero )
			pSoundSet->Play( HERO_LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	default:
		break;
	}


	return;
}


void ConnectFour::DropPiece(PieceType piece) 
{
	HRESULT hr;
	int index;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		centerCube[RED_SQUARE] = false;
		index = RED_SQUARE;
		DoSound( DROP_PIECE );
		break;
	case BLUE_RUBIKS_PIECE:
		centerCube[BLUE_SQUARE] = false;
		index = BLUE_SQUARE;
		DoSound( DROP_PIECE );
		break;
	case GREEN_RUBIKS_PIECE:
		centerCube[GREEN_SQUARE] = false;
		index = GREEN_SQUARE;
		DoSound( DROP_PIECE );
		break;
	case YELLOW_RUBIKS_PIECE:
		centerCube[YELLOW_SQUARE] = false;
		index = YELLOW_SQUARE;
		DoSound( DROP_PIECE );
		break;
	case WHITE_RUBIKS_PIECE:
		centerCube[WHITE_SQUARE] = false;
		index = WHITE_SQUARE;
		DoSound( DROP_PIECE );
		break;
	case ORANGE_RUBIKS_PIECE:
		centerCube[ORANGE_SQUARE] = false;
		index = ORANGE_SQUARE;
		DoSound( DROP_PIECE );
		break;
	default:
		// This wasn't a Rubiks piece so don't do nuthin!
		return;
		break;
	}
	
	// Drop a power-up also if you lost a piece
	if(multi_shot)
		multi_shot = FALSE;
	else if(ricochete)
		ricochete = FALSE;

	//Remove the frame from the cube 
	
	int temp;
	LPDIRECT3DRMFRAME3 frame = NULL;
	if(BottomCheckers[index]) {
		hr = base->DeleteChild(BottomCheckers[index]);
		if(!pieceStack.empty()) {
			temp = pieceStack.top();
			//PlaceCheckers(temp, ChoosenColor);
			if(Columns[temp]) {
				frame = CheckersFrames[temp][Columns[temp]-1];
				if(frame) {
				  hr = board->DeleteChild(frame);
				  RELEASE(frame);
				}
				CheckersFrames[temp][Columns[temp]-1] = NULL;
				Checkers[temp][Columns[temp]-1] = 0xffffffff;
				Columns[temp]--;
			}
			pieceStack.pop();
		}

		if(!pieceStack.empty()) {
			temp = pieceStack.top();
			//PlaceCheckers(temp, ChoosenColor);
			if(Columns[temp]) {
				frame = CheckersFrames[temp][Columns[temp]-1];
				if(frame) {
				  hr = board->DeleteChild(frame);
				  RELEASE(frame);
				}
				CheckersFrames[temp][Columns[temp]-1] = NULL;
				Checkers[temp][Columns[temp]-1] = 0xffffffff;
				Columns[temp]--;
			}
			pieceStack.pop();
		}

		BottomCheckers[index]= NULL;
	}
}
