/**
  *
  * RubiksCube.h  -- Defines a class that draws a Rubiks Cube in
  *                  Direct3D.
  *
  * Author: Kevin Audleman
  *
  */

#ifndef  __CONNECT_FOUR__
#define  __CONNECT_FOUR__


#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <ddraw.h>

#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <stack>

#include "rubiksInfo.h"
#include "colors.h"
#include "ClientPlayer.h"
#include "RubiksSoundSet.h"
#include "SoundTypes.h"
#include "dsound.h"	// Added by ClassView

#ifndef RELEASE
#define RELEASE(x) if (x != NULL) { x->Release(); x = NULL; } 
#endif

// Macro to display a message box containing the given string. 
#ifndef DISPLAYMSG
//#define DISPLAYMSG(x) MessageBox(NULL, x, "Rubiks ICON", MB_OK);
#define DISPLAYMSG(x) OutputDebugString(x)
#endif

typedef std::stack<int> ConnectPieces;

class ConnectFour: public CClientPlayer {

public:
	virtual void DoSound( SOUND_STATE eState );
	virtual BOOL UpdateListener();
	ConnectFour(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, BOOL centerPieces[6], 
		        char playername[], SoundSet *pSound = NULL, BOOL isHero=FALSE);
	virtual ~ConnectFour();

	//Sets our information based on the message we receive
	virtual void ReceiveMessage(rubiks_info *message);

	//Call this function every time the Player app wants
	//to refresh this cube. 
	virtual void Update();
	
	// return if we're a player or not.

	virtual void PickupPiece(PieceType piece);
	virtual void DropPiece(PieceType piece);
	void CreateChecker(LPDIRECT3DRMMESHBUILDER3 builder, D3DCOLOR color) ;

protected:
	//All of the information for this block. This is the message that
	//will be passed in between the server and player.


protected:
	LPDIRECTSOUNDBUFFER lpdsb;
	D3DVECTOR vFront;
	D3DVECTOR vTop;
	SoundSet * pSoundSet;
	//Movement functions. Now that we're passing messages, these
	//might want to be private.
	BOOL getHoppingFrame();
	BOOL getStandingFrame();
	BOOL getFiringFrame();
	BOOL getHitByProjectileFrame();

	//Builds the hopping animation ANIMATION nodes
	void createHoppingAnimation();
	void createBackwardHoppingAnimation();
	void createRamAnimations();
	void AddChecker();
	BOOL getBeginStompFrame();
	BOOL getStompFrame();
	BOOL getEndStompFrame();
	BOOL getRamFrame(int type);
	BOOL getBeginFiringFrame();
	BOOL getEndFiringFrame();
	BOOL getBackwardHoppingFrame();
	void createShadow(LPDIRECT3DRMFRAME3 parent, LPDIRECT3DRM3 rm);

	LPDIRECT3DRMFRAME3 checkers[6];
	//Frames
	LPDIRECT3DRMFRAME3 parent_frame;
	LPDIRECT3DRMFRAME3 arm1, arm2, base, board, shadow;

	LPDIRECT3DRMANIMATION2 block_rotate, base_hop, arm1_hop,arm2_hop,board_hop, base_back_hop,
		board_ram, arm1_ram_end, arm2_ram_end, arm1_ram_begin, arm2_ram_begin;  //Hopping

	LPDIRECT3DRMANIMATION2 cannon_begin, cannon, cannon_end;
	D3DCOLOR  CheckerColor(PieceType piece);
	void AddChecker(PieceType piece, int index);
	void AddChecker(int x, int y, D3DCOLOR color);
	bool CanWin(int column,D3DCOLOR color);
	int RandomRow(int BadRow, D3DCOLOR color);
	void PlaceCheckers(int column, D3DCOLOR color);
	void AddCheckerInBottom(int index, D3DCOLOR color);
	void CreateChecker(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 * checker, D3DCOLOR color);

	LPDIRECT3DRMFRAME3 BottomCheckers[6];
	LPDIRECT3DRMFRAME3 CheckersFrames[4][4], lpPlayerIconFrame;

	D3DCOLOR Checkers[4][4];

	int NumCheckers;
	int Columns[4];
	int CheckersInWinningRow;
	D3DCOLOR OpponentColor;
	D3DCOLOR ChoosenColor;
	/* Other variables */

	//Default specular power will be set in the constructor. If we want to 
	//changed it during gameplay (which I doubt we will), i'll add some access functions.
	D3DVALUE	specularPower;		

	BOOL modified;  //Set when ReceiveMessage is called
	//Moved this to CCLientPlayer: char name[10];
	LPDIRECT3DRM3 lp3drm;
	int ChoosenRow;
	ConnectPieces pieceStack;
};



#endif  //__CONNECT_FOUR__