
#ifndef _MESSAGES_H_
#define _MESSAGES_H_

#include <dplay.h>
#include <d3drm.h>
#include <map>
#include "rubiksInfo.h"

#define RED_SQUARE 0
#define GREEN_SQUARE 1
#define ORANGE_SQUARE 2
#define BLUE_SQUARE 3
#define YELLOW_SQUARE 4
#define WHITE_SQUARE 5

#define DPSVRMSG_MOVE		0x9001
#define DPSVRMSG_ADDPIECE	0x9002
#define DPSVRMSG_REMOVEPIECE	0x9003
#define DPSVRMSG_ADDPLAYER	0x9004
#define DPSVRMSG_PICKEDUPPIECE 0x9005
#define DPSVRMSG_DROPPIECE 0x9006
#define DPSVRMSG_WINNER 0x9103
#define DPSVRMSG_RESETGAME 0x9102
#define DPSVRMSG_SET_STATE	0x9101


#define DPAPPMSG_MOVE		0xa001
#define DPAPPMSG_ADDPIECE	0xa002
#define DPAPPMSG_NEWGAMEPLEASE	0xa003

#pragma warning(disable:4200)

using namespace std;

typedef int Animation;

typedef enum _PieceType {
	INVISIBLE_BOX,
	RUBICS_CUBE, 
	ALPHA_BLOCK_A, 
	ALPHA_BLOCK_B, 
	ALPHA_BLOCK_C, 
	MISSILE,  //Probably won't get used later
	CUBE_PROJECTILE,
	WALL,
	RED_RUBIKS_PIECE,
	BLUE_RUBIKS_PIECE,
	GREEN_RUBIKS_PIECE,
	YELLOW_RUBIKS_PIECE,
	WHITE_RUBIKS_PIECE,
	ORANGE_RUBIKS_PIECE,
	CONNECT_FOUR,
	CLOCK,
	BALLOON,
	TOWER,
	NONE,
	VIEWER,
	CONNECT_PROJECTILE,
	PEZ,
	RAM_OBJECT,  //Special attack #1
	TORNADO_OBJECT,  //Special attack #2
	ARCING_OBJECT,  //Special attack #3
	MULTI_SHOT,  //Cannon Powerup #1
	RICOCHETE,  //Cannon Powerup #2
	RICOCHETE_PROJ,
	HOMING_PROJ,
	MORTAR_PROJ,
	CONECT_MORTAR_PROJ,
	LEGO_GUY,
	DART_BOARD,
	POOL_TABLE,
	ENGINE,
	AIR_PLANE,
	POWER_BOX,
	HORSE,
	RAM_BLOCK,
	DECK,
	CHAIR,
	CRAYON,
	DOMINO,
	DICE
} PieceType;

/*typedef struct _Position {
	float x, y, z;
} Position;
*/
typedef D3DVECTOR Position;

typedef struct _Direction {
	float velocity;
	float y;
	float theta_xz;
} Direction;

typedef int PieceID;


typedef DPMSG_GENERIC SVRMSG_RESETGAME;
typedef LPDPMSG_GENERIC LPSVRMSG_RESETGAME;

typedef struct
{
    DWORD  dwType;         // Message type
	PieceID idPiece;
	PieceType color;
} SVRMSG_PICKEDUPPIECE, FAR *LPSVRMSG_PICKEDUPPIECE;

typedef struct
{
    DWORD  dwType;         // Message type
	PieceID idPiece;
	PieceType color;
	char loser_name[10];
	char attacker_name[10];
	int msg_index;
} SVRMSG_DROPPIECE, FAR *LPSVRMSG_DROPPIECE;


typedef struct _Piece {
	DPID PlayerID;
	int idPiece;
} Piece;

typedef struct _Action {
	float velocity;
	float theta;
	BOOL action1;
	BOOL action2;
	BOOL jump;
	BOOL strafe;
} Action; 

typedef struct {
	PieceType ObjectType;
	D3DRMBOX box;
	Position pos;
	Action action;
	Direction dir;
	Animation animation;
} Object, * LPObject;

typedef struct
{
	DWORD       dwType;         // Message type
	PieceID idPiece;
	Action action;
	
} APPMSG_MOVE , FAR *LPAPPMSG_MOVE ;

typedef struct
{
    DWORD       dwType;         // Message type
	PieceID idPiece;
	Action action;

} SVRMSG_MOVE, FAR *LPSVRMSG_MOVE;


typedef struct
{
    DWORD       dwType;         // Message type
	PieceID idPiece;
	D3DRMBOX box;
	PieceType typePiece;
	char name[10];
	int team;
} APPMSG_ADDPIECE, FAR *LPAPPMSG_ADDPIECE;

typedef struct
{
    DWORD    dwType;         // Message type
	PieceID idPiece ;
	Position pos;
	D3DRMBOX box;
	PieceType typePiece;
	float theta_xz;
	float y;
	float scale;
	BOOL centerSquare[6];
	char name[10];
	int team;
} SVRMSG_ADDPIECE, FAR *LPSVRMSG_ADDPIECE;

typedef struct
{
    DWORD    dwType;         // Message type
	short PlayerID;
	char name[10];
	int team;
} SVRMSG_ADDPLAYER, FAR *LPSVRMSG_ADDPLAYER;

/*typedef struct
{
    DWORD       dwType;         // Message type
	PieceID idPiece;
	PieceType typePiece;
} APPMSG_REMOVEPIECE, FAR *LPAPPMSG_REMOVEPIECE;
*/

typedef struct
{
    DWORD    dwType;         // Message type
	PieceID idPiece ;
	PieceType typePiece;
} SVRMSG_REMOVEPIECE, FAR *LPSVRMSG_REMOVEPIECE;

typedef struct {
	long piece;
	rubiks_info info;
} APP_PIECE;

typedef struct
{
    DWORD       dwType;         // Message type
	DWORD       dwSize;
	int NumPiece;
	APP_PIECE objects[];
} SVRMSG_SET_STATE, FAR * LPSVRMSG_SET_STATE;

typedef struct
{
    DWORD    dwType;         // Message type
	PieceID idPiece ;
	int team;
	PieceType typePiece;
} SVRMSG_WINNER, FAR *LPSVRMSG_WINNER;

typedef struct
{
    DWORD    dwType;         // Message type
	PieceID idPiece ;
	char name[10];
} APPMSG_NEWGAMEPLEASE, FAR *LPAPPMSG_NEWGAMEPLEASE;

#endif //_MESSAGES_H_