//-----------------------------------------------------------------------------
// File: RubicsIcons.cpp
//
//
// 4-25-1999:	Grant
//-----------------------------------------------------------------------------

#include "RubicsIcons.h"
#include "RubicsObjects.h"
#include "RubicsDirect3D.h"
#include "RubicsMain.h"
#include "Messages.h"

/***************************************************
 *
 * CIcons()
 *
 * Constructor
 *
 **************************************************/
CIcons::CIcons ( LPDIRECT3DRM3 lpD3DRM, LPDIRECT3DRMFRAME3 Parent, Position pos,
				 float orientation, PieceType type, int rotation, SoundSet *pSS )
	
{
	LPDIRECT3DRMMESHBUILDER3 meshBuilder = NULL;
	LPDIRECT3DRMMESH		 mesh = NULL;
	LPDIRECT3DRMTEXTURE3     pTex = NULL;
	LPDIRECT3DRMFACE2        face = NULL;
	self_frame = NULL;
	pSoundSet = pSS;

	// Create the Mesh Builder for the Wagon
	if (FAILED( lpD3DRM->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	/*
	if( dxfile != NULL) {		// Check to see if we have an X File to load
		// Load the X file
		if( FAILED( meshBuilder->Load((LPVOID) dxfile ,NULL,D3DRMLOAD_FROMFILE, NULL,NULL) ) )
		{
			DISPLAYMSG("Failed to load .X file");
			goto error;
		}
	}
	else goto error;
	*/
	Get3DRectangle( meshBuilder, 
					4.0f,4.0f, 4.0f,			// Cube Dimensions 
					0.0f, 5.0f, 0.0f,	// Center of cube in relation to meshBuilder
					WHITE, WHITE, WHITE, WHITE, WHITE, WHITE);  // Color of the cube

	switch (GetPlayerType())
	{
	case RUBICS_CUBE: 
		switch (type)
		{
			case MULTI_SHOT:
				lpD3DRM->LoadTexture( "3shot.bmp", &pTex ); 
				break;
			case TORNADO_OBJECT:
				lpD3DRM->LoadTexture( "tornado.bmp", &pTex );
				break;
			case RAM_OBJECT:
				lpD3DRM->LoadTexture( "ram.bmp", &pTex ); 
				break;
			case ARCING_OBJECT:
				lpD3DRM->LoadTexture( "mortar.bmp", &pTex ); 
				break;
			case RICOCHETE:
				lpD3DRM->LoadTexture( "Ricochet.bmp", &pTex ); 
				break;
			default:
				lpD3DRM->LoadTexture( "mystery.bmp", &pTex ); 
				break;
		}
		break;
	case CONNECT_FOUR: 
		switch (type)
		{
			case MULTI_SHOT:
				lpD3DRM->LoadTexture( "3shot_checker.bmp", &pTex ); 
				break;
			case TORNADO_OBJECT:
				lpD3DRM->LoadTexture( "missile.bmp", &pTex );
				break;
			case RAM_OBJECT:
				lpD3DRM->LoadTexture( "ram.bmp", &pTex ); 
				break;
			case ARCING_OBJECT:
				lpD3DRM->LoadTexture( "mortar_checker.bmp", &pTex ); 
				break;
			case RICOCHETE:
				lpD3DRM->LoadTexture( "Ricochet.bmp", &pTex ); 
				break;
			default:
				lpD3DRM->LoadTexture( "mystery.bmp", &pTex ); 
				break;
		}
		break;
	default:
		switch (type)
		{
			case MULTI_SHOT:
				lpD3DRM->LoadTexture( "3shot.bmp", &pTex ); 
				break;
			case TORNADO_OBJECT:
				lpD3DRM->LoadTexture( "tornado.bmp", &pTex );
				break;
			case RAM_OBJECT:
				lpD3DRM->LoadTexture( "ram.bmp", &pTex ); 
				break;
			case ARCING_OBJECT:
				lpD3DRM->LoadTexture( "mortar.bmp", &pTex ); 
				break;
			case RICOCHETE:
				lpD3DRM->LoadTexture( "Ricochet.bmp", &pTex ); 
				break;
			default:
				lpD3DRM->LoadTexture( "mystery.bmp", &pTex ); 
				break;
		}
		break;
	}
	
	if(    !SetTextureToFace( 1, 4.0f, pTex, meshBuilder)  
		|| !SetTextureToFace( 2, 4.0f, pTex, meshBuilder) 
		|| !SetTextureToFace( 3, 4.0f, pTex, meshBuilder) 
		|| !SetTextureToFace( 4, 4.0f, pTex, meshBuilder) 
		|| !SetTextureToFace( 5, 4.0f, pTex, meshBuilder) 
		|| !SetTextureToFace( 0, 4.0f, pTex, meshBuilder))
	{ 	
		DISPLAYMSG("Unable to set Texture to face.");
		goto error;
	}

	/*Create a shadow */
	if (FAILED(meshBuilder->CreateFace(&face)))
	{
		DISPLAYMSG("Unable to create shadow in icon");
		goto error;
	}
	face->AddVertex(-2.0f, 0.1f, -2.0f);
	face->AddVertex(-2.0f, 0.1f,  2.0f);
	face->AddVertex( 2.0f, 0.1f,  2.0f);
	face->AddVertex( 2.0f, 0.1f, -2.0f);
	face->SetColorRGB(0.0f, 0.0f, 0.0f);
	RELEASE(face);

	//if (FAILED(meshBuilder->AddFace

	// Create Mesh to be used for the Wagon
	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;
	}
	
	// Create frame that the outside world will see
	if (FAILED(lpD3DRM->CreateFrame( Parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create self frame");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to frame" );
		goto error;
	}

	if (FAILED(self_frame->SetPosition(Parent, pos.x, pos.y, pos.z)))
				DISPLAYMSG("Error setting Position on Non-Movable Object");

	if (FAILED(self_frame->SetOrientation(Parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on Non-Movable Object");

	switch(rotation){
	case(0):
		break;
	case(1):
		if (FAILED(self_frame->SetRotation(Parent, D3DVAL(0), D3DVAL(1), 
												D3DVAL(0), D3DVAL(0.2))))
				DISPLAYMSG("Error setting rotation on our icon");
		break;
	default:
		break;
	}
error:
	RELEASE( meshBuilder );
	RELEASE( mesh );
	
}

/***************************************************
 *
 * SetTexture
 *
 * Sets the Texture on the front and back of the block
 *
 ***************************************************/
BOOL CIcons::SetTextureToFace( int faceNumber, float fBlockSize,
		 					   LPDIRECT3DRMTEXTURE3 pTex, 
							   LPDIRECT3DRMMESHBUILDER3 mb )
{
	int x = 0,y = 0,z = 0;	// Wrap direction vectors
	LPDIRECT3DRMFACE2 face = NULL;
	LPDIRECT3DRMWRAP  lpWrap = NULL;
	float xwrap = 3.0f;
	float ywrap = 1.0f;	

	switch( faceNumber )
	{
	case 0:
		y = 1;
		xwrap = 1.0;
		break;
	case 1:
		y = -1;
		xwrap = 1.0;
		break;
	case 2:
		z = 1;
		break;
	case 3:
		z = -1;
		break;
	case 4:
		x = 1;
		break;
	case 5:
		x = -1;
		break;
	}

	////////////////////////////////////////////////
	// Set Texture of Alphabet Block's face
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,							// Wrap Type and Reference Frame
        D3DVAL(-2.0f), D3DVAL(3.0), D3DVAL(-2.0f),			// Wrap Origin
		D3DVAL(x), D3DVAL(y), D3DVAL(z),				// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),			// Wrap Up Vector
   		D3DVAL(1.0), D3DVAL(0.0),						// Texture Origin
		//DVAL(-0.25), D3DVAL(-0.25),	// Texture Scale Factor
		D3DVAL(-1.0/fBlockSize), D3DVAL(-1.0/fBlockSize/ywrap),	// Texture Scale Factor
        &lpWrap);

	if( FAILED( mb->GetFace( faceNumber, &face ) ) )
	{
		DISPLAYMSG("Unable to get face");
		goto error;
	}


    if( FAILED( lpWrap->Apply( (LPDIRECT3DRMOBJECT) face ) ) )
	{
		DISPLAYMSG("Unable to apply wrap to face");
		goto error;
	}

	if( FAILED( face->SetTexture( pTex ) ) )
	{
		DISPLAYMSG("Unable to set texture to face");
		goto error;
	}
	
	RELEASE( face );
	RELEASE( lpWrap );
	return TRUE;
error:
	RELEASE( face );
	RELEASE( lpWrap );
	return FALSE;
}

CIcons::ScaleMe(float fx, float fy, float fz )
{
	self_frame->AddScale(D3DRMCOMBINE_BEFORE, fx, fy, fz);;
}