//-----------------------------------------------------------------------------
// File: RubicsMain.cpp
//
// Desc: This is the main function for our rubics game.
//
// 4-1-1999:	Modified the RMBegin2 to sample
//-----------------------------------------------------------------------------
#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <windowsx.h>
#include <wtypes.h>
#include <cguid.h>
#include <dplobby.h>	// to init guids
#include <map>
#include <math.h>
#include <crtdbg.h>
#include <dsound.h>

#include "RubicsMain.h"
#include "RubiksAlphabetBlock.h"
#include "RubicsWall.h"
#include "RubicsDirect3D.h"
#include "RubicsObjects.h"
#include "Messages.h"
#include "dpchat.h"
#include "resource.h"
#include "devices.h"
#include "rubikscube.h"
#include "ConnectFour.h"
#include "client_objects.h"
#include "RubicsIcons.h"
#include "ClientProjectile.h"
#include "ConnectProjectile.h"
#include "ClientMortar.h"
#include "ClientViewer.h"
#include "ClientRubiksPiece.h"
#include "dmusic.h"
#include "RubiksSoundSet.h"
#include "Overlay.h"
#include "ClientPlayer.h"	// Added by ClassView
#include <winuser.h>
#define NUM_VIEWS 5
#define INITIAL_DISTANCE 45
#define INITIAL_VORIENT -0.24
#define CLIPPING_DIST 1000.0
#define SOUND_CLEANUP_CYCLE_TIME 2000 // Sound buffers cleaned up every 2 sec

#define NUM_INPUTS 9
#define KEY_UP 0
#define KEY_DOWN 1
#define KEY_RIGHT 2
#define KEY_LEFT 3
#define ACTION1 4
#define ACTION_2 5
#define JUMP 6
#define STRAFE 7
#define RUN 8


extern LPDPLAYINFO lpDPInfo;

#pragma warning(disable:4786)
using namespace std;



//A structure for keeping track of the current direction the
//users piece is heeding.
typedef struct _DIRECTIONS 
{
	int right;
	int left;
	int up;
	int down;
} DIRECTIONS;

// Global Structure
struct _myglobs
{
    // Direct3DRM device. 
    LPDIRECT3DRMDEVICE3 dev;           

    // Direct3DRM window device. 
    LPDIRECT3DRMWINDEVICE windevice;           

    // Direct3DRM viewport through which to view the scene. 
    LPDIRECT3DRMVIEWPORT2 view;  

    // Master frame in which other frames are placed. 
    LPDIRECT3DRMFRAME3 scene;    

    // Frame describing the users point of view. 
    LPDIRECT3DRMFRAME3 camera;   

	LPDIRECT3DRMFRAME3 player;   
	
	LPDIRECT3DRMFRAME3 board;   
	
	//Light used for shadows
	LPDIRECT3DRMLIGHT shadowLight;
	
	LPDIRECTDRAWSURFACE4 lpddsPrimary; 
	LPDIRECTDRAWSURFACE4 lpddsRender;
    // Application is minimized. 

    HWND hWnd;

	BOOL bMinimized;            
	
	char * strStatus;

    // All D3DRM objects have been initialized. 
    BOOL bInitialized;          
	
	DIRECTIONS directions;
	
	short PlayerID;
	short nObjectID;

	int view_number;
	float vorientation;
	int distance;

	float angle;
	float vel;

	PieceID MyID;
	DPID ServerID;
	PieceType type;

	int  team;
	int  deck_color;

	bool input_changed[NUM_INPUTS];
	bool action1;
	bool action2;
	bool jump;
	bool strafe;
	bool run;

	bool fullscreen;
	

	DMusic * rubMus;	// Direct Music

	SoundSet *pSoundSet; // Direct Sound Set
	Overlay* rubOverlay; // overlay


} myglobs;

ObjectMap mapPieces;


//////////////////////////////////////////////////////////
/////////////////// Prototypes 
//////////////////////////////////////////////////////////
BOOL             BuildScene( LPDIRECT3DRM3, LPDIRECT3DRMDEVICE3, 
                             LPDIRECT3DRMFRAME3, LPDIRECT3DRMFRAME3 );
static BOOL      RenderScene();
static BOOL		 CreateObjects(HINSTANCE hInstance,  HWND win );
static HWND      InitApp( HINSTANCE, int );
int APIENTRY     WinMain( HINSTANCE, HINSTANCE, LPSTR, int );
LRESULT CALLBACK WindowProc( HWND, UINT, WPARAM, LPARAM );

void GetXandZ(DIRECTIONS *lpDirection, float * x, float *z);
HRESULT SetPlayerVelocity(LPDIRECT3DRMFRAME3 player, DIRECTIONS * lpDirection);
HRESULT MovePieces(DIRECTIONS * lpDirections);

bool HandleApplicationMessage(LPDPLAYINFO lpDPInfo, LPDPMSG_GENERIC lpMsg, DWORD dwMsgSize,
							  DPID idFrom, DPID idTo);
bool ReceiveMessage(LPDPLAYINFO lpDPInfo);
void NewPiece(PieceID idPiece, PieceType type, Position pos, float orientation, BOOL centerSquare[6], int team, float scale, char playername[]);
void RemovePiece(PieceID idPiece, PieceType type);
//RubiksCube* NewRubicsPiece();
//void NewAlphaPiece(Position pos);
void ChangeCameraView(int view_number);
void AddPlayer(short id, char playername[]);
void SendAddPlayerMsg(char playername[]);
void DropPiece(PieceID id, PieceType piece);
void PickupPiece(PieceID id, PieceType piece);
PieceType ChooseType(HINSTANCE hInstance, HWND win);
BOOL CALLBACK SetupGameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void UpdateObjects();
void DeleteChildren(LPDIRECT3DRMFRAME parent);
void ResetGame();
void AttachCamera(LPDIRECT3DRMFRAME3 piece);

//-----------------------------------------------------------------------------
// Name: GetXandY()
// Desc: Converts the direction the player is heading into x and z coordinates
//-----------------------------------------------------------------------------
void GetXandZ(DIRECTIONS *lpDirection, float * x, float *z) {
	*x = (lpDirection->right - lpDirection->left)/ SPEED_FACTOR;
	*z = (lpDirection->up - lpDirection->down) / SPEED_FACTOR;
}

//-----------------------------------------------------------------------------
// Name: SetPlayerVelocity()
// Desc: Given a direction it computes and sets the players velocity.
//-----------------------------------------------------------------------------
HRESULT SetPlayerVelocity(LPDIRECT3DRMFRAME3 player, DIRECTIONS * lpDirection) {
	float x, z;
	GetXandZ(lpDirection, &x, &z);
	if(player)
		return player->SetVelocity(myglobs.scene, x, 0, z, false);
	else return S_OK;
}

//-----------------------------------------------------------------------------
// Name: SoundCleanUpProc()
// Desc: Cleans up the sound set whenever it is called
//-----------------------------------------------------------------------------
VOID CALLBACK SoundCleanUpProc(
  HWND hwnd,     // handle of window for timer messages
  UINT uMsg,     // WM_TIMER message
  UINT idEvent,  // timer identifier
  DWORD dwTime   // current system time
  ) 
{
	myglobs.pSoundSet->CleanUpBuffers();	
}



//-----------------------------------------------------------------------------
// Name: MovePieces()
// Desc: This function will move all the pieces in the ObjectSet that need to be moved.
//
//4-1-1999:	 It currently only moves the current players piece.
//-----------------------------------------------------------------------------
HRESULT MovePieces(DIRECTIONS * lpDirections) {
	//ObjectMap mapPieces
	return  SetPlayerVelocity(myglobs.player, lpDirections);
}


//-----------------------------------------------------------------------------
// Name: BuildScene()
// Desc: Create the scene to be rendered. 
//-----------------------------------------------------------------------------
BOOL BuildScene( LPDIRECT3DRM3 lpD3DRM,  LPDIRECT3DRMDEVICE3 dev, 
                 LPDIRECT3DRMFRAME3 scene, LPDIRECT3DRMFRAME3 camera )
{
    LPDIRECT3DRMMESHBUILDER3 board_builder = NULL;
    LPDIRECT3DRMMESH board_mesh = NULL;
    LPDIRECT3DRMFRAME3 board = NULL;
    LPDIRECT3DRMLIGHT light1 = NULL;
    LPDIRECT3DRMLIGHT light2 = NULL;
    LPDIRECT3DRMFRAME3 lights = NULL;
    LPDIRECT3DRMWRAP wrap = NULL;
	
	/*AlphabetBlock *pBlock;
	LPDIRECT3DRMFRAME3 pFrame = NULL;*/

	HRESULT rval;
    
    if (FAILED(scene->SetSceneBackground(BLACK)))
        goto generic_error;

    // Create a meshbuilder for each object.
    if (FAILED(lpD3DRM->CreateMeshBuilder(&board_builder)))
        goto generic_error;

	rval = GetBoard(board_builder);
    
    if (FAILED(rval)) {
        DISPLAYMSG("Failed to create the stupid board.");
        goto ret_with_error;
    }

    if (FAILED(lpD3DRM->CreateFrame(scene, &board)))
        goto generic_error;
	myglobs.board = board;
	board->AddRef();

    // Create meshes from the meshbuilders to avoid the same future 
    // conversion by Direct3D Retained mode. The meshbuilders could be
    // released now, but for clarity they are released with the other
    // objects at the end of this function.
    if (FAILED(board_builder->CreateMesh(&board_mesh))) 
        goto generic_error;

    // Add the meshes into the frames.
    if (FAILED(board->AddVisual((LPDIRECT3DRMVISUAL) board_mesh)))
        goto generic_error;

    // Set up the frame positions.
    if (FAILED(camera->SetPosition(scene, D3DVAL(0), D3DVAL(myglobs.distance), 
                                   -D3DVAL(myglobs.distance))))
        goto generic_error;
    if (FAILED(camera->SetOrientation(scene, D3DVAL(0), -D3DVAL(1), 
                                   D3DVAL(1), D3DVAL(0), D3DVAL(1.0), 
                                   D3DVAL(0))))
        goto generic_error;
	if (FAILED(board->SetPosition(scene, D3DVAL(0), D3DVAL(0), 
                                D3DVAL(0))))
        goto generic_error;
    
    // Initialize the lights in the scene. 
    // - Create a light frame. Set its position within the scene 
    //   (straight ahead, directly in front of the sun, to illuminate
    //   the sun), and attach a point light to the light frame.
    // - Create a soft ambient light and add it to the scene.
    
    if (FAILED(lpD3DRM->CreateFrame(scene, &lights)))
        goto generic_error;
    if (FAILED(lights->SetPosition(scene, D3DVAL(0), D3DVAL(100),
                                -D3DVAL(7)))) 
        goto generic_error;
	if (FAILED(lpD3DRM->CreateLightRGB(D3DRMLIGHT_PARALLELPOINT, D3DVAL(1.0),
                                  D3DVAL(1.0), D3DVAL(1.0), &light1)))
        goto generic_error;
	//Add a pointer to this light so we can construct shadows later
	myglobs.shadowLight = light1;
	light1->AddRef();

    if (FAILED(lights->AddLight(light1)))
        goto generic_error;

    // Create an ambient light and add it to the scene. 
    if (FAILED(lpD3DRM->CreateLightRGB(D3DRMLIGHT_AMBIENT, D3DVAL(0.8),
                                  D3DVAL(0.8), D3DVAL(0.8), &light2)))
        goto generic_error;
    if (FAILED(scene->AddLight(light2)))
        goto generic_error;
  
	/*// Testing for Alphabet Block
	pBlock = new AlphabetBlock( lpD3DRM, board );
	pBlock->GetFrame( &pFrame );
	if (FAILED(pFrame->SetRotation(scene, D3DVAL(0), D3DVAL(1), 
												D3DVAL(0), D3DVAL(0.2))))
		goto generic_error;	*/				 

	
    // Clean up.
	//RELEASE(pFrame);
    RELEASE(board_builder);
    RELEASE(board_mesh);
    RELEASE(board);
    RELEASE(lights);
    RELEASE(light1);
    RELEASE(light2);
    return TRUE;

generic_error:
    DISPLAYMSG("A failure occurred while building the scene.");
ret_with_error:
	//RELEASE(pFrame);
    RELEASE(board_builder);
    RELEASE(board_mesh);
    RELEASE(board);
    RELEASE(lights);
    RELEASE(light1);
    RELEASE(light2);
    return FALSE;
}


//-----------------------------------------------------------------------------
// Name: RenderScene()
// Desc: Clear the viewport, render the next frame, and update the window.
//-----------------------------------------------------------------------------

static BOOL RenderScene()
{
    HRESULT rval;
	//unreferenced: HDC dc;

    // Move the scene.
    //MovePieces(Pieces,&myglobs.directions);

	UpdateObjects();

	rval = myglobs.scene->Move(D3DVAL(1.0));
    if (FAILED(rval)) {
        DISPLAYMSG("Moving scene failed.");
        return FALSE;
    }

    // Clear the viewport.
	rval = myglobs.view->ForceUpdate(0,0, myglobs.view->GetWidth(), myglobs.view->GetHeight());
	if (FAILED(rval)) {
        DISPLAYMSG("Clearing viewport failed.");
        return FALSE;
    }
	rval = myglobs.view->Clear(D3DRMCLEAR_ALL);
    if (FAILED(rval)) {
        DISPLAYMSG("Clearing viewport failed.");
        return FALSE;
    }
	
	rval = myglobs.view->Render(myglobs.scene);
    if (FAILED(rval)) {
        DISPLAYMSG("Rendering scene failed.");
        return FALSE;
    }
	
	// draw the overlays
	if(myglobs.rubOverlay->drawEndingOverlay == FALSE) {
		myglobs.rubOverlay->DrawRadar(myglobs.MyID,mapPieces);
		myglobs.rubOverlay->DrawMyStat(mapPieces, myglobs.MyID);
		myglobs.rubOverlay->DrawAllPlayersStat(mapPieces,myglobs.MyID);
		myglobs.rubOverlay->DisplayMessage();
	}
	if(myglobs.rubOverlay->drawEndingOverlay)
		myglobs.rubOverlay->DisplayEndingOverlay(myglobs.team);
	// Flip Front and Back buffers (Primary, Render) 
    if(myglobs.fullscreen && myglobs.lpddsPrimary) {
	rval =myglobs.lpddsPrimary->Flip (myglobs.lpddsRender/*NULL*/, 0); 
		if(rval == DDERR_SURFACELOST) {
			rval = myglobs.lpddsPrimary->Restore();
			if (!FAILED(rval)) {
				rval =myglobs.lpddsPrimary->Flip (NULL, 0); 
			}		
		}
	}
	

	/*rval = myglobs.view->Clear(D3DRMCLEAR_ALL);
    if (FAILED(rval)) {
        DISPLAYMSG("Clearing viewport failed.");
        return FALSE;
    }*/

    
	if (FAILED(rval)) {
        //DISPLAYMSG("Flipping viewport failed.");
        //return FALSE;
    }

	// Render the scene to the viewport.

    // Update the window.
    rval = myglobs.dev->Update();
    if (FAILED(rval)) {
        DISPLAYMSG("Updating device failed.");
        return FALSE;
    }
    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: AddMediaPath()
// Desc: Looks in the system registry to determine the media path for the
//       sample. Then, it adds that path to the string passed in, checks if the
//       file exists, and returns a path to the file.
//-----------------------------------------------------------------------------
VOID AddMediaPath( LPDIRECT3DRM3 pD3DRM )
{
	HKEY   key;
	LONG   result;
	TCHAR  strPath[512];
	DWORD  type, size = 512;

	// Open the registry
	result = RegOpenKeyEx( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\DirectX",
						   0, KEY_READ, &key );
	if( ERROR_SUCCESS != result )
		return;

	// Search for the desired registry value, and close the registry
	result = RegQueryValueEx( key, "DX6SDK Samples Path", NULL, &type, 
		                      (BYTE*)strPath, &size );
	RegCloseKey( key );

	if( ERROR_SUCCESS != result )
		return;

	strcat( strPath, "\\D3DRM\\Media" );

	pD3DRM->AddSearchPath( strPath );
	_getcwd (strPath, size);
	pD3DRM->AddSearchPath( strPath );
	return;
}


/* **----------------------------------------------------------------------------- ** 
 Name:       FlagsToBitDepth **  Purpose:    Gets Bit Depth from DDPF Flags 
**----------------------------------------------------------------------------- */  
DWORD FlagsToBitDepth (DWORD dwFlags) 
{     
	if (dwFlags & DDBD_1)
		return 1L;   
	else if (dwFlags & DDBD_2)     
		return 2L;    
	else if (dwFlags & DDBD_4)      
		return 4L;     
	else if (dwFlags & DDBD_8)  
		return 8L;     
	else if (dwFlags & DDBD_16)   
		return 16L;    
	else if (dwFlags & DDBD_24)      
		return 24L;     
	else if (dwFlags & DDBD_32)   
		return 32L;     
	else         return 8L;
} // End FlagsToBitDepth 


//-----------------------------------------------------------------------------
// Name: CreateObjects()
// Desc: Initialize globals, create the device and objects. 
//-----------------------------------------------------------------------------
BOOL CreateObjects(HINSTANCE hInstance,  HWND win )
{
    HRESULT rval; // Return value 
    RECT rc;      // Bounding rectangle for main window 
    int width;    // Device's width 
    int height;   // Device's height
	
	DDSURFACEDESC ddsd; 
	DDSCAPS ddscaps;
	DDCAPS_DX6 DeviceDesc;	
	//CAPS\ DeviceDesc1;
	
	
	LPDIRECTDRAWSURFACE lpddsPrimary1 = NULL;
	LPDIRECTDRAWSURFACE lpddsRender1 = NULL;
	LPDIRECTDRAWSURFACE lpddsBack1 = NULL;
	LPDIRECTDRAWSURFACE lpddsZBuff = NULL;
	
	LPDIRECTDRAW lpDD1 = NULL;
	
	LPGUID lpDevGuid = 0, lpDrawGuid = 0;
	
    // Initialize the entire global variable structure to zero. 
	myglobs.distance = INITIAL_DISTANCE;
	myglobs.vorientation = (float)INITIAL_VORIENT;
	mapPieces.clear();
	
	// Create the Direct3DRM object.
    LPDIRECT3DRM pD3DRMTemp;
    rval = Direct3DRMCreate(&pD3DRMTemp);
    if (FAILED(rval)) 
    {
        DISPLAYMSG("Failed to create Direct3DRM.");
        return FALSE;
    }
    if( FAILED( pD3DRMTemp->QueryInterface(IID_IDirect3DRM3, (void **)&lpD3DRM) ) )
    {
        RELEASE(pD3DRMTemp);
        DISPLAYMSG("Failed query for Direct3DRM3.\n" );
        return FALSE;
    }
    RELEASE(pD3DRMTemp);
	
	ChooseDrawDeviceUsingDialog(hInstance, &lpDrawGuid);
	if( FAILED(DirectDrawCreate(lpDrawGuid, &lpDD1, NULL)))
	{
        DISPLAYMSG("Failed query for DirectDraw.\n");
        return FALSE;
    }
	if( FAILED( lpDD1->QueryInterface(IID_IDirectDraw4, (void **)&lpDD) ) )
    {
        DISPLAYMSG("Failed query for DirectDraw4.\n");
        return FALSE;
    }
	if(lpDrawGuid) {
		myglobs.fullscreen = true;
		ChooseDeviceUsingDialog(hInstance, &lpDevGuid);
		
		if( FAILED(lpDD->SetCooperativeLevel(win, /*DDSCL_NORMAL |*/ DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWREBOOT ) ))
		{
			DISPLAYMSG("Failed to SetCooperativeLevel.\n" );
			return FALSE;
		}
		
		DWORD dwFlags =0;
		
		HDC hdc;
		hdc = GetDC(NULL);
		int bpp = GetDeviceCaps(hdc, BITSPIXEL);
		width = GetDeviceCaps(hdc, HORZRES);
		height = GetDeviceCaps(hdc, VERTRES);
		
		// Special check for mode 320 x 200 x 8 
		if ((width == 320) && (height== 200) && (bpp == 8))  
		{ 
			// Make sure we use Mode 13 instead of Mode X 
			dwFlags = DDSDM_STANDARDVGAMODE;  
		}       // Set Requested Fullscreen mode   
		
				/*if( FAILED(lpDD->SetDisplayMode (width, height, bpp, 0, dwFlags) ))
				{
				DISPLAYMSG("Failed to SetDisplayMode.\n" );
				return FALSE;
	}*/
		
		memset(&ddsd, 0, sizeof(ddsd));
		
		ddsd.dwSize= sizeof(ddsd);
		ddsd.dwFlags= DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
		ddsd.ddsCaps.dwCaps= DDSCAPS_VIDEOMEMORY | DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP |DDSCAPS_COMPLEX | DDSCAPS_3DDEVICE;
		// Create a D3D compatible surface
		ddsd.dwBackBufferCount= 1;
		// Create Primary surface
		
		rval = lpDD1->CreateSurface (&ddsd, &lpddsPrimary1, NULL);
		if (FAILED (rval ))     {
			DISPLAYMSG("Failed to CreateSurface.");
			return FALSE;
		} 

		rval = lpddsPrimary1->QueryInterface(IID_IDirectDrawSurface4 , (void **)&myglobs.lpddsPrimary);
		if (FAILED (rval ))     {
			DISPLAYMSG("Failed to CreateSurface.");
			return FALSE;
		} 

		/*ddsd.dwFlags        = DDSD_WIDTH | DDSD_HEIGHT | DDSD_CAPS;
		ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY | DDSCAPS_3DDEVICE;
		
		  // Create a D3D compatible surface
		  //ddsd.dwBackBufferCount= 1;
		  // Set the dimensions of the back buffer. Note that if our window changes
		  // size, we need to destroy this surface and create a new one.
		  ddsd.dwWidth  = width;
		  ddsd.dwHeight = height;
		  
			// Create the back buffer. The most likely reason for failure is running
			// out of video memory. (A more sophisticated app should handle this.)
			rval = lpDD1->CreateSurface( &ddsd, &lpddsBack1, NULL );
			
			  if (FAILED (rval ))     {
			  DISPLAYMSG("Failed to CreateSurface.");
			  return FALSE;
	} */
		
		ddscaps.dwCaps = DDSCAPS_BACKBUFFER; 
		rval = lpddsPrimary1->GetAttachedSurface (&ddscaps, &lpddsRender1);
		if (FAILED (rval)) 	{
			DISPLAYMSG("Error GetAttachedSurface");
			return rval; 
		}
		
		rval = lpddsRender1->QueryInterface(IID_IDirectDrawSurface4 , (void **)&myglobs.lpddsRender);
		if (FAILED (rval ))     {
			DISPLAYMSG("Failed to CreateSurface.");
			return FALSE;
		} 
		char buffer[256];
		DWORD dwMemType;
		//if (myglobs.fullscreen) { 
			// Hardware device 
			// Z-buffer has to be in Video memory 
			ZeroMemory(&DeviceDesc, sizeof(DeviceDesc));
			//ZeroMemory(&DeviceDesc1, sizeof(DeviceDesc1));
			rval = lpDD->GetCaps(&DeviceDesc,NULL);
			if (FAILED (rval)) {
				//sprintf(buffer, "Error GetCaps. rval = %d", rval);
				//DISPLAYMSG(buffer);
				//DISPLAYMSG("Error GetCaps"); 
				//return rval; 
			} 
			dwMemType = DDSCAPS_VIDEOMEMORY; 
		//} 
		/*else { // Software device  
			// Z-Buffer has to be in System memory 
			lpDD1->GetCaps(NULL,&DeviceDesc );
			dwMemType = DDSCAPS_SYSTEMMEMORY;
		} */ 
		// Should we create a Z-buffer ?!?     
		if ((0L != DeviceDesc.dwZBufferBitDepths)) 
		{ 
			// Get Z-buffer surface info (w, h, bpp, video vs. system memory) 
			ddsd.dwSize = sizeof(ddsd); 
			rval = lpddsPrimary1->GetSurfaceDesc (&ddsd); 
			if (FAILED (rval)) {
				DISPLAYMSG("Error GetSurfaceDesc "); 
				return rval; 
			} 
			DWORD dwWidth   = ddsd.dwWidth;
			DWORD dwHeight  = ddsd.dwHeight;
			DWORD dwBPP  = FlagsToBitDepth (DeviceDesc.dwZBufferBitDepths);

			// Create the z-buffer.         
			ZeroMemory (&ddsd, sizeof(ddsd));
			ddsd.dwSize            = sizeof(ddsd);
			ddsd.dwFlags           = DDSD_CAPS | DDSD_WIDTH  | DDSD_HEIGHT | DDSD_ZBUFFERBITDEPTH;
			ddsd.ddsCaps.dwCaps    = DDSCAPS_ZBUFFER | dwMemType;
			ddsd.dwWidth           = dwWidth; 
			ddsd.dwHeight          = dwHeight;
			ddsd.dwZBufferBitDepth = dwBPP;
			
			rval = lpDD1->CreateSurface (&ddsd, &lpddsZBuff, NULL);
			if (FAILED(rval))
			{
				//DISPLAYMSG("Failed: Error creating Zbuffer");
				// Note: we may be able to continue without a z buffer
				// So don't exit
			} else { // Attach Z-buffer to rendering surface 
				rval = lpddsRender1->AddAttachedSurface (lpddsZBuff);
				if (FAILED (rval)) {
					sprintf(buffer, "Error adding Zbuffer. rval = %d", rval);
					DISPLAYMSG(buffer);
					//DISPLAYMSG("Error adding Zbuffer");
					if (lpddsZBuff) {
						lpddsZBuff->Release ();
						lpddsZBuff = NULL;
					}  
				}
			}
		}
		
		rval = lpD3DRM->CreateDeviceFromSurface(lpDevGuid, lpDD1, lpddsRender1/*lpddsPrimary1*/, 0 ,  &myglobs.dev);
		if (FAILED(rval)) {
			DISPLAYMSG("Failed to create the D3DRM device.");
			return FALSE;
		}
	} else {
		myglobs.fullscreen = false;
		// Create a DirectDrawClipper object and associate the window with it.
		rval = DirectDrawCreateClipper(0, &lpDDClipper, NULL);
		if (FAILED(rval)) {
			DISPLAYMSG("Failed to create DirectDrawClipper object.");
			return FALSE;
		}
		rval = lpDDClipper->SetHWnd(0, win);
		if (FAILED(rval)) {
			DISPLAYMSG("Failed to set the window handle for the DirectDrawClipper.");
			return FALSE;
		}
		
		GetClientRect(win, &rc);
		rval = lpD3DRM->CreateDeviceFromClipper(lpDDClipper, 
			lpDevGuid, //Choosen device
			rc.right, rc.bottom, &myglobs.dev);
		if (FAILED(rval)) {
			DISPLAYMSG("Failed to create the D3DRM device.");
			return FALSE;
		}
	}
	
	rval = myglobs.dev->SetQuality(D3DRMRENDER_GOURAUD);
	
    // Create a Direct3D Retained Mode window device to handle the
    // WM_ACTIVATE and WM_PAINT messages.
    rval = myglobs.dev->QueryInterface(IID_IDirect3DRMWinDevice,
		(void **) &myglobs.windevice);
    if (FAILED(rval)) {
        //DISPLAYMSG("Failed to create the window device.");
        return FALSE;
    }
    
    
    // Create the master scene frame and the camera frame.
    rval = lpD3DRM->CreateFrame(NULL, &myglobs.scene);
    if (FAILED(rval)) {
        DISPLAYMSG("Failed to create the master scene frame.");
        return FALSE;
    }
	
    rval = lpD3DRM->CreateFrame(myglobs.scene, &myglobs.camera);
    if (FAILED(rval)) {
        DISPLAYMSG("Failed to create the camera's frame.");
        return FALSE;
    }
	
    // Create the Direct3DRM viewport using the device, camera frame,
    // and the device's width and height.
    
	width = myglobs.dev->GetWidth();
    height = myglobs.dev->GetHeight();
	
    rval = lpD3DRM->CreateViewport(myglobs.dev, myglobs.camera, 0, 0, 
		width, height, &myglobs.view);
    if (FAILED(rval)) {
        myglobs.bInitialized = FALSE;
        RELEASE(myglobs.dev);      
        return FALSE;
    }
	
	//if (FAILED(myglobs.dev->SetSceneBackground(BLACK)))
    //    goto generic_error;

	// Set our new clipping distance
	if (FAILED(myglobs.view->SetBack(D3DVAL(CLIPPING_DIST)))){
        myglobs.bInitialized = FALSE;
        RELEASE(myglobs.dev);
        return FALSE;
    }
	
	// Add the media path so our textures and .x files can be found
	AddMediaPath( lpD3DRM );
	// create the overlay object
	myglobs.rubOverlay = new Overlay(lpDD,myglobs.lpddsRender,win);
    // Create the scene to be rendered.
    if (!BuildScene(lpD3DRM, myglobs.dev, myglobs.scene, 
		myglobs.camera))
        return FALSE;

	// Create and initialize DirectMusic
	myglobs.rubMus = new DMusic(NULL);
	myglobs.rubMus->PlayMidi(NULL,TRUE);

	// Create and initialize Direct Sound
	myglobs.pSoundSet = new SoundSet( win, NULL );
	SetTimer(NULL, NULL, SOUND_CLEANUP_CYCLE_TIME, SoundCleanUpProc);
	
	
    // Globals are initialized.
    myglobs.bInitialized = TRUE;
    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: InitApp()
// Desc: Create the main window and initialize objects. 
//-----------------------------------------------------------------------------
static HWND InitApp( HINSTANCE this_inst, int cmdshow )
{
    HWND win;     // Main window handle 
    WNDCLASS wc;
	
	
    // Set up and register the window class.
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = this_inst;
    wc.hIcon = LoadIcon(this_inst, "AppIcon"); 
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "D3DRM Example";
    if (!RegisterClass(&wc))
        return FALSE;

    // Create the window.
    win = CreateWindow(   
            "D3DRM Example",                   // class 
            "RubX Cubed", // caption 
            /*WS_MINIMIZE */WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX /* WS_OVERLAPPEDWINDOW*/,               // style 
            CW_USEDEFAULT,                     // init. x pos 
            CW_USEDEFAULT,                     // init. y pos 
            /*0*/350,                               // init. x size 
            /*0*/300,                               // init. y size 
            NULL,                              // parent window 
            NULL,                              // menu handle 
            this_inst,                         // program handle 
            NULL                               // create parms 
        );
    if (!win)
        return FALSE;

	myglobs.hWnd = win;
	myglobs.rubMus = NULL;
    //myglobs.type = RUBICS_CUBE;
	//myglobs.type = ChooseType(this_inst, win);
	// Initialize global variables, enumerate devices, and create the 
    // D3DRM objects.
    if (!CreateObjects(this_inst, win))
        return FALSE;

    // Display the window.
    //if(!myglobs.fullscreen) {
		ShowWindow(win, cmdshow);
		UpdateWindow(win);
	//} 
	/*else {
		ShowWindow(win, cmdshow SW_SHOWMINIMIZED);
		UpdateWindow(win);
	}*/
  
	myglobs.ServerID = DPID_ALLPLAYERS;

	// Initialize the input booleans
	for(int i=0; i < NUM_INPUTS; i++){
		myglobs.input_changed[i] = true;
	}
	myglobs.run = false;
	myglobs.action1 = false;
	myglobs.action2 = false;
	myglobs.jump = false;
	myglobs.strafe = false;
    myglobs.deck_color = 0;

	//Give this piece a unique identifier
	myglobs.nObjectID = 1;

    return win;
}


//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Initialize the application, process messages, and render the scene.
//-----------------------------------------------------------------------------
int RWinMain( HINSTANCE this_inst, HINSTANCE, LPSTR cmdline, int cmdshow )
{
    HWND    hwnd;
    MSG     msg;

	bool done = false;

	bool bRender = false;
	memset(&myglobs, 0, sizeof(myglobs));

	if(strstr( cmdline, "/c"))
		myglobs.type = CONNECT_FOUR;
	else if(strstr( cmdline, "/r"))
		myglobs.type = RUBICS_CUBE;
	else 
		myglobs.type = ChooseType(this_inst, NULL);
	
    // Create the window and initialize objects. 
    if (!(hwnd = InitApp(this_inst, cmdshow)))
        return 1;
	

    while (!done) {  
        // Monitor the message queue and process messages. PeekMessage 
        // returns control to the application immediately so the 
        // application can both process messages and continue rendering. 
        // If the message is WM_QUIT, break out of this function
        // because the application is terminating.
        while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {

            if (msg.message == WM_QUIT) {
                done = true;
				break;
			}
			TranslateMessage(&msg);
			DispatchMessage(&msg);
        }
    
        // If the application is not minimized and the Direct3D Retained 
        // Mode objects are initialized, render.
        bRender = ReceiveMessage(lpDPInfo);
		if (bRender && !myglobs.bMinimized && myglobs.bInitialized) {

            // Render one frame of the scene. If rendering fails, post
            // the WM_DESTROY message, enabling the application to 
            // free resources before terminating the application.
			if (!RenderScene()) {
                DISPLAYMSG("Rendering failed. Aborting execution.");
                PostMessage(NULL, WM_DESTROY, 0, 0);
                break;
            }

        // Yield to other applications if this application is
        // not currently rendering.
        } Sleep(0); /* else 
              WaitMessage();*/

    }
    return msg.wParam;
}


//-----------------------------------------------------------------------------
// Name: WindowProc()
// Desc: Handle messages for the main window.
//-----------------------------------------------------------------------------
LRESULT CALLBACK WindowProc( HWND win, UINT msg, WPARAM wparam, LPARAM lparam )
{
HRESULT rval;
Action act;
int size = sizeof(APPMSG_MOVE);
APPMSG_MOVE appmsg;
LPAPPMSG_MOVE lpMessage = &appmsg;
lpMessage->dwType = DPAPPMSG_MOVE;
lpMessage->idPiece = myglobs.MyID;
float speed;
float ang_speed;

// Choose between walking and running speed
if(myglobs.run){
	speed = (float)3.0f*DEFAULT_SPEED;
	ang_speed = (float)3.0f*DEFAULT_THETA;
}
else{
	speed = (float)DEFAULT_SPEED;
	ang_speed = (float)DEFAULT_THETA;
}
 

switch (msg) {

	case WM_KEYDOWN:	
		if(lparam & 0x400000) {
			switch(wparam) {
				case VK_NUMPAD7:
					wparam = '1';
					break;
				case VK_NUMPAD9:
					wparam = '2';
					break;
				case VK_NUMPAD8:
					wparam = VK_UP;
					break;
				case VK_NUMPAD5:
					wparam = VK_DOWN;
					break;
				case VK_NUMPAD6:
					wparam = VK_RIGHT;
					break;
				case VK_NUMPAD4:
					wparam = VK_LEFT;
					break;
				case VK_NUMPAD3:
					//wparam = VK_INSERT;
					wparam = '.';
					break;
				case VK_NUMPAD1:
					//wparam = VK_DELETE;
					wparam = 'm';
					break;
				case VK_NUMPAD2:
					wparam =  VK_HOME;
					break;
				case VK_SUBTRACT:
					wparam =  '-';
					break;
				case VK_ADD:
					wparam = '+';
					break;
				case VK_INSERT:
					wparam = '.';
					break;
				case VK_DELETE:
					wparam = 'm';
					break;
			}
		}
		switch (wparam) {
		case VK_MENU:
			return true;
		
		//case VK_NUMPAD7:
		case '1':
		case 'u':
		case 'U':
			if(myglobs.input_changed[STRAFE]) {
				myglobs.input_changed[STRAFE] = false;
				myglobs.angle = ang_speed;
				myglobs.strafe = true;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
			break;

		//case VK_NUMPAD9:
		case '2':
		case 'o':
		case 'O':
			if(myglobs.input_changed[STRAFE]) {
				myglobs.input_changed[STRAFE] = false;
				myglobs.angle = -ang_speed;
				myglobs.strafe = true;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
			break;

		case ' ':
		case VK_TAB:
			if(myglobs.input_changed[JUMP]) {
				myglobs.input_changed[JUMP] = false;
				myglobs.jump = true;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
			break;

		case VK_CONTROL:
		case 's':
		case 'S':
			if(myglobs.input_changed[RUN]){
				myglobs.input_changed[RUN] = false;
				myglobs.run = true;

				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				
				//if we're already turning increase rate
				if(myglobs.angle > 0){
					act.theta = (float)3.0f*DEFAULT_THETA;
				}
				else if(myglobs.angle < 0){
					act.theta = (float)3.0f*-DEFAULT_THETA;	
				}
				else{
					act.theta = 0;
				}
				myglobs.angle = act.theta;

				// if we're already moving just increase speed
				if(myglobs.vel > 0){
					act.velocity = (float)3.0f*DEFAULT_SPEED;	
				}
				else if(myglobs.vel < 0){
					act.velocity = (float)3.0f*-DEFAULT_SPEED;
				}
				else{
					act.velocity = 0;
				}
				myglobs.vel = act.velocity;

				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
						DPSEND_GUARANTEED, lpMessage, size);
			}
			break;

		//case ' ':
		case VK_CAPITAL:
		case 'd':
		case 'D':
			if(myglobs.input_changed[ACTION1]) {
				myglobs.input_changed[ACTION1] = false;
				myglobs.action1 = true;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
			break;
		case 'f':
		case 'F':
		case VK_SHIFT:
			if(myglobs.input_changed[ACTION_2]) {
				myglobs.input_changed[ACTION_2] = false;
				myglobs.action2 = true;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
			break;
		
		//case VK_NUMPAD8:
		case VK_UP:
		case 'i':
		case 'I':
			if(myglobs.input_changed[KEY_UP]){
				myglobs.input_changed[KEY_UP] = false;
				myglobs.vel = speed;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
			break;

		//case VK_NUMPAD5:
        case VK_DOWN:
		case 'k':
		case 'K':
			if(myglobs.input_changed[KEY_DOWN])
			{
				myglobs.input_changed[KEY_DOWN] = false;
				myglobs.vel = -speed;
				act.velocity = myglobs.vel;
				act.theta = myglobs.angle;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
            break;
		
		//case VK_NUMPAD6:
        case VK_RIGHT:
		case 'l':
		case 'L':
			if(myglobs.input_changed[KEY_RIGHT])
			{
				myglobs.input_changed[KEY_RIGHT] = false;
				myglobs.angle = -ang_speed;
				act.theta = myglobs.angle;
				act.velocity = myglobs.vel;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
            break;

		//case VK_NUMPAD4:
        case VK_LEFT:
		case 'j':
		case 'J':
			if(myglobs.input_changed[KEY_LEFT]){
				myglobs.input_changed[KEY_LEFT] = false;
				myglobs.angle = ang_speed;
				act.theta = myglobs.angle;
				act.velocity = myglobs.vel;
				act.action1 = myglobs.action1;
				act.action2 = myglobs.action2;
				act.strafe = myglobs.strafe;
				act.jump = myglobs.jump;
				lpMessage->action = act;
				lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			}
            break;
		
		
        //case VK_SUBTRACT:
		case '-':
		case '_':
			if( myglobs.distance <= CLIPPING_DIST && myglobs.player){
				myglobs.distance = myglobs.distance / 0.85;
				ChangeCameraView(myglobs.view_number);
			}
			break;
		
        //case VK_ADD:
		case '=':
		case '+':
			if(myglobs.distance >= 10.0 && myglobs.player){
				myglobs.distance = myglobs.distance * 0.85;
				ChangeCameraView(myglobs.view_number);
			}
            break;
		
		//case VK_NUMPAD3:
		//case VK_INSERT:
		//.
		case 0xbe:
		case '.':
		case '>':
			if(myglobs.vorientation <= 0.65 && myglobs.player){
				myglobs.vorientation += 0.04f;
				ChangeCameraView(myglobs.view_number);
			}
            break;
		
		//case VK_NUMPAD1:
		//case VK_DELETE:
		case 'm':
		case 'M':
			if(myglobs.vorientation >= -0.75 && myglobs.player){
				myglobs.vorientation -= 0.04f;
				ChangeCameraView(myglobs.view_number);
			}
            break;
  
		//case VK_NUMPAD2:
		case VK_HOME:
		case 0xbc:
		case ',':
		case '<':
			myglobs.vorientation = (float)INITIAL_VORIENT;
			myglobs.distance = INITIAL_DISTANCE;
			if(myglobs.type != VIEWER)
				myglobs.view_number = 0;
			else
				myglobs.view_number = 3;
			ChangeCameraView(myglobs.view_number);
			break;
		
		case VK_ESCAPE:
			myglobs.bInitialized = FALSE;
			RELEASE(myglobs.view);
			RELEASE(myglobs.windevice);
			RELEASE(myglobs.dev);
			RELEASE(myglobs.camera);
			RELEASE(myglobs.scene);
			RELEASE(lpD3DRM);
			RELEASE(lpDDClipper);
			myglobs.rubMus->StopMidi();
			PostQuitMessage(0);

			break;
		
		case VK_F1:
			if(myglobs.player)
				myglobs.view_number = (myglobs.view_number + 1) % (NUM_VIEWS);
				ChangeCameraView(myglobs.view_number);
			break;
		
		}
		break;
	
		case WM_KEYUP:
			if(lparam & 0x400000) {
			switch(wparam) {
				case VK_NUMPAD7:
					wparam = '1';
					break;
				case VK_NUMPAD9:
					wparam = '2';
					break;
				case VK_NUMPAD8:
					wparam = VK_UP;
					break;
				case VK_NUMPAD5:
					wparam = VK_DOWN;
					break;
				case VK_NUMPAD6:
					wparam = VK_RIGHT;
					break;
				case VK_NUMPAD4:
					wparam = VK_LEFT;
					break;
				case VK_NUMPAD3:
					wparam = VK_INSERT;
					break;
				case VK_NUMPAD1:
					wparam = VK_DELETE;
					break;
				case VK_NUMPAD2:
					wparam =  VK_HOME;
					break;
				
				case VK_MENU:
						return true;
						
				case VK_F2: // Decrease Rolloff Factor
					if(myglobs.pSoundSet)
						myglobs.pSoundSet->ModifyListenerRolloff( (D3DVALUE) 1.5 );
					break;

				case VK_F3: // Increase Rolloff Factor
					if(myglobs.pSoundSet)
						myglobs.pSoundSet->ModifyListenerRolloff( (D3DVALUE) .75 );
					break;

				case VK_F4:
				case VK_F12: // toggle the background music
					if(myglobs.rubMus)
						myglobs.rubMus->NextMidi();
					break;
				case VK_F11:
					if(myglobs.rubMus)
						myglobs.rubMus->StopMidi();
					break;
			}
		}
		switch (wparam) {
		//case VK_NUMPAD7:
		case 'u':
		case 'U':
		case '1':
			myglobs.input_changed[STRAFE] = true;
			myglobs.strafe = false;
			myglobs.angle = 0;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;
		  
		  //case VK_NUMPAD9:
		  case '2':
		  case 'o':
		  case 'O':
			myglobs.input_changed[STRAFE] = true;
			myglobs.strafe = false;
			myglobs.angle = 0;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;
		
		case ' ':
		case VK_TAB:
			myglobs.input_changed[JUMP] = true;
			myglobs.jump = false;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;

		case VK_CONTROL:
		case 's':
		case 'S':
			myglobs.input_changed[RUN] = true;
			myglobs.run = false;

			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;

			//if we're already turning decrease rate
			if(myglobs.angle > 0){
				act.theta = (float)DEFAULT_THETA;
			}
			else if(myglobs.angle < 0){
				act.theta = (float)-DEFAULT_THETA;
			}
			else{
				act.theta = 0;
			}
			myglobs.angle = act.theta;

			// if we're already moving just decrease speed
			if(myglobs.vel > 0){
				act.velocity = (float)DEFAULT_SPEED;
			}
			else if(myglobs.vel < 0){
				act.velocity = (float)-DEFAULT_SPEED;
			}
			else{
				act.velocity = 0;
			}
			myglobs.vel = act.velocity;

			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
					DPSEND_GUARANTEED, lpMessage, size);
			break;

		//case ' ':
		case VK_CAPITAL:
		case 'd':
		case 'D':
			myglobs.input_changed[ACTION1] = true;
			myglobs.action1 = false;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;
			
		case 'f':
		case 'F':
		case VK_SHIFT:
			myglobs.input_changed[ACTION_2] = true;
			myglobs.action2 = false;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;
		
		//case VK_NUMPAD8:
		case VK_UP:
		case 'i':
		case 'I':
			myglobs.input_changed[KEY_UP] = true;
			myglobs.vel = 0;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;

		//case VK_NUMPAD5:
        case VK_DOWN:
		case 'k':
		case 'K':
			myglobs.input_changed[KEY_DOWN] = true;
			myglobs.vel = 0;
			act.velocity = myglobs.vel;
			act.theta = myglobs.angle;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;

		//case VK_NUMPAD6:
        case VK_RIGHT:
		case 'l':
		case 'L':
			myglobs.input_changed[KEY_RIGHT] = true;
			myglobs.angle = 0;
			act.theta = myglobs.angle;
			act.velocity = myglobs.vel;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;

		//case VK_NUMPAD4:
        case VK_LEFT:
		case 'j':
		case 'J':
			myglobs.input_changed[KEY_LEFT] = true;
			myglobs.angle = 0;
			act.theta = myglobs.angle;
			act.velocity = myglobs.vel;
			act.action1 = myglobs.action1;
			act.action2 = myglobs.action2;
			act.strafe = myglobs.strafe;
			act.jump = myglobs.jump;
			lpMessage->action = act;
			lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
				DPSEND_GUARANTEED, lpMessage, size);
            break;

		}
		break;

    case WM_SIZE:
        // Handle resizing of the window. If the application was 
        // minimized, set a flag so rendering will stop while minimized
        // and break out of this function. Otherwise, make sure the
        // minimized flag is clear and check for window resizing other
        // than minimized.
        {
        if (SIZE_MINIMIZED == wparam) 
        {
            myglobs.bMinimized = TRUE;
            break;
        }
        else
            myglobs.bMinimized = FALSE;

        // Handle resizing of the window. Obtain the window's current
        // width and height.
        int width = LOWORD(lparam);
        int height = HIWORD(lparam);

        // If the global variables such as the viewport and device 
        // have been initialized, then obtain the viewport's and
        // device's width and height. Create a new device and window
        // device or viewport as needed. A new device and window device
        // are only needed if the window is larger than the old device.
        if (myglobs.bInitialized && !myglobs.fullscreen) 
        {
            int view_width = myglobs.view->GetWidth();
            int view_height = myglobs.view->GetHeight();
            int dev_width = myglobs.dev->GetWidth();
            int dev_height = myglobs.dev->GetHeight();

            // Check whether the window size increased in either
            // width or height.
            if (width > dev_width || height > dev_height)
            {
                // Application window is now larger than the current  
                // device. Destroy the old viewport and device and create 
                // new ones. Be sure to free the viewport before freeing
                // the device or a general protection fault might occur.
                myglobs.bInitialized = FALSE;
                RELEASE(myglobs.view);
                RELEASE(myglobs.dev);

                // Create a new default Direct3DRM device.
                rval = lpD3DRM->CreateDeviceFromClipper(lpDDClipper, 
                                        NULL, // Default device 
                                        width, height, 
                                        &myglobs.dev);

                if (FAILED(rval)) 
                {
                    // If the new device could not be created, notify 
                    // the user and shut down the application.
                    DISPLAYMSG("Failed to create the D3DRM device.");
                    PostMessage(NULL, WM_DESTROY, 0, 0);
                    return FALSE;
                } 

                // Destroy the old Direct3D Retained Mode window device 
                // and create a new one to match the newly created 
                // device. The window device handles the WM_ACTIVATE and 
                // WM_PAINT messages.
                RELEASE(myglobs.windevice);
                rval = myglobs.dev->QueryInterface(
                                       IID_IDirect3DRMWinDevice,
                                       (void **) &myglobs.windevice);
                if (FAILED(rval)) {
                    DISPLAYMSG("Failed to create the window device.");
                    return FALSE;
                }

            }

            // If the window sized changed, free the old viewport and
            // create a new viewport of the proper size.
            myglobs.bInitialized = FALSE;
            RELEASE(myglobs.view);
            rval = lpD3DRM->CreateViewport(myglobs.dev, 
                                           myglobs.camera,
                                           0, 0, width, height,
                                           &myglobs.view);
            if (FAILED(rval)) 
            {
                // If the new viewport could not be created, notify 
                // the user and shut down the application.
                DISPLAYMSG("Failed to create a new viewport.");
                PostMessage(NULL, WM_DESTROY, 0, 0);
                break;
            }

			// Set our new clipping distance
			if (FAILED(myglobs.view->SetBack(D3DVAL(CLIPPING_DIST)))){
				myglobs.bInitialized = FALSE;
				RELEASE(myglobs.dev);
				return FALSE;
			}

            myglobs.bInitialized = TRUE;
        }
        }
        break;

    case WM_ACTIVATE:
        // Use the window device's HandleActivate method to handle 
        // window activation.
        if (myglobs.bInitialized)
            if (FAILED(myglobs.windevice->HandleActivate(wparam)))
                DISPLAYMSG("Failed to handle WM_ACTIVATE.");
        break;

    case WM_PAINT:
        // Use the window device's HandlePaint method to handle 
        // repainting unless global variables have not yet been 
        // initialized. 
        if (!myglobs.bInitialized)
            return DefWindowProc(win, msg, wparam, lparam);

        // Use the window device to handle repainting.
        RECT rc;
        PAINTSTRUCT ps;

        if (GetUpdateRect(win, &rc, FALSE)) {
            BeginPaint(win, &ps);
            if (FAILED(myglobs.windevice->HandlePaint(ps.hdc)))
                DISPLAYMSG("Failed to handle WM_PAINT.");
        } 
        EndPaint(win, &ps);
        break;

    case WM_DESTROY:
        // Clear the bInitialized flag, free objects, and post the WM_QUIT
        // message to terminate the application. Be sure to free the 
        // viewport before freeing the device or a general protection
        // fault might occur.
        myglobs.bInitialized = FALSE;
        RELEASE(myglobs.view);
        RELEASE(myglobs.windevice);
        RELEASE(myglobs.dev);
        RELEASE(myglobs.camera);
        RELEASE(myglobs.scene);
        RELEASE(lpD3DRM);
        RELEASE(lpDDClipper);
        PostQuitMessage(0);
        break;

    default:
        return DefWindowProc(win, msg, wparam, lparam);
    }
    return 0L;
}


//-----------------------------------------------------------------------------
// Name: ReceiveMessage(LPDPLAYINFO lpDPInfo)
// Desc: Handle Dplay messages.
// return code: returns true if the screen needs to be updated.
//-----------------------------------------------------------------------------
bool ReceiveMessage(LPDPLAYINFO lpDPInfo)
{
	DPID				idFrom, idTo;
	LPVOID				lpvMsgBuffer;
	DWORD				dwMsgBufferSize;
	HRESULT				hr;

	bool bRender =false;

	lpvMsgBuffer = NULL;
	dwMsgBufferSize = 0;

	// loop to read all messages in queue
	do
	{
		// loop until a single message is successfully read
		do
		{
			// read messages from any player, including system player
			idFrom = 0;
			idTo = 0;

			hr = lpDPInfo->lpDirectPlay4A->Receive(&idFrom, &idTo, DPRECEIVE_ALL,
												   lpvMsgBuffer, &dwMsgBufferSize);

			// not enough room, so resize buffer
			if (hr == DPERR_BUFFERTOOSMALL)
			{
				if (lpvMsgBuffer)
					GlobalFreePtr(lpvMsgBuffer);
				lpvMsgBuffer = GlobalAllocPtr(GHND, dwMsgBufferSize);
				if (lpvMsgBuffer == NULL)
					hr = DPERR_OUTOFMEMORY;
			}
		} while (hr == DPERR_BUFFERTOOSMALL);

		if ((SUCCEEDED(hr)) &&							// successfully read a message
			(dwMsgBufferSize >= sizeof(DPMSG_GENERIC)))	// and it is big enough
		{
			// check for system message
			if (idFrom == DPID_SYSMSG)
			{
				HandleSystemMessage(lpDPInfo, (LPDPMSG_GENERIC) lpvMsgBuffer,
									dwMsgBufferSize, idFrom, idTo);
			}
			else
			{
				bRender = bRender | HandleApplicationMessage(lpDPInfo, (LPDPMSG_GENERIC) lpvMsgBuffer,
										 dwMsgBufferSize, idFrom, idTo);
			}
		}
	} while (SUCCEEDED(hr));

	// free any memory we created
	if (lpvMsgBuffer)
		GlobalFreePtr(lpvMsgBuffer);

	return (bRender);
}


bool HandleApplicationMessage(LPDPLAYINFO lpDPInfo, LPDPMSG_GENERIC lpMsg, DWORD dwMsgSize,
							  DPID idFrom, DPID idTo)
{
	LPSTR	lpszStr = NULL;
	HRESULT	hr;

	bool bRender = false;
	switch (lpMsg->dwType)
	{
	case APPMSG_CHATSTRING:
		{
			LPMSG_CHATSTRING   lp = (LPMSG_CHATSTRING) lpMsg;

			// create string to display
			hr = NewChatString(lpDPInfo->lpDirectPlay4A, idFrom, lp->szMsg, &lpszStr);
			if FAILED(hr)
				break;
		}
		if (lpszStr)
		{
			// make sure window is still valid
			if (ghChatWnd)
				PostMessage(ghChatWnd, WM_USER_ADDSTRING, (WPARAM) 0, (LPARAM) lpszStr);
			else
				GlobalFreePtr(lpszStr);
		}
		break;
	case DPSVRMSG_ADDPIECE:
		{
			myglobs.ServerID = idFrom;
			LPSVRMSG_ADDPIECE   lp = (LPSVRMSG_ADDPIECE) lpMsg;
			((LPSVRMSG_ADDPIECE)lpMsg)->typePiece;
			NewPiece(lp->idPiece, lp->typePiece, lp->pos, lp->theta_xz, lp->centerSquare, lp->team, lp->scale, lp->name);
			
		}
		break;
	case DPSVRMSG_REMOVEPIECE:
		{
			LPSVRMSG_REMOVEPIECE   lp = (LPSVRMSG_REMOVEPIECE) lpMsg;
			RemovePiece(lp->idPiece, lp->typePiece);
			
		}
		break;
	case DPSVRMSG_SET_STATE:
		{
			LPSVRMSG_SET_STATE   lp = (LPSVRMSG_SET_STATE) lpMsg;

			UpdateState(lp->NumPiece, lp->objects);
			bRender = true;
			
		}
		break;
	case DPSVRMSG_ADDPLAYER:
		{
			LPSVRMSG_ADDPLAYER   lp = (LPSVRMSG_ADDPLAYER) lpMsg;
			AddPlayer(lp->PlayerID, lp->name);
		}
		break;
	case DPSVRMSG_PICKEDUPPIECE:
		{
			LPSVRMSG_PICKEDUPPIECE lp = (LPSVRMSG_PICKEDUPPIECE) lpMsg;
			PickupPiece(lp->idPiece, lp->color);
			
		}
		break;
	case DPSVRMSG_DROPPIECE:
		{
			LPSVRMSG_DROPPIECE lp = (LPSVRMSG_DROPPIECE) lpMsg;
			DropPiece(lp->idPiece, lp->color);
			myglobs.rubOverlay->SetMessageIndex(lp->attacker_name, lp->loser_name, lp->msg_index);
			
		}
		break;
	case DPSVRMSG_WINNER:
		{
			LPSVRMSG_WINNER   lp = (LPSVRMSG_WINNER) lpMsg;
			//Need To add some function here!!! -Grant
			myglobs.rubOverlay->InitEndingOverlay(lp->idPiece,lp->team,lp->typePiece,TRUE);
			myglobs.pSoundSet->Play(FIRE_WORKS);
		}
		break;
	case DPSVRMSG_RESETGAME:
		{
			//LPSVRMSG_RESETGAME lp = (LPSVRMSG_RESETGAME) lpMsg;

			ResetGame();
			
		}
		break;
	}

	return bRender;	
}

void AttachCamera(CClientObjects *pObject) {
	LPDIRECT3DRMFRAME3 piece;
	myglobs.rubOverlay->drawEndingOverlay = FALSE;
	pObject->GetFrame( &piece );
	AttachCamera(piece);
	((CClientPlayer*)pObject)->SetHero();
}

void AttachCamera(LPDIRECT3DRMFRAME3 piece) {	
	RELEASE(myglobs.camera);
	
	// Set bool in player to signify that this player is the Hero for this particular client
	// Used for updating the listener

	HRESULT rval = lpD3DRM->CreateFrame(piece, &myglobs.camera);
	if (FAILED(rval)) {
        DISPLAYMSG("Failed to create the camera's frame.");
		return;
    }
	
    // Create the Direct3DRM viewport using the device, camera frame,
    // and the device's width and height.
    if (FAILED(myglobs.camera->SetPosition(piece, -D3DVAL(myglobs.distance), D3DVAL(myglobs.distance), 
                                   D3DVAL(0))))
        return;
    
	if (FAILED(myglobs.camera->LookAt(piece, myglobs.scene, D3DRMCONSTRAIN_Z)))
        return;
	
	int width = myglobs.dev->GetWidth();
    int height = myglobs.dev->GetHeight();

    rval = lpD3DRM->CreateViewport(myglobs.dev, myglobs.camera, 0, 0, 
		width, height, &myglobs.view);
    
	if (FAILED(rval)) {
        myglobs.bInitialized = FALSE;
        RELEASE(myglobs.dev);
        return;
    }

	// Set our new clipping distance
	if (FAILED(myglobs.view->SetBack(D3DVAL(CLIPPING_DIST)))){
		myglobs.bInitialized = FALSE;
		RELEASE(myglobs.dev);
		return;
	}

	// Set default player view;
	if(myglobs.type != VIEWER)
		myglobs.view_number = 0;
	else
		myglobs.view_number = 3;
	ChangeCameraView(myglobs.view_number);
}


void ChangeCameraView(int view_number) {
	switch(view_number){
		//camera angles
		// 3/4 view that Aaron likes
		case 3:
		  myglobs.camera->SetPosition(myglobs.player, -D3DVAL(myglobs.distance), D3DVAL(myglobs.distance), 
								D3DVAL(0));
		  myglobs.camera->SetOrientation(myglobs.player, D3DVAL(1), D3DVAL(myglobs.vorientation - 0.75f), D3DVAL(0),
								D3DVAL(0), D3DVAL(1.0), D3DVAL(0));
		break;

		//1st person view
		case 1:
		  myglobs.camera->SetPosition(myglobs.player, D3DVAL(3.8), D3DVAL(1.4), 
								D3DVAL(0.0));
		  myglobs.camera->SetOrientation(myglobs.player, D3DVAL(1.0), D3DVAL(myglobs.vorientation), D3DVAL(0.0),
								D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0));
		break;

		//Overhead
		case 2:
		  myglobs.camera->SetPosition(myglobs.player, D3DVAL(0), D3DVAL(myglobs.distance), 
								D3DVAL(0));
		  myglobs.camera->SetOrientation(myglobs.player, D3DVAL(0), -D3DVAL(1), D3DVAL(0),
								D3DVAL(1), D3DVAL(1), D3DVAL(0));
		break;

		//Default view, Main chaser
		case 0:
		  myglobs.camera->SetPosition(myglobs.player, -D3DVAL(40), D3DVAL(20), 
								D3DVAL(0));
		  //myglobs.camera->SetOrientation(myglobs.player, D3DVAL(1.0), -D3DVAL(0.25), D3DVAL(0),
		  myglobs.camera->SetOrientation(myglobs.player, D3DVAL(1.0), D3DVAL(myglobs.vorientation), D3DVAL(0),
								D3DVAL(0), D3DVAL(1.0), D3DVAL(0));
		break;

		//Right Side
		case 4:
		  myglobs.camera->SetPosition(myglobs.player, D3DVAL(0), D3DVAL(5), 
								-D3DVAL(9));
		  myglobs.camera->SetOrientation(myglobs.player, D3DVAL(0), -D3DVAL(0.35), D3DVAL(1.0),
								D3DVAL(0), D3DVAL(1.0), D3DVAL(0));
		break;
	}

	return;
}


void NewPiece (PieceID idPiece, PieceType type, Position pos, float orientation, BOOL centerSquare[6], int team, float scale, char playername[] ) {

	CClientObjects *pBlock = NULL;
	CClientObjectsNonMovable *pObject = NULL;
	CClientObjects *pIconObject = NULL;
	LPDIRECT3DRMFRAME3 pFrame = NULL;
	

	switch(type) {
		case VIEWER:
			pBlock = (CClientObjects*)new CClientViewer(lpD3DRM, myglobs.board, &pos,orientation, myglobs.pSoundSet );

			if(idPiece == myglobs.MyID) {
				
				pBlock->GetFrame(&myglobs.player);
				AttachCamera(pBlock);
			}

			((CClientPlayer*)pBlock)->SetTeam(3);
			break;
		case RUBICS_CUBE:
			//Type casting here is not quite perfect.
			//Attach camera to player.  We DO NOT need to do this for every player piece
			if(idPiece == myglobs.MyID) 
			{
				pBlock = (CClientObjects*)new RubiksCube(lpD3DRM, myglobs.board, 
														 centerSquare, playername,
													     myglobs.pSoundSet, TRUE);
				pBlock->GetFrame(&myglobs.player);
				AttachCamera(pBlock);
			}
			else
			{
				pBlock = (CClientObjects*)new RubiksCube(lpD3DRM, myglobs.board, 
														 centerSquare, playername,
													     myglobs.pSoundSet, FALSE);
			}

			/* Working on code to create shadow HERE 
			((RubiksCube*)pBlock)->GetFrame(&pFrame);
			assert(SUCCEEDED(lpD3DRM->CreateShadow(pFrame, myglobs.shadowLight, 0.0f, 0.1f, 0.0f, 
															   0.0f, 1.0f, 0.0f, &shadow)));
			assert(SUCCEEDED(myglobs.board->AddVisual(shadow)));
			//RELEASE(shadow);
			//RELEASE(pFrame); */

			((CClientPlayer*)pBlock)->SetTeam(team);
			break;
		case CONNECT_FOUR:
			//Type casting here is not quite perfect.
			if(idPiece == myglobs.MyID) 
			{
				/* Make us the hero */
				pBlock = (CClientObjects*)new ConnectFour(lpD3DRM, myglobs.board, centerSquare, playername,
													 myglobs.pSoundSet, TRUE);

				pBlock->GetFrame(&myglobs.player);
				AttachCamera(pBlock);
			}
			else
			{
				pBlock = (CClientObjects*)new ConnectFour(lpD3DRM, myglobs.board, centerSquare, playername,
							 							  myglobs.pSoundSet, FALSE);
			}
			
			/* Working on code to create shadow HERE 
			((RubiksCube*)pBlock)->GetFrame(&pFrame);
			assert(SUCCEEDED(lpD3DRM->CreateShadow(pFrame, myglobs.shadowLight, 0.0f, 0.1f, 0.0f, 
															   0.0f, 1.0f, 0.0f, &shadow)));
			assert(SUCCEEDED(myglobs.board->AddVisual(shadow)));
			//RELEASE(shadow);
			//RELEASE(pFrame); */
			//Attach camera to player.  We DO NOT need to do this for every player piece
			
			((CClientPlayer*)pBlock)->SetTeam(team);
			break;

		case ALPHA_BLOCK_A:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( ABLOCK, lpD3DRM, myglobs.board, pos, orientation, scale*16.0f );
			break;

		case ALPHA_BLOCK_B:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( BBLOCK, lpD3DRM, myglobs.board, pos, orientation, scale*16.0f );
			break;

		case ALPHA_BLOCK_C:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( CBLOCK, lpD3DRM, myglobs.board, pos, orientation, scale*16.0 );
			break;

		case POWER_BOX:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( PBOX, lpD3DRM, myglobs.board, pos, orientation, 4.0 );
			break;

		case WALL:
			pObject = (CClientObjectsNonMovable*)new RubicsWall( lpD3DRM, myglobs.board, pos, orientation, WALL_HEIGHT, BOARD_X, "Wall.bmp" );
			break;

		case CLOCK:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "clock.x", myglobs.pSoundSet );
			pObject->ScaleMe(150.0f, 150.0f, 150.0f);
			break;

		case CRAYON:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "crayon2.x", myglobs.pSoundSet );
			pObject->ScaleMe(150.0f, 150.0f, 150.0f);
			break;

		case BALLOON:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "balloons.x", myglobs.pSoundSet );
			pObject->ScaleMe(3.0f, 3.0f, 3.0f);
			break;

		case PEZ:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "pez.x", myglobs.pSoundSet );
			pObject->ScaleMe(4.0f, 4.0f, 4.0f);
			break;

		case TOWER:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "tower.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.05f, 0.05f, 0.05f);
			break;

		case CHAIR:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "chair03.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.05f, 0.05f, 0.05f);
			break;

		case LEGO_GUY:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "legoman.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.2f, 0.2f, 0.2f);
			break;

		case AIR_PLANE:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "airplane.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.01f, 0.01f, 0.01f);
			break;

		case ENGINE:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "engine.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.05f, 0.05f, 0.05f);
			break;

		case DART_BOARD:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "dartbord.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.05f, 0.05f, 0.05f);
			break;

		case DECK:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( GDECK, lpD3DRM, myglobs.board, pos, orientation, scale*16.0f, (myglobs.deck_color%4) );
			myglobs.deck_color ++;
			break;

		case DOMINO:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( DOMINOBLOCK, lpD3DRM, myglobs.board, pos, orientation, scale*16.0f );
			break;

		case DICE:
			pObject = (CClientObjectsNonMovable*)new AlphabetBlock( DIE, lpD3DRM, myglobs.board, pos, orientation, scale*16.0f );
			break;

		case POOL_TABLE:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "pptable.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.02f, 0.02f, 0.02f);
			break;

		/*case HORSE:
			pObject = (CClientObjectsNonMovable*)new CClientObjectsNonMovable( lpD3DRM, myglobs.board, pos, orientation, scale, (LPVOID) "rockhrse.x", myglobs.pSoundSet );
			pObject->ScaleMe(0.05f, 0.05f, 0.05f);
			break;*/

		/***** Icons *****/
		case MISSILE:
			pBlock = (CClientObjects*)new CIcons( lpD3DRM, myglobs.board, pos, 
				                                  orientation, MISSILE, 1 );
			break;
		case MULTI_SHOT:
			pBlock = (CClientObjects*)new CIcons( lpD3DRM, myglobs.board, pos, 
				                                  orientation, MULTI_SHOT, 1 );
			break;
		case RICOCHETE:
			pBlock = (CClientObjects*)new CIcons( lpD3DRM, myglobs.board, pos, 
				                                  orientation, RICOCHETE, 1 );
			break;
		case RAM_OBJECT:
			pBlock = (CClientObjects*)new CIcons( lpD3DRM, myglobs.board, pos, 
				                                  orientation, RAM_OBJECT, 1 );
			break;
		case TORNADO_OBJECT:
			pBlock = (CClientObjects*)new CIcons( lpD3DRM, myglobs.board, pos, 
				                                  orientation, TORNADO_OBJECT, 1 );
			break;
		case ARCING_OBJECT:
			pBlock = (CClientObjects*)new CIcons( lpD3DRM, myglobs.board, pos, 
				                                  orientation, ARCING_OBJECT, 1 );
			break;

		case CUBE_PROJECTILE:
			pBlock = (CClientObjects*) new CClientProjectile( lpD3DRM, myglobs.board, &pos, myglobs.pSoundSet );
			//pBlock = (CClientObjects*) new CClientProjectile( lpD3DRM, myglobs.board, &pos);
			break;

		case HOMING_PROJ:
			pBlock = (CClientObjects*) new CConnectProjectile( lpD3DRM, myglobs.board, &pos, myglobs.pSoundSet, 0.1f, RED );
			break;

		case CONNECT_PROJECTILE:
			pBlock = (CClientObjects*) new CConnectProjectile( lpD3DRM, myglobs.board, &pos, myglobs.pSoundSet );
			break;

		case CONECT_MORTAR_PROJ:
		case MORTAR_PROJ:
			pBlock = (CClientObjects*) new CClientMortar( lpD3DRM, myglobs.board, &pos, type, myglobs.pSoundSet );
			break;
		
		case RED_RUBIKS_PIECE:
		case BLUE_RUBIKS_PIECE:
		case GREEN_RUBIKS_PIECE:
		case YELLOW_RUBIKS_PIECE:
		case WHITE_RUBIKS_PIECE:
		case ORANGE_RUBIKS_PIECE:
			pBlock = (CClientObjects*) new CClientRubiksPiece( lpD3DRM, myglobs.board, &pos, orientation, type, myglobs.type);
			break;

		default:
			// don't render anything!  Needed for our invisible bounding box object.
			break;
	}

	//If we're a player set our team value
	//if(pBlock != NULL && pBlock->IsPlayer() && )
	//	((CClientPlayer*)pBlock)->SetTeam(team);
	//Only add this if it's a moveable object: players, icons, weapons.
	if (pBlock != NULL) mapPieces.insert(ObjectMap::value_type(idPiece, pBlock));

	return;
}


void RemovePiece(PieceID idPiece, PieceType type) {
	ObjectMap::iterator it;
	LPDIRECT3DRMFRAME3 lpParent = NULL;
	LPDIRECT3DRMFRAME3 lppiece = NULL;

	it = mapPieces.find(idPiece);
	//_ASSERT(it!=mapPieces.end());
	if(it!=mapPieces.end()) {
		// Play sound if we picked up an icon!
		/*if(type == BLUE_RUBIKS_PIECE)
			((CClientRubiksPiece*)it->second)->RemoveSound();*/

		it->second->GetFrame(&lppiece);
		
		// I'm always gonna attempt to delete from the map if it passes the assert above!
		lppiece->GetParent(&lpParent);
		_ASSERT(lpParent != NULL);
		if(lpParent != NULL)
		{
			lpParent->DeleteChild( (lppiece));
		}
		
		if( it->second->IsProjectile() )
		  delete (CClientProjectile*)(it->second); // This will call the projectile's destructor
		else
		  delete it->second;
		
		mapPieces.erase(it);
		RELEASE(lppiece);
	}
	return;
}
void UpdateObjects() {
	ObjectMap::iterator it;

	for(it =mapPieces.begin(); it !=mapPieces.end(); it++)	 {
		it->second->Update();
	}
}

void UpdateState(	int NumPiece, APP_PIECE objects[] ) {
	ObjectMap::iterator it;
	// unreferenced! LPDIRECT3DRMFRAME3 f;

	for( int i =0; i <NumPiece; i++)	 {
		it = mapPieces.find(objects[i].piece);
		//_ASSERT(it != mapPieces.end() );
		if(it != mapPieces.end()){
		it->second->ReceiveMessage(&(objects[i].info));
		
		//This function plays the sound corresponding to the animations
		it->second->DoSound();
		//Put the Update here for now.  Needs to be moved to the render loop.
		
		/*(it->second)->GetFrame(&f);
		f->SetPosition(myglobs.board, objects[i].pos.x, objects[i].pos.y,
			objects[i].pos.z);
		
		f->SetOrientation(myglobs.board, (cos(objects[i].dir.theta_xz)), 
										  0,
										  (sin(objects[i].dir.theta_xz)), 0, 1, 0);*/
		}
	}
}

void PickupPiece(PieceID id, PieceType piece) {
	ObjectMap::iterator it;
	
	it = mapPieces.find(id);
	_ASSERT(it != mapPieces.end() );
	if(it != mapPieces.end() )
		((CClientPlayer *) it->second)->PickupPiece(piece);
}

void DropPiece(PieceID id, PieceType piece) {
	ObjectMap::iterator it;
	
	it = mapPieces.find(id);
	_ASSERT(it != mapPieces.end() );
	if(it != mapPieces.end() )
		((CClientPlayer *) it->second)->DropPiece(piece);
}
void AddPlayer(short id, char playername[]) {
	myglobs.PlayerID = id;
	SendAddPlayerMsg(playername);
}

void SendAddPlayerMsg(char playername[]) {
	int size = sizeof(APPMSG_ADDPIECE);
	APPMSG_ADDPIECE appmsg;
	LPAPPMSG_ADDPIECE lpMessage = &appmsg;
	lpMessage->dwType = DPAPPMSG_ADDPIECE;

	// Create a unique ID for this piece
	lpMessage->idPiece = (PIECE_ID(myglobs.PlayerID, myglobs.nObjectID));

	//increment the unique ID
	myglobs.nObjectID++;
	myglobs.MyID = lpMessage->idPiece;
	/*char buffer[256];
	sprintf(buffer, "%d", myglobs.MyID);
	DISPLAYMSG(buffer);*/

	lpMessage->typePiece = myglobs.type;
	lpMessage->team = myglobs.team;
	for(int i=0; i<10; i++)
		lpMessage->name[i] = playername[i];
	
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
		DPSEND_GUARANTEED, lpMessage, size);
	_ASSERT(!FAILED(hr));
}

PieceType ChooseType(HINSTANCE hInstance, HWND win) {
	PieceType type = RUBICS_CUBE;
	DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_SETUP_GAME),
		win, (DLGPROC) SetupGameWndProc, (LPARAM) &type );
	return type;
}

void ResetGame() {
	HRESULT hr;
	LPDIRECT3DRMFRAME scene = NULL;
	ObjectMap::iterator it;
	APPMSG_NEWGAMEPLEASE msg;
	LPAPPMSG_NEWGAMEPLEASE lpMessage = &msg;

	it = mapPieces.find(myglobs.MyID);
	//Try to get my name from myself out of the map.
	if(it != mapPieces.end())
		for(int i=0; i<10; i++)
			lpMessage->name[i] = ((CClientPlayer*)it->second)->name[i];

	hr = myglobs.scene->QueryInterface(IID_IDirect3DRMFrame, (void **)&scene);
	if(FAILED(hr))
		return;

	DeleteChildren(scene);

	for(it = mapPieces.begin(); it != mapPieces.end(); it++) {
		delete(it->second);
	}
	mapPieces.clear();
	// Reset our unique piece ID number
	myglobs.nObjectID = 1;

	//AttachCamera(myglobs.scene);
	if(!BuildScene(lpD3DRM, myglobs.dev, myglobs.scene, myglobs.camera))
		DISPLAYMSG("Failed to BuildScene");

	//Given our architecture we can't just send an addpiece message
	//because before we rely on a player adding via DirectPlay message...
	//Please ask Grant if you have questions about this!!!
	lpMessage->dwType = DPAPPMSG_NEWGAMEPLEASE;
	lpMessage->idPiece = myglobs.MyID;
	
	hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, myglobs.ServerID,
		DPSEND_GUARANTEED, lpMessage, sizeof(APPMSG_NEWGAMEPLEASE));
	_ASSERT(!FAILED(hr));
	return;
}

void DeleteChildren(LPDIRECT3DRMFRAME parent) {
	HRESULT hr;
	LPDIRECT3DRMFRAMEARRAY children = NULL;
	LPDIRECT3DRMFRAME frame;
	hr = parent->GetChildren(&children);
	if(FAILED(hr))
		return;
	for(int i = 0; i < children->GetSize(); i++) {
		children->GetElement(i, &frame);
		DeleteChildren(frame);
		parent->DeleteChild(frame);
		RELEASE(frame);
	}
	RELEASE(children);
}

BOOL CALLBACK SetupGameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static PieceType* lpType ;
	switch(uMsg)
	{
	case WM_INITDIALOG:
	// save the connection info pointer
		lpType = (PieceType *) lParam;
		
		SendDlgItemMessage(hWnd, IDC_RUBIKS_CUBE, BM_SETCHECK, (WPARAM) BST_CHECKED, (LPARAM) 0);
		SendDlgItemMessage(hWnd, IDC_CONNECT, BM_SETCHECK, (WPARAM) BST_UNCHECKED, (LPARAM) 0);
		SendDlgItemMessage(hWnd, IDC_VIEWER, BM_SETCHECK, (WPARAM) BST_UNCHECKED, (LPARAM) 0);
		SendDlgItemMessage(hWnd, IDC_TEAM1, BM_SETCHECK, (WPARAM) BST_CHECKED, (LPARAM) 0);
		SendDlgItemMessage(hWnd, IDC_TEAM2, BM_SETCHECK, (WPARAM) BST_UNCHECKED, (LPARAM) 0);
		break;

	case WM_DESTROY:
		break;

	case WM_COMMAND:
	switch(LOWORD(wParam))
	{
		case IDOK:
		  if(BST_CHECKED == SendDlgItemMessage(hWnd, IDC_TEAM1, BM_GETCHECK, (WPARAM) 0, (LPARAM) 0))
			myglobs.team = 1;
		  else 
			myglobs.team = 2;

		  if(BST_CHECKED == SendDlgItemMessage(hWnd, IDC_VIEWER, BM_GETCHECK, (WPARAM) 0, (LPARAM) 0)){
			*lpType =  VIEWER;
			myglobs.team = 3;
		  }
		  else if(BST_CHECKED == SendDlgItemMessage(hWnd, IDC_CONNECT, BM_GETCHECK, (WPARAM) 0, (LPARAM) 0))
			*lpType =  CONNECT_FOUR;
		  else 
			*lpType =  RUBICS_CUBE;

		  EndDialog(hWnd, TRUE);
		break;

	case IDCANCEL:
			EndDialog(hWnd, FALSE);
		break;
	}

	break;
	}

	// Allow for default processing
	return FALSE;
}

LPDIRECT3DRMFRAME3 GetCamera()
{
	return myglobs.camera;
}

LPDIRECT3DRMLIGHT GetShadowLight()
{
	return myglobs.shadowLight;
}

PieceType GetPlayerType()
{
	return myglobs.type;
}
