 //-----------------------------------------------------------------------------
// File: RubicsObjects.cpp
//
// Desc: This file creates the object used in the Game
//
// 4-1-1999:	Aaron:  Created file and two basic objects.
//-----------------------------------------------------------------------------

#pragma warning( disable : 4244) 

#include "RubicsObjects.h"
#include "RubicsDirect3D.h"
#include <math.h>
//-----------------------------------------------------------------------------
// Name: GetBoard()
// Desc: Returns the board as a MeshBuilder.
//-----------------------------------------------------------------------------
HRESULT GetBoard(LPDIRECT3DRMMESHBUILDER3 board_builder) {
	
	int nHeight;
	int nWidth;
	double dGroundXFactor;	
	double dGroundYFactor;	
	double dWallXFactor;
	double dWallYFactor;
	double dScaleFactor = 1.0;
	
	HRESULT hr = S_OK;

	LPDIRECT3DRMFACE2 face = NULL;
    LPDIRECT3DRMMATERIAL2 mat = NULL;
	LPDIRECT3DRMTEXTURE3 pWallTex = NULL;
	LPDIRECT3DRMTEXTURE3 pGroundTex = NULL;
	LPDIRECT3DRMWRAP lpWrap = NULL;
	
	// WE NOW MAKE THE WALLS OBJECTS & DON'T CREATE THEM HERE
	// THEY'RE IN RubicsWall.cpp

	/* Load the textures
	if( FAILED( lpD3DRM->LoadTexture( "wall.bmp", &pWallTex ) ) )
		DISPLAYMSG("Failed to load Texture HA\n");
	*/
	// Load and tweak the texture
	if( FAILED( lpD3DRM->LoadTexture( "woodFloor.bmp", &pGroundTex ) ) )
		DISPLAYMSG("Failed to load Texture\n");

	if( FAILED( pGroundTex->SetColors( 32 ) ) )
		DISPLAYMSG("Failed to Set Texture Colors");

	if( FAILED( pGroundTex->SetDecalScale( TRUE ) ) )
		DISPLAYMSG("Failed to Set Decal Scale");
	
    if( FAILED( pGroundTex->SetDecalOrigin( 128, 128 ) ) )
		DISPLAYMSG("Failed to Set Decal Origin");


/////////////////////////////////////////////////////////////////////////////
// Create Ground level
/////////////////////////////////////////////////////////////////////////////
	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-BOARD_X/2.0), 0, (BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), 0, (BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), 0, (-BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), 0, (-BOARD_Y/2.0));
	
	/////////////////////////////////////////////////////////////////////////////
	// Create Ground Wrap and Texture
	/////////////////////////////////////////////////////////////////////////////
	nHeight = BOARD_Y;
	nWidth  = BOARD_X;
	
	dGroundXFactor = 0.025;	// Will scale textures to fit their respective faces
	dGroundYFactor = 0.025;	
	dWallXFactor   = 1.0;
	dWallYFactor   = 1.0;
	

	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
        D3DDivide(nWidth,D3DVAL(2.0)), D3DDivide(nHeight, D3DVAL(2.0)), D3DVAL(0.0),// Wrap Origin
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),	// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(0.0), D3DVAL(1.0),	// Wrap Up Vector
   		0.0, -0.5,								// Texture Origin
		D3DVAL(dGroundXFactor), D3DVAL(dGroundYFactor),// Texture Scale Factor
        &lpWrap);

    lpWrap->Apply( (LPDIRECT3DRMOBJECT) face );

	
	// Apply the texture to the ground
	if( FAILED( face->SetTexture( pGroundTex ) ) )
	    DISPLAYMSG("Failed to Set Texture Colors");

	RELEASE(face);




/*///////////////////////////////////////////////////////////////////////////
// Create Back Wall
/////////////////////////////////////////////////////////////////////////////
	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-BOARD_X/2.0), WALL_HEIGHT, (BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), WALL_HEIGHT, (BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), 0, (BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), 0, (BOARD_Y/2.0));

	/////////////////////////////////////////////////////////////////////////////
	// Create Back Wall Wrap and Texture
	/////////////////////////////////////////////////////////////////////////////
	nHeight = WALL_HEIGHT;
	nWidth  = BOARD_X;

	dGroundXFactor = 1.0 / -nWidth;
	dGroundYFactor = 1.0 / -nHeight;
	
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
        D3DDivide(-nWidth,D3DVAL(2.0)), D3DDivide(-nHeight, D3DVAL(2.0)), D3DVAL(0.0),	// Wrap Origin
        D3DVAL(0.0), D3DVAL(0.0), D3DVAL(1.0),	// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),	// Wrap Up Vector
   		0,-.5,		// Texture Origin
		D3DVAL(dGroundXFactor), D3DVAL(dGroundYFactor),	// Texture Scale Factor
        &lpWrap);

    lpWrap->Apply( (LPDIRECT3DRMOBJECT) face );

	// Apply the texture to the ground
	if( FAILED( face->SetTexture( pWallTex ) ) )
	    DISPLAYMSG("Failed to Set Back Wall Texture Colors");
	
	RELEASE(face);



/////////////////////////////////////////////////////////////////////////////
// Create Left Wall
/////////////////////////////////////////////////////////////////////////////
	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-BOARD_X/2.0), WALL_HEIGHT, (-BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), WALL_HEIGHT, (BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), 0, (BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), 0, (-BOARD_Y/2.0));

	/////////////////////////////////////////////////////////////////////////////
	// Create Left Wall Wrap and Texture
	/////////////////////////////////////////////////////////////////////////////
	nHeight = WALL_HEIGHT;
	nWidth  = BOARD_Y;
	dGroundXFactor = 1.0 / -nWidth;
	dGroundYFactor = 1.0 / -nHeight;
	
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
        D3DDivide(nWidth,D3DVAL(2.0)), D3DDivide(nHeight, D3DVAL(2.0)), D3DVAL(0.0),	// Wrap Origin
        D3DVAL(-1.0), D3DVAL(0.0), D3DVAL(0.0),	// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),	// Wrap Up Vector
   		0,-.5,	// Texture Origin
		D3DVAL(dGroundXFactor), D3DVAL(dGroundYFactor),// Texture Scale Factor
        &lpWrap);

    lpWrap->Apply( (LPDIRECT3DRMOBJECT) face );

	// Apply the texture to the ground
	if( FAILED( face->SetTexture( pWallTex ) ) )
	    DISPLAYMSG("Failed to Set Left Wall Texture Colors");	
	RELEASE(face);

/////////////////////////////////////////////////////////////////////////////
// Create Near Wall
/////////////////////////////////////////////////////////////////////////////
	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((BOARD_X/2.0), WALL_HEIGHT, (-BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), WALL_HEIGHT, (-BOARD_Y/2.0));
	face->AddVertex((-BOARD_X/2.0), 0, (-BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), 0, (-BOARD_Y/2.0));

	/////////////////////////////////////////////////////////////////////////////
	// Create Near Wall Wrap and Texture
	/////////////////////////////////////////////////////////////////////////////
	nHeight = WALL_HEIGHT;
	nWidth  = BOARD_X;
	dGroundXFactor = 1.0 / -nWidth;
	dGroundYFactor = 1.0 / -nHeight;
	
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
        D3DDivide(nWidth,D3DVAL(2.0)), D3DDivide(nHeight, D3DVAL(2.0)), D3DVAL(0.0),// Wrap Origin
        D3DVAL(0.0), D3DVAL(0.0), D3DVAL(-1.0),	// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),	// Wrap Up Vector
   		0,-.5,	// Texture Origin
		D3DVAL(dGroundXFactor), D3DVAL(dGroundYFactor),// Texture Scale Factor
        &lpWrap);

    lpWrap->Apply( (LPDIRECT3DRMOBJECT) face );

	// Apply the texture to the ground
	if( FAILED( face->SetTexture( pWallTex ) ) )
	    DISPLAYMSG("Failed to Set Left Wall Texture Colors");	
	RELEASE(face);


/////////////////////////////////////////////////////////////////////////////
// Create Right Wall
/////////////////////////////////////////////////////////////////////////////
	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((BOARD_X/2.0), WALL_HEIGHT, (BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), WALL_HEIGHT, (-BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), 0, (-BOARD_Y/2.0));
	face->AddVertex((BOARD_X/2.0), 0, (BOARD_Y/2.0));

	/////////////////////////////////////////////////////////////////////////////
	// Create Right Wall Wrap and Texture
	/////////////////////////////////////////////////////////////////////////////
	nHeight = WALL_HEIGHT;
	nWidth  = BOARD_Y;
	dGroundXFactor = 1.0 / -nWidth;
	dGroundYFactor = 1.0 / -nHeight;
	
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
        D3DDivide(nWidth,D3DVAL(2.0)), D3DDivide(nHeight, D3DVAL(2.0)), D3DVAL(0.0),// Wrap Origin
        D3DVAL(1.0), D3DVAL(0.0), D3DVAL(0.0),	// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),	// Wrap Up Vector
   		0,-.5,	// Texture Origin
		D3DVAL(dGroundXFactor), D3DVAL(dGroundYFactor),	// Texture Scale Factor
        &lpWrap);

    lpWrap->Apply( (LPDIRECT3DRMOBJECT) face );

	// Apply the texture to the ground
	if( FAILED( face->SetTexture( pWallTex ) ) )
	    DISPLAYMSG("Failed to Set Left Wall Texture");	
	RELEASE(face);
*/


generic_error:
	RELEASE(face);
	RELEASE(mat);
	RELEASE(pWallTex);
	RELEASE(pGroundTex);
	RELEASE(lpWrap);
	return hr;
}



HRESULT GetSquare(LPDIRECT3DRMMESHBUILDER3 board_builder, float size) {
	return GetSquare(board_builder, size, BLACK, BLACK, BLACK, BLACK, BLACK, BLACK);
}

HRESULT GetSquare(LPDIRECT3DRMMESHBUILDER3 board_builder, float size, D3DCOLOR color) {
	return GetSquare(board_builder, size, color, color, color, color, color, color);
}



//-----------------------------------------------------------------------------
// Name: GetSquare()
// Desc: Returns a Square of a given size as a MeshBuilder.
//-----------------------------------------------------------------------------

HRESULT GetSquare(LPDIRECT3DRMMESHBUILDER3 board_builder, float size, D3DCOLOR c1,D3DCOLOR c2, D3DCOLOR c3, D3DCOLOR c4, D3DCOLOR c5, D3DCOLOR c6) {
	
	HRESULT hr = S_OK;

	LPDIRECT3DRMFACE2 face = NULL;
    LPDIRECT3DRMMATERIAL2 mat = NULL;
	LPDIRECT3DRMTEXTURE3 tex = NULL;

	if (FAILED(hr = lpD3DRM->CreateMaterial(D3DVAL(10.0), &mat)))
		goto generic_error;
	
	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;
				
	face->AddVertex((-size/2.0), size/2, (size/2.0));
	face->AddVertex((size/2.0), size/2, (size/2.0));
	face->AddVertex((size/2.0), size/2, (-size/2.0));
	face->AddVertex((-size/2.0), size/2, (-size/2.0));
	face->SetColor(c1);

	RELEASE(face);

	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;

	face->AddVertex((-size/2.0), -size/2, (-size/2.0));
	face->AddVertex((size/2.0), -size/2, (-size/2.0));
	face->AddVertex((size/2.0), -size/2, (size/2.0));
	face->AddVertex((-size/2.0), -size/2, (size/2.0));
	face->SetColor(c2);
	
	RELEASE(face);

	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;

	face->AddVertex((-size/2.0), -size/2, (size/2.0));			
	face->AddVertex((size/2.0), -size/2, (size/2.0));
	face->AddVertex((size/2.0), size/2, (size/2.0));
	face->AddVertex((-size/2.0), size/2, (size/2.0));
	face->SetColor(c3);

	RELEASE(face);

	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;

	face->AddVertex((-size/2.0), size/2, (-size/2.0));
	face->AddVertex((size/2.0), size/2, (-size/2.0));
	face->AddVertex((size/2.0), -size/2, (-size/2.0));
	face->AddVertex((-size/2.0), -size/2, (-size/2.0));
	face->SetColor(c4);

	RELEASE(face);

	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;

	
	face->AddVertex((size/2.0), -size/2, (size/2.0));
	face->AddVertex((size/2.0), -size/2, (-size/2.0));
	face->AddVertex((size/2.0), size/2, (-size/2.0));	
	face->AddVertex((size/2.0), size/2, (size/2.0));
	face->SetColor(c5);

	RELEASE(face);

	if (FAILED(hr = board_builder->CreateFace(&face)))
		goto generic_error;

	face->AddVertex((-size/2.0), -size/2, (-size/2.0));
	face->AddVertex((-size/2.0), -size/2, (size/2.0));
	face->AddVertex((-size/2.0), size/2, (size/2.0));
	face->AddVertex((-size/2.0), size/2, (-size/2.0));
	face->SetColor(c6);
	
generic_error:
	RELEASE(face);
	RELEASE(mat);
	return hr;
}

HRESULT Get3DRectangle(LPDIRECT3DRMMESHBUILDER3 meshBuilder, 
					   float width, float height, float depth, 
					   float x, float y, float z,
					   D3DCOLOR c1,D3DCOLOR c2, D3DCOLOR c3, 
					   D3DCOLOR c4, D3DCOLOR c5, D3DCOLOR c6) 
{
	
	HRESULT hr = S_OK;
	LPDIRECT3DRMFACE2 face = NULL;
	
	// Create Top Face
	if (FAILED(hr = meshBuilder->CreateFace(&face)))
		goto generic_error;			
	face->AddVertex((-width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->AddVertex((width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->AddVertex((width/2.0)+x, height/2+y, (-depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, height/2+y, (-depth/2.0)+z);
	face->SetColor(c1);
	RELEASE(face);

	// Create Bottom Face
	if (FAILED(hr = meshBuilder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-width/2.0)+x, -height/2+y, (-depth/2.0)+z);
	face->AddVertex((width/2.0)+x, -height/2+y, (-depth/2.0)+z);
	face->AddVertex((width/2.0)+x, -height/2+y, (depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, -height/2+y, (depth/2.0)+z);
	face->SetColor(c2);
	RELEASE(face);

	// Create Back Face
	if (FAILED(hr = meshBuilder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-width/2.0)+x, -height/2+y, (depth/2.0)+z);			
	face->AddVertex((width/2.0)+x, -height/2+y, (depth/2.0)+z);
	face->AddVertex((width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->SetColor(c3);
	RELEASE(face);

	//  Create Front Face
	if (FAILED(hr = meshBuilder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-width/2.0)+x, height/2+y, (-depth/2.0)+z);
	face->AddVertex((width/2.0)+x, height/2+y, (-depth/2.0)+z);
	face->AddVertex((width/2.0)+x, -height/2+y, (-depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, -height/2+y, (-depth/2.0)+z);
	face->SetColor(c4);
	RELEASE(face);

	// Create Right Face
	if (FAILED(hr = meshBuilder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((width/2.0)+x, -height/2+y, (depth/2.0)+z);
	face->AddVertex((width/2.0)+x, -height/2+y, (-depth/2.0)+z);
	face->AddVertex((width/2.0)+x, height/2+y, (-depth/2.0)+z);	
	face->AddVertex((width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->SetColor(c5);
	RELEASE(face);

	// Create Left Face
	if (FAILED(hr = meshBuilder->CreateFace(&face)))
		goto generic_error;
	face->AddVertex((-width/2.0)+x, -height/2+y, (-depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, -height/2+y, (depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, height/2+y, (-depth/2.0)+z);
	face->SetColor(c6);
	
generic_error:
	RELEASE(face);
	return hr;
}

extern void triangle(LPDIRECT3DRMMESHBUILDER3 meshbuilder, float x1, float y1, float z1,float x2, float y2, float z2,float x3, float y3, float z3);

extern void DrawRectangle(LPDIRECT3DRMMESHBUILDER3 meshbuilder, float x1, float y1, float x2, float y2, float depth);

HRESULT GetCylinder(LPDIRECT3DRMMESHBUILDER3 meshBuilder, float height, float radius, D3DCOLOR color, int NumSidesInCircle) {
	const float PI = 3.14159f;
	
	float last_x, last_y, main_x, main_y, current_x, current_y, CenterX, CenterY, theta, delta_theta;
	int i;
	if(NumSidesInCircle > 0) {
		//Do top make sure normal is facing up.
		//This will matter in direct 3d;
		//Make a triangle fan from this point.
		main_x = 0;
		main_y = 0;
		
		CenterX = 0;
		CenterY = 0;
		theta = 0.0;
	
		delta_theta = (2.0f * PI ) / NumSidesInCircle;
		theta = PI;

		last_x = CenterX + radius * cos(theta);
		last_y = CenterY + radius * sin(theta);
		theta += delta_theta;

		for(i = 0 ; i < NumSidesInCircle; i++) {
			current_x = CenterX + radius * cos(theta);
			current_y = CenterY + radius * sin(theta);

			triangle(meshBuilder, main_x, main_y, 0, last_x, last_y, 0, current_x, current_y, 0);
			triangle(meshBuilder, main_x, main_y, height, -last_x, -last_y, height, current_x, current_y, height);
			DrawRectangle(meshBuilder, current_x, current_y, last_x, last_y, height);
			theta += delta_theta;
			last_x = current_x;
			last_y = current_y;
		}
	}
	return S_OK;
}


