//-----------------------------------------------------------------------------
// File: RubicsWall.cpp
//
// Desc: This file creates the walls used in the Game
//
// 4-1-1999:	Grant
//-----------------------------------------------------------------------------

#pragma warning( disable : 4244) 

#include "RubicsObjects.h"
#include "RubicsWall.h"
#include "RubicsDirect3D.h"

/***************************************************
 *
 * RubicsWall()
 *
 * Constructor
 *
 ***************************************************/
RubicsWall::RubicsWall (LPDIRECT3DRM3 lpD3DRM, 
							  LPDIRECT3DRMFRAME3 Parent, Position pos, float orientation,
							  int wall_height, int wall_width, char* bmpFile, SoundSet *pSS )
{
	//int nHeight;
	//int nWidth;
	double dGroundXFactor;	
	double dGroundYFactor;
	//double dScaleFactor = 1.0;
	pSoundSet = pSS;

	LPDIRECT3DRMMESHBUILDER3 meshBuilder;
	LPDIRECT3DRMFACE2 face = NULL;
	LPDIRECT3DRMMESH mesh = NULL;
	LPDIRECT3DRMTEXTURE3 pTex = NULL;
	LPDIRECT3DRMWRAP lpWrap = NULL;
	self_frame = NULL;
	LPDIRECT3DRMOBJECT faceObj = NULL; 
	HRESULT hr;

	// Initialize member variables
	//m_fBlockSize = fSize;	// Length of Block's sides

	// Create the meshbuilder
	if (FAILED( lpD3DRM->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	// Load the textures
	if( FAILED( lpD3DRM->LoadTexture( bmpFile, &pTex ) ) )
		DISPLAYMSG("Failed to load Texture to Wall\n");

	if (FAILED(meshBuilder->CreateFace(&face)))
		DISPLAYMSG("Failed to create Wall face HA\n");

	face->AddVertex((-wall_width/2.0), wall_height, 0);
	face->AddVertex((wall_width/2.0), wall_height, 0);
	face->AddVertex((wall_width/2.0), 0, 0);
	face->AddVertex((-wall_width/2.0), 0, 0);

	/////////////////////////////////////////////////////////////////////////////
	// Create Back Wall Wrap and Texture
	/////////////////////////////////////////////////////////////////////////////
	//nHeight = WALL_HEIGHT;
	//nWidth  = BOARD_X;

	dGroundXFactor = 1.0 / (-wall_width / 3);
	dGroundYFactor = 1.0 / -wall_height;
	
	hr = face->QueryInterface(IID_IDirect3DRMObject, (void **) &faceObj );

	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,					// Wrap Type and Reference Frame
        D3DDivide(-wall_width,D3DVAL(2.0)), D3DDivide(-wall_height, D3DVAL(2.0)), D3DVAL(0.0),	// Wrap Origin
        D3DVAL(0.0), D3DVAL(0.0), D3DVAL(1.0),	// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),	// Wrap Up Vector
   		0,-.5,		// Texture Origin
		D3DVAL(dGroundXFactor), D3DVAL(dGroundYFactor),	// Texture Scale Factor
        &lpWrap);

    lpWrap->Apply(  faceObj );
	RELEASE(faceObj);

	// Apply the texture to the wall
	if( FAILED( face->SetTexture( pTex ) ) )
	    DISPLAYMSG("Failed to Set Wall Texture Colors");
	
	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;
	}
	
	if (FAILED(lpD3DRM->CreateFrame( Parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create frame in WALL");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to WALL frame" );
		goto error;
	}

	if (FAILED(self_frame->SetPosition(Parent, pos.x, pos.y, pos.z)))
				DISPLAYMSG("Error setting Position on WALL");

	if (FAILED(self_frame->SetOrientation(Parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on WALL");
	
error:
	RELEASE( meshBuilder );
	RELEASE( mesh );
//	RELEASE( pTex );
//	RELEASE(face);
	
}

/***************************************************
 *
 * ~AlphabetBlock
 *
 * Destructor
 *
 ***************************************************/
RubicsWall::~RubicsWall( )
{
	RELEASE( self_frame );
}


/***************************************************
 *
 * GetFrame
 *
 * Returns a frame that contains the Alphabet Block
 *
 ***************************************************/
BOOL RubicsWall::GetFrame( LPDIRECT3DRMFRAME3 *pFrame )
{
	*pFrame = self_frame;
	//We're making another pointer to this cubes frame, so increment the
	//reference count.
	if (self_frame != NULL) self_frame->AddRef();
	return TRUE;
}



/***************************************************
 *
 * SetTexture
 *
 * Sets the Texture on the front and back of the block
 *
 **************************************************
BOOL RubicsWall::SetTextureToFace( int faceNumber, float fBlockSize,
									  LPDIRECT3DRMTEXTURE3 pTex, 
									  LPDIRECT3DRMMESHBUILDER3 board_builder )
{
	int x = 0,y = 0,z = 0;	// Wrap direction vectors
	LPDIRECT3DRMFACE2 face = NULL;
	LPDIRECT3DRMWRAP  lpWrap = NULL;
	
	switch( faceNumber )
	{
	case 2:
		z = 1;
		break;
	case 3:
		z = -1;
		break;
	}



	////////////////////////////////////////////////
	// Set Texture of Alphabet Block's face
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,							// Wrap Type and Reference Frame
        D3DVAL(0.0), D3DVAL(0.0), D3DVAL(0.0),			// Wrap Origin
		D3DVAL(x), D3DVAL(y), D3DVAL(z),				// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),			// Wrap Up Vector
   		D3DVAL(-0.5), D3DVAL(0.0),						// Texture Origin
		D3DVAL(-1.0/fBlockSize), D3DVAL(-1.0/fBlockSize),	// Texture Scale Factor
        &lpWrap);

	if( FAILED( board_builder->GetFace( faceNumber, &face ) ) )
	{
		DISPLAYMSG("Unable to get face");
		goto error;
	}

    if( FAILED( lpWrap->Apply( (LPDIRECT3DRMOBJECT) face ) ) )
	{
		DISPLAYMSG("Unable to apply wrap to face");
		goto error;
	}

	if( FAILED( face->SetTexture( pTex ) ) )
	{
		DISPLAYMSG("Unable to set texture to face");
		goto error;
	}
	
	RELEASE( face );
	RELEASE( lpWrap );
	return TRUE;
error:
	RELEASE( face );
	RELEASE( lpWrap );
	return FALSE;
}

void RubicsWall::GetWall(LPDIRECT3DRMMESHBUILDER3 meshBuilder, 
					   float width, float height, float depth, 
					   float x, float y, float z,
					   D3DCOLOR c1)
{
	
	//HRESULT hr = S_OK;
	LPDIRECT3DRMFACE2 face = NULL;
	
	// Create Back Face
	if (FAILED(meshBuilder->CreateFace(&face)))
		DISPLAYMSG("Unable to create wall face");
	face->AddVertex((-width/2.0)+x, -height/2+y, (depth/2.0)+z);			
	face->AddVertex((width/2.0)+x, -height/2+y, (depth/2.0)+z);
	face->AddVertex((width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->AddVertex((-width/2.0)+x, height/2+y, (depth/2.0)+z);
	face->SetColor(c1);
	RELEASE(face);
}
*/
