#include "RubiksAlphabetBlock.h"
#include "RubicsDirect3D.h"
#include "RubicsObjects.h"


/***************************************************
 *
 * AlphabetBlock()
 *
 * Constructor
 *
 ***************************************************/
AlphabetBlock::AlphabetBlock ( BlockType type, LPDIRECT3DRM3 lpD3DRM, 
							  LPDIRECT3DRMFRAME3 Parent, Position pos, float orientation, float fSize,
							  int bcolor, SoundSet *pSound )
{
	LPDIRECT3DRMMESHBUILDER3 meshBuilder;
	LPDIRECT3DRMMESH mesh;
	LPDIRECT3DRMTEXTURE3 pTex;
	//float BlockX, BlockY, BlockZ;
	//D3DCOLOR m_ColorCenter;
	self_frame = NULL;
	pSoundSet = pSound;
	color_index = bcolor;

	// Initialize member variables
	m_fBlockSize = fSize;	// Length of Block's sides
	m_eBlockType = type;	// Type of cube (ie "A", "B",... etc. )
	SetAttributes();		// Sets the attributes of the cube based on block type

	// Create the meshbuilder
	if (FAILED( lpD3DRM->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}


	// Create alphabet cube with six faces
	Get3DRectangle( meshBuilder, 
					BlockX, BlockY, BlockZ,			// Cube Dimensions 
					0.0, (float)(BlockY/2.0), 0.0,	// Center of cube in relation to meshBuilder
					m_Color, m_Color, m_ColorCenter, m_ColorCenter, m_Color, m_Color);  // Color of the cube


	if( m_pcTextureName != NULL ) 
	{
		if (FAILED( lpD3DRM->LoadTexture( m_pcTextureName, &pTex ) ) )
		{
			DISPLAYMSG("Failed to load bitmap");
		} 
		
		if(type == DIE)// We're a DIE
		{
			if (FAILED( lpD3DRM->LoadTexture( "die1.bmp", &pTex ) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 1, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if (FAILED( lpD3DRM->LoadTexture( "die2.bmp", &pTex) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 2, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if (FAILED( lpD3DRM->LoadTexture( "die3.bmp", &pTex) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 3, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if (FAILED( lpD3DRM->LoadTexture( "die4.bmp", &pTex) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 4, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if (FAILED( lpD3DRM->LoadTexture( "die5.bmp", &pTex) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 5, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if (FAILED( lpD3DRM->LoadTexture( "die6.bmp", &pTex) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 0, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
		}
		else if(type == ABLOCK || type == BBLOCK || type == CBLOCK)
		//I added this so the fullscreen would work;
		// Set textures to front and back faces
		{
			if( !SetTextureToFace( 2, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}

			if( !SetTextureToFace( 3, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 3");
				goto error;}

			if (FAILED( lpD3DRM->LoadTexture( "Litewood.bmp", &pTex ) ) )
			{
				DISPLAYMSG("Failed to load bitmap");
			}
			if( !SetTextureToFace( 0, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if( !SetTextureToFace( 1, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if( !SetTextureToFace( 4, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
			if( !SetTextureToFace( 5, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}
		}
		else
		//I added this so the fullscreen would work;
		// Set textures to front and back faces
		{
			if( !SetTextureToFace( 2, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 2");
				goto error;}

			if( !SetTextureToFace( 3, m_fBlockSize, pTex, meshBuilder ) ){ 
				DISPLAYMSG("Unable to set Texture to face 3");
				goto error;}
		}

		RELEASE( pTex );
	}

	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;
	}
	
	if (FAILED(lpD3DRM->CreateFrame( Parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create frame in Alphabet Block");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to frame" );
		goto error;
	}

	if (FAILED(self_frame->SetPosition(Parent, pos.x, pos.y, pos.z)))
				DISPLAYMSG("Error setting Position on an Alphabet Block");

	if (FAILED(self_frame->SetOrientation(Parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on an Alphabet Block");
	
error:
	RELEASE( meshBuilder );
	RELEASE( mesh );
	
}

/***************************************************
 *
 * ~AlphabetBlock
 *
 * Destructor
 *
 ***************************************************/
AlphabetBlock::~AlphabetBlock( )
{
	RELEASE( self_frame );
}


/***************************************************
 *
 * GetFrame
 *
 * Returns a frame that contains the Alphabet Block
 *
 ***************************************************/
BOOL AlphabetBlock::GetFrame( LPDIRECT3DRMFRAME3 *pFrame )
{
	*pFrame = self_frame;
	//We're making another pointer to this cubes frame, so increment the
	//reference count.
	if (self_frame != NULL) self_frame->AddRef();
	return TRUE;
}



/***************************************************
 *
 * SetTexture
 *
 * Sets the Texture on the front and back of the block
 *
 ***************************************************/
BOOL AlphabetBlock::SetTextureToFace( int faceNumber, float fBlockSize,
									  LPDIRECT3DRMTEXTURE3 pTex, 
									  LPDIRECT3DRMMESHBUILDER3 mb )
{
	int x = 0,y = 0,z = 0;	// Wrap direction vectors
	LPDIRECT3DRMFACE2 face = NULL;
	LPDIRECT3DRMWRAP  lpWrap = NULL;
	float xwrap = 0.0;
	
	switch( faceNumber )
	{
	case 0:
		y = 1;
		xwrap = 1.0;
		break;
	case 1:
		y = -1;
		xwrap = 1.0;
		break;
	case 2:
		z = 1;
		break;
	case 3:
		z = -1;
		break;
	case 4:
		x = 1;
		break;
	case 5:
		x = -1;
		break;
	}

	////////////////////////////////////////////////
	// Set Texture of Alphabet Block's face
	lpD3DRM->CreateWrap
        (D3DRMWRAP_FLAT, NULL,							// Wrap Type and Reference Frame
        D3DVAL(xwrap), D3DVAL(0.0), D3DVAL(0.0),			// Wrap Origin
		D3DVAL(x), D3DVAL(y), D3DVAL(z),				// Wrap Direction Vector
        D3DVAL(0.0), D3DVAL(1.0), D3DVAL(0.0),			// Wrap Up Vector
   		D3DVAL(-0.5), D3DVAL(0.0),						// Texture Origin
		D3DVAL(-1.0/fBlockSize), D3DVAL(-1.0/fBlockSize/ywrap),	// Texture Scale Factor
        &lpWrap);

	if( FAILED( mb->GetFace( faceNumber, &face ) ) )
	{
		DISPLAYMSG("Unable to get face");
		goto error;
	}

    if( FAILED( lpWrap->Apply( (LPDIRECT3DRMOBJECT) face ) ) )
	{
		DISPLAYMSG("Unable to apply wrap to face");
		goto error;
	}

	if( FAILED( face->SetTexture( pTex ) ) )
	{
		DISPLAYMSG("Unable to set texture to face");
		goto error;
	}
	
	RELEASE( face );
	RELEASE( lpWrap );
	return TRUE;
error:
	RELEASE( face );
	RELEASE( lpWrap );
	return FALSE;
}

//	This function just sets the texture to match which letter the block has on it
void AlphabetBlock::SetAttributes( )
{
	switch( m_eBlockType ) 
	{
		case ABLOCK:
			m_pcTextureName = "blockA.bmp";
			m_Color			= RED;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize;
			BlockZ = m_fBlockSize;
			m_ColorCenter = WHITE;
			ywrap = 1.0;
			break;
		
		case BBLOCK:
			m_pcTextureName = "blockB.bmp";
			m_Color			= BLUE;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize;
			BlockZ = m_fBlockSize;
			m_ColorCenter = WHITE;
			ywrap = 1.0;
			break;

		case CBLOCK:
			m_pcTextureName = "blockC.bmp";
			m_Color			= YELLOW;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize;
			BlockZ = m_fBlockSize;
			m_ColorCenter = WHITE;
			ywrap = 1.0;
			break;

		case PBOX:
			m_pcTextureName = "blockC.bmp";
			m_Color			= YELLOWO;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize;
			BlockZ = m_fBlockSize;
			m_ColorCenter = WHITE;
			ywrap = 1.0;
			break;

		case DOMINOBLOCK:
			m_pcTextureName = "domino.bmp";
			m_Color			= BLACK;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize/2.0f;
			BlockZ = m_fBlockSize/8.0f;
			ywrap = 0.5;
			m_ColorCenter = WHITE;
			break;

		case DIE:
			m_pcTextureName = "blockC.bmp";
			m_Color			= WHITE;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize;
			BlockZ = m_fBlockSize;
			m_ColorCenter = WHITE;
			ywrap = 1.0;
			break;

		case GDECK:
			m_pcTextureName = NULL;
			switch(color_index){
			case 0:
			  m_Color			= BLUEO;
			  break;
			case 1:
			  m_Color			= REDO;
			  break;
			case 2:
			  m_Color			= GREENO;
			  break;
			case 3:
			  m_Color			= YELLOWO;
			  break;
			default:
			  m_Color			= BLUEO;
			  break;
			}
			BlockX = m_fBlockSize;
			BlockY = 4.0f;
			BlockZ = m_fBlockSize;
			ywrap = 1.0;
			m_ColorCenter = m_Color;
			break;

		default:
			m_pcTextureName = NULL;
			m_Color			= BLACK;
			BlockX = m_fBlockSize;
			BlockY = m_fBlockSize;
			BlockZ = m_fBlockSize;
			ywrap = 1.0;
			m_ColorCenter = WHITE;
			break;
	}
}
