 /*
  *
  * RubiksAlphabetBlock.h  
  *
  *	Alphabet Block class that will be used as an obstacle in
  * the game playing field
  *              
  * Author: Dave Pace
  * Seriously Modified by Grant Jorde Please ask me questions
  *
  */
#ifndef  __RUBIKS_ALPHABETBLOCK__
#define  __RUBIKS_ALPHABETBLOCK__

#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <ddraw.h>
#include <stdio.h>
#include <math.h>

#include "colors.h"
#include "rubiksInfo.h"
#include "client_objects.h"
#include "messages.h"
#include "SoundTypes.h"
#include "RubiksSoundSet.h"

#ifndef RELEASE
#define RELEASE(x) if (x != NULL) { x->Release(); x = NULL; } 
#endif

// Macro to display a message box containing the given string. 
#ifndef DISPLAYMSG
//#define DISPLAYMSG(x) MessageBox(NULL, x, "Rubiks ICON", MB_OK);
#define DISPLAYMSG(x) OutputDebugString(x)
#endif

typedef enum _BlockType {
	ABLOCK, BBLOCK, CBLOCK, PBOX, GDECK, DOMINOBLOCK, DIE
} BlockType;

class AlphabetBlock : CClientObjectsNonMovable{

	public:
		AlphabetBlock( BlockType TYPE, LPDIRECT3DRM3 lpD3DRM, 
			LPDIRECT3DRMFRAME3 Parent, Position Pos, float orientation, float fSize, int bcolor = 0, SoundSet *pSound = NULL );	
	
		~AlphabetBlock( );

		// Returns a frame that contains the Alphabet Block
		BOOL GetFrame( LPDIRECT3DRMFRAME3 *pFrame );

		D3DCOLOR m_Color;
		void SetAttributes( );

	private:
		D3DCOLOR m_ColorCenter;
		SoundSet * pSoundSet;
		float m_fBlockSize;
		float BlockX, BlockY, BlockZ;
		float ywrap;
		char* m_pcTextureName;
		BlockType m_eBlockType;
		int color_index;

		//LPDIRECT3DRMFRAME3 selfFrame;
		BOOL SetTextureToFace( int face, float fSize, LPDIRECT3DRMTEXTURE3 pTex, LPDIRECT3DRMMESHBUILDER3 mb );
};

#endif //__RUBIKS_ALPHABETBLOCK__