#include "RubiksCube.h"
#include "Messages.h"
#include "RubicsDirect3D.h"
#include "RubicsMain.h"



RubiksCube::~RubiksCube()
{
	/* Blocks*/
	for (int i = 0; i < 27; i++)  RELEASE(blocks[i]);	
	delete [] blocks;
	blocks = NULL;

	/* Animations*/
	for (i = 0; i < 3; i++)	RELEASE(stack_move[i]);
	RELEASE(block_rotate);


	/* Frames */
	RELEASE(parent_frame);
	RELEASE(self_frame);
	RELEASE(shadow);

	/*Info*/
	//delete info;
}



RubiksCube::RubiksCube(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, BOOL centerPieces[6], 
					   char playername[], SoundSet *pSound, BOOL isHero )
{
	specularPower = 20.0;
	parent_frame = parent;
	parent->AddRef();
	modified = FALSE;
	multi_shot = FALSE;
	ricochete = FALSE;
	pSoundSet = pSound;
	currentWeapon = Ram;
	bHero = isHero;  
	lpdsb = NULL;
	lpdsb3d = NULL;

	int i;

	//Create the main frame
	if (FAILED(rm->CreateFrame(parent, &self_frame)))
	{
		DISPLAYMSG("Unable to create self_frame in RubiksCube");
		return;
	}

	//Setup defaults for the info. This will make the guy
	//stand still at the origin and do nothing.
	//info = new rubiks_info;

	info.head  = 0.0f;
	info.pos.x = 0.0f;
	info.pos.y = 0.0f;
	info.pos.z = 0.0f;
	info.framecount = 0.0f;
	info.anim = STANDING;

	//We have no center cubes to begin with
	/* This was a false assumption... when a player starts their game, all the other players
	   who've already collected pieces get added with this constructor and they do have
	   center pieces to start with!
	for (i = 0; i < 6; i++)
		centerCube[i] = FALSE;
	*/
	for (i = 0; i < 6; i++)
		centerCube[i] = centerPieces[i];
	
	for (i = 0; i<10; i++)
		name[i] = playername[i];

	//Create all 27 block frames
	blocks = new LPDIRECT3DRMFRAME3[27];

	for (i = 0; i < 27; i++) blocks[i] = NULL;

	
	getBlockFrame(RED, BLACK, BLACK, BLUE, BLACK, WHITE, rm, &blocks[0], self_frame, specularPower);
	getBlockFrame(RED, BLACK, BLACK, BLACK, BLACK, WHITE, rm, &blocks[1], self_frame, specularPower);
	getBlockFrame(RED, GREEN, BLACK, BLACK, BLACK, WHITE, rm, &blocks[2], self_frame, specularPower);
	getBlockFrame(RED, BLACK, BLACK, BLUE, BLACK, BLACK, rm, &blocks[3], self_frame, specularPower);
	getBlockFrame(RED, BLACK, BLACK, BLACK, BLACK, BLACK, rm, &blocks[4], self_frame, specularPower);
	getBlockFrame(RED, GREEN, BLACK, BLACK, BLACK, BLACK, rm, &blocks[5], self_frame, specularPower);
	getBlockFrame(RED, BLACK, BLACK, BLUE, YELLOW, BLACK, rm, &blocks[6], self_frame, specularPower);
	getBlockFrame(RED, BLACK, BLACK, BLACK, YELLOW, BLACK, rm, &blocks[7], self_frame, specularPower);
	getBlockFrame(RED, GREEN, BLACK, BLACK, YELLOW, BLACK, rm, &blocks[8], self_frame, specularPower);

	getBlockFrame(BLACK, BLACK, BLACK, BLUE, BLACK, WHITE, rm, &blocks[9], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, BLACK, BLACK, BLACK, WHITE, rm, &blocks[10], self_frame, specularPower);
	getBlockFrame(BLACK, GREEN, BLACK, BLACK, BLACK, WHITE, rm, &blocks[11], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, BLACK, BLUE, BLACK, BLACK, rm, &blocks[12], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, BLACK, BLACK, BLACK, BLACK, rm, &blocks[13], self_frame, specularPower);
	getBlockFrame(BLACK, GREEN, BLACK, BLACK, BLACK, BLACK, rm, &blocks[14], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, BLACK, BLUE, YELLOW, BLACK, rm, &blocks[15], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, BLACK, BLACK, YELLOW, BLACK, rm, &blocks[16], self_frame, specularPower);
	getBlockFrame(BLACK, GREEN, BLACK, BLACK, YELLOW, BLACK, rm, &blocks[17], self_frame, specularPower);

	getBlockFrame(BLACK, BLACK, ORANGE, BLUE, BLACK, WHITE, rm, &blocks[18], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, ORANGE, BLACK, BLACK, WHITE, rm, &blocks[19], self_frame, specularPower);
	getBlockFrame(BLACK, GREEN, ORANGE, BLACK, BLACK, WHITE, rm, &blocks[20], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, ORANGE, BLUE, BLACK, BLACK, rm, &blocks[21], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, ORANGE, BLACK, BLACK, BLACK, rm, &blocks[22], self_frame, specularPower);
	getBlockFrame(BLACK, GREEN, ORANGE, BLACK, BLACK, BLACK, rm, &blocks[23], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, ORANGE, BLUE, YELLOW, BLACK, rm, &blocks[24], self_frame, specularPower);
	getBlockFrame(BLACK, BLACK, ORANGE, BLACK, YELLOW, BLACK, rm, &blocks[25], self_frame, specularPower);
	getBlockFrame(BLACK, GREEN, ORANGE, BLACK, YELLOW, BLACK, rm, &blocks[26], self_frame, specularPower);

	
	/* Don't make a bitmap if you're the player */
	if (!bHero) getPlayerIcon( rm, &lpPlayerIconFrame, self_frame ); 

	

	//Add an extra pointer to the center cubes. Necessary because
	//we remove them from the frame during gameplay
	blocks[4]->AddRef();
	blocks[10]->AddRef();
	blocks[12]->AddRef();
	blocks[14]->AddRef();
	blocks[16]->AddRef();
	blocks[22]->AddRef();
	
	createShadow(self_frame, rm);

	/* Remove the center cubes if we don't have them! */
	if(!centerCube[RED_SQUARE])
		self_frame->DeleteChild(blocks[4]);
	if(!centerCube[WHITE_SQUARE])
		self_frame->DeleteChild(blocks[10]);
	if(!centerCube[BLUE_SQUARE])
		self_frame->DeleteChild(blocks[12]);
	if(!centerCube[GREEN_SQUARE])
		self_frame->DeleteChild(blocks[14]);
	if(!centerCube[YELLOW_SQUARE])
		self_frame->DeleteChild(blocks[16]);
	if(!centerCube[ORANGE_SQUARE])
		self_frame->DeleteChild(blocks[22]);

	/** Set up the animation interfaces */

	if (FAILED(rm->CreateAnimation(&block_rotate))    || 
		FAILED(rm->CreateAnimation(&cannon))          || 
		FAILED(rm->CreateAnimation(&stack_move[0]))   ||
		FAILED(rm->CreateAnimation(&stack_move[1]))   ||
		FAILED(rm->CreateAnimation(&stack_move[2])))   
	{
		DISPLAYMSG("Unable to create block_rotate in RubiksCube");
		return;
	}

	if (FAILED(block_rotate->SetOptions(D3DRMANIMATION_OPEN | D3DRMANIMATION_SPLINEPOSITION
								       | D3DRMANIMATION_POSITION 
									   | D3DRMANIMATION_SCALEANDROTATION)))
	{
			DISPLAYMSG("Unable to set animation options in RubiksCube");
		return;
	}
	//Copy the options
	cannon->SetOptions(block_rotate->GetOptions());
	stack_move[0]->SetOptions(block_rotate->GetOptions());
	stack_move[1]->SetOptions(block_rotate->GetOptions());
	stack_move[2]->SetOptions(block_rotate->GetOptions());


	// Orientation vectors for the 3D Listener
	vFront.x = cos( info.head );
	vFront.y = 0.0;
	vFront.z = sin( info.head );

	vTop.x = 0.0;
	vTop.y = 1.0;
	vTop.z = 0.0;



	createHoppingAnimation();

	//Set the default position for the frames
	getStandingFrame();
	
}






///////////////////////////////////////////////////////////
////          Animation for forward hopping           /////
///////////////////////////////////////////////////////////
void RubiksCube::createHoppingAnimation()
{
	D3DVECTOR v;
		v.x = 0.0f; 
		v.y = 0.0f; 
		v.z = 1.0f;

	D3DRMQUATERNION quat;

	//X direction slide of each slice, along with Y pull-apart.
	for (int i = 0; i < 3; i++)
	{
	//Before we start moving
	stack_move[i]->AddPositionKey(0.0f,  0.0f,           -.8f*i,                0.0f);
	//Start jumping
	stack_move[i]->AddPositionKey(4.0f,  .8f*i,           .5f*i,                0.0f);
	//Suspend in mid-air
	stack_move[i]->AddPositionKey(9.0f,   0.0f,           .5f*i + .9f*(2-i),    0.0f);
	stack_move[i]->AddPositionKey(16.0f, .0f,             .5f*i + .9f*(2-i),    0.0f);
	//Reverse the jump
	stack_move[i]->AddPositionKey(20.0f, .8f*(2-i),       .5f*i,                0.0f);
	//Land and crunch
	stack_move[i]->AddPositionKey(24.0f, .0f,            -.2f*i,                0.0f);
    //Prepare for next frame
	stack_move[i]->AddPositionKey(30.0f, .0f,            -.6f * i,              0.0f);
	
	/**Rotations of each face */
	//Start a bit tucked forward
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f*i);
	stack_move[i]->AddRotateKey(0.0f, &quat);
    //Jump up and rotate to normal
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	stack_move[i]->AddRotateKey(5.0f, &quat);
    //Lean back on jump a bit as we reach the crest
	D3DRMQuaternionFromRotation(&quat, &v, -0.20f);
	stack_move[i]->AddRotateKey(9.0f, &quat);
    //Lean WAY back as we start coming down!
	D3DRMQuaternionFromRotation(&quat, &v, -0.20f*i);
	stack_move[i]->AddRotateKey(24.0f, &quat);
    //SPLAT!! Hit the ground and level out
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f);
	stack_move[i]->AddRotateKey(27.0f, &quat);
    //Lean forward again
	D3DRMQuaternionFromRotation(&quat, &v, 0.0f*i);
	stack_move[i]->AddRotateKey(30.0f, &quat);
	}
}





void RubiksCube::ReceiveMessage(rubiks_info *message)
{
	memcpy(&info, message, sizeof(rubiks_info));

		/*ReceiveMessage
	info.head  = message->head;
	info.pos.x = message->pos.x;
	info.pos.y = message->pos.y;
	info.pos.z = message->pos.z;
	info.framecount = message->framecount;
	info.anim = message->anim;*/
	//We should re-evaluate ourselves next time Update is called
	modified = TRUE;


}

/** Updates the drawing of the cube based on
  * the information it's received
  */
void RubiksCube::Update()
{
	//Only re-evaluate ourselves if a call to ReceiveMessage was made.
	if (modified == FALSE) return;

	switch (info.anim)
	{
	case STANDING:
		getStandingFrame();
		break;

	case HOPPING:
		getHoppingFrame(true);
		break;

	case BACKWARDS:
		getHoppingFrame(false);
		break;

	case BEGINFIRING:
		getBeginFiringFrame();
		break;
	case FIRING:
		getFiringFrame();
		break;
	case ENDFIRING:
		getEndFiringFrame();
		break;

	case BEGINACTION2:
		switch (currentWeapon)
		{
			case Tornado:
				getTornadoFrame(0);
				break;
			case Ram:
				getBeginRamFrame();
				break;
			default:
			    getStandingFrame();
				break;
		}
		break;

	case ACTION2:
		switch (currentWeapon)
		{
			case Tornado:
				getTornadoFrame(1);
				break;
			case Ram:
				getRamFrame();
				break;
			default:
				getStandingFrame();
				break;
		}
		break;

	case ENDACTION2:
		switch (currentWeapon)
		{
			case Tornado:
				getTornadoFrame(2);
				break;
			case Ram:
				getEndRamFrame();
				break;
			default:
			    getStandingFrame();
				break;
		}
		break;

	case COLLIDEWITHPROJECTILE:
	case COLLIDEWITHPLAYER:
		getHitByProjectileFrame();
		break;
	default:
		getStandingFrame();
	}

	
	/* Global map position and location */
	self_frame->AddRotation(D3DRMCOMBINE_REPLACE, 0.0f, 1.0f, 0.0f, info.head);
	self_frame->SetPosition(parent_frame, info.pos.x, info.pos.y, info.pos.z);

	/** Position of player bitmap **/
	if (!bHero)
	{
		lpPlayerIconFrame->LookAt( GetCamera(), self_frame, D3DRMCONSTRAIN_X );
		lpPlayerIconFrame->LookAt( GetCamera(), self_frame, D3DRMCONSTRAIN_Z );
	}
	/*Shadow position. Simulate it staying in place by moving it down at the
	  same rate the cube goes up */
	D3DVALUE y = info.pos.y;
	shadow->SetPosition(self_frame, 0, -y, 0);
//	shadow->AddScale(D3DRMCOMBINE_BEFORE, 1.0f - 0.6f*(y/50.0f), 1.0f, 1.0f - 0.6f*(y/50.0f));
	
	modified = FALSE;
}

/* Can hop forwards or backwards. Just reverses the framecount
 * for the backwards case.
 */
BOOL RubiksCube::getHoppingFrame(bool forwards)
{
	LPDIRECT3DRMFRAME3 block;
	int index, frame;

	if (forwards) frame = info.framecount;
	else		  frame = MAX_HOPPING - info.framecount;

	/** Model each individual block **/
	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			index = x + 3*y + 9*z;
			block = blocks[index];

			stack_move[y]->SetFrame(block);
			stack_move[y]->SetTime(D3DVALUE(frame));

			//Moves block to its place in the cube
			block->AddTranslation(D3DRMCOMBINE_BEFORE, -1.55f + 1.05f*x, 1.05f*y,
													  -1.55f + 1.05f * z);
		  }

	RELEASE(block)
	return TRUE;
}

BOOL RubiksCube::getBeginFiringFrame()
{
	return getFiringFrame();
}

BOOL RubiksCube::getFiringFrame()
{
	LPDIRECT3DRMFRAME3 block;
	D3DVALUE           spin;

	spin = (info.framecount/MAX_FIRING) * 6.28f; 

	/** Model each individual block **/
	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			//Get the current block. 
			block = blocks[x + 3*y + 9*z];
			block->AddTranslation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 0.0f);
			//Reset our position
			if (y == 2)
			block->AddRotation(D3DRMCOMBINE_BEFORE, 0.0f, 1.0f, 0.0f, spin);

			//Moves block to its place in the cube
			block->AddTranslation(D3DRMCOMBINE_BEFORE, -1.55f + 1.05f*x, 
			                                           1.05f*y, -1.55f + 1.05f * z);
			
	}
	

	RELEASE(block)
	return TRUE;
}


BOOL RubiksCube::getEndFiringFrame()
{
	return getFiringFrame();
}

//type corresponds to BEGINTORNADO, TORNADO, ENDTORNADO
BOOL RubiksCube::getTornadoFrame(int type)
{
	LPDIRECT3DRMFRAME3 block;
	D3DVALUE spin;
	D3DVALUE tx=0.0f, ty=0.0f, tz=0.0f, scale=0.0f, increment=0.0f;
	

	/** Model each individual block **/
	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			//Get the current block. 
			block = blocks[x + 3*y + 9*z];

			/* Reset the animation */
			block->AddTranslation(D3DRMCOMBINE_REPLACE, 0.0f, 0.0f, 0.0f);

			 if (type == 0)  
				 spin = (info.framecount/MAX_BEGINTORNADO) * 6.28;
			 else if (type == 1) 
			 {
				 spin = 3.0f * (info.framecount/MAX_TORNADO) * 6.28;
				 while (spin > 6.28) spin -= 6.28f;
			 }
			 else 
				 spin = (info.framecount/MAX_ENDTORNADO) * 6.28;
			
			 

			/* Rotate the piece around the center of the hurricane */
			block->AddRotation(D3DRMCOMBINE_BEFORE, 0.0f, 1.0f, 0.0f, spin);
			

			  /* Direction in which this piece should fly */
			  if (x == 0)	   tx = -1;
			  else if (x == 1) tx = 0;
			  else			   tx = 1;

			  if (z == 0)	   tz = -1;
			  else if (z == 1) tz = 0;
			  else			   tz = 1;

			  /* Make the non-corner pieces go out farther
			     to make a nice circle */
				   if (x == 1 && z == 0) tz = -1.5f;
			  else if (x == 1 && z == 2) tz = 1.5f;
			  else if (x == 0  && z == 1) tx = -1.5f;
			  else if (x == 2 && z == 1) tx = 1.5f;

			  /* Scatter the height of each piece in a spiral.
			     Make a ring at the top. */
			  if (y == 2)
				  if (x+z % 2 == 0)   ty = 5.0f;
				  else				  ty = 4.5f;
			  else ty = ((x+y+z)/5.0f) *5.5f;
			  
			  /* Scale everything based on our height. */
			  if (y == 0) scale = 0.2f;
			  else if (y == 1) scale = 0.45f;
			  else scale = 0.7f;

			  int max = 3;
			

			if (type == 0) increment = max * (info.framecount / MAX_BEGINTORNADO);
			else if (type == 1) increment = max;
			else increment = max - (max * (info.framecount / MAX_ENDTORNADO));
			
			//Hurricane outward motion		
			block->AddTranslation(D3DRMCOMBINE_BEFORE, tx*scale*increment,
									   ty * increment, tz*scale*increment);
	
			/* Normal translation to cube shape */
			block->AddTranslation(D3DRMCOMBINE_BEFORE, -1.55f + 1.05f*x, 
			                                            1.05f*y, -1.55f + 1.05f * z);

			/* Spin each cube a random number of degrees around its own axis */
			D3DVALUE mag;
			if (type == 0) mag = 0.628f;
			else mag = 1.2f;
			spin = 0.0f;
			D3DVALUE rnd = (float)(rand() % 100);
			rnd = mag * (rnd / 100.0f);
			spin = (info.framecount * rnd );
			while (spin > 6.28) spin -= 6.28f;

			//assert(SUCCEEDED(block->AddTranslation(D3DRMCOMBINE_BEFORE, 0.5f, 0.5, 0.5f)));
			block->AddRotation(D3DRMCOMBINE_BEFORE, 1.0f, 1.0f, 1.0f, spin);
			//assert(SUCCEEDED(block->AddTranslation(D3DRMCOMBINE_BEFORE, -0.5f, -0.5, -0.5f)));

	}


	RELEASE(block)
	return TRUE;
}


BOOL RubiksCube::getBeginRamFrame()
{
return getStandingFrame();
}

BOOL RubiksCube::getRamFrame()
{
	LPDIRECT3DRMFRAME3 block;
	D3DVALUE xv = 0.0f;

	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			//Get the current block. 
			block = blocks[x + 3*y + 9*z];
			//Moves the block to its place in the cube. The center block is centered around the origin.
			block->AddTranslation(D3DRMCOMBINE_REPLACE, -1.55f + 1.05f*x, 1.05f*y, -1.55f + 1.05f*z);		

			if (x == 2)
			{
			   if (info.framecount < (2*MAX_STOMP) / 3)
					xv = 6.0f * (info.framecount / (MAX_STOMP * .666f));
			   else xv = 6.0f - (((MAX_STOMP - info.framecount)/(MAX_STOMP * 0.333f)) * 6.0f);
				block->AddTranslation(D3DRMCOMBINE_BEFORE, xv, 0.0f, 0.0f);
			}
			
		  }
			
	RELEASE(block);
	return TRUE;
}

BOOL RubiksCube::getEndRamFrame()
{
return getStandingFrame();
}



BOOL RubiksCube::getHitByProjectileFrame()
{
	LPDIRECT3DRMFRAME3 block;
	// unreferenced:  D3DVALUE xv, yv, zv;
	

	/** Model each individual block **/
	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			//Get the current block. 
			block = blocks[x + 3*y + 9*z];

			block->AddTranslation(D3DRMCOMBINE_BEFORE, -1.55f + 1.05f*x, 1.05f*y,
													  -1.55f + 1.05f * z);
			
			D3DVALUE spin = 0.0f;			
			D3DVALUE mag = 0.628f;
			spin = 0.0f;
			D3DVALUE rnd = (float)(rand() % 100);
			rnd = mag * (rnd / 100.0f);
			spin = (info.framecount * rnd );
			while (spin > 6.28) spin -= 6.28f;

			//assert(SUCCEEDED(block->AddTranslation(D3DRMCOMBINE_BEFORE, 0.5f, 0.5, 0.5f)));
			block->AddRotation(D3DRMCOMBINE_BEFORE, 1.0f, 1.0f, 1.0f, spin);

#if 0
			//-1.55f + 1.05f*x, 
			//1.05f*y, 
			//-1.55f + 1.05f*z);
			xv = -1.55f + 1.05f*x + 0.2f*info.framecount; 
			yv = 1.05*y + (0.2 - 0.05f*info.framecount);
			zv = -1.55f + 1.05f*z;
			if (yv < 0.0f) yv = 0.0f;
			block->AddTranslation(D3DRMCOMBINE_REPLACE, xv, yv, zv);
			//else
			//block->AddTranslation(D3DRMCOMBINE_REPLACE, -1.55f + 1.05f*x, 
//														 1.05f*y, -1.55f + 1.05f * z);

			//assert(SUCCEEDED(block->AddRotation(D3DRMCOMBINE_BEFORE, 0.0f, 1.0f, 0.0f, 0.2f)));
#endif
	  }


	RELEASE(block)
	return TRUE;
}


BOOL RubiksCube::getStandingFrame()
{
	LPDIRECT3DRMFRAME3 block;

	for (int z = 0; z < 3; z++)
	   for (int y = 0; y < 3; y++)
		  for (int x = 0; x < 3; x++)
		  {
			//Get the current block. 
			block = blocks[x + 3*y + 9*z];

			//Moves the block to its place in the cube. The center block is centered around the origin.
			block->AddTranslation(D3DRMCOMBINE_REPLACE, -1.55f + 1.05f*x, 1.05f*y, -1.55f + 1.05f*z);		
		  }
			
	RELEASE(block);
	return TRUE;
}

BOOL RubiksCube::IsPlayer()
{
	return TRUE;
}





/* 
 *  Returns a frame for a single block of the rubiks cube. 
 */
BOOL RubiksCube::getBlockFrame(D3DCOLOR c1, D3DCOLOR c2, D3DCOLOR c3,
								D3DCOLOR c4, D3DCOLOR c5, D3DCOLOR c6,
								LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3* frame,
							    LPDIRECT3DRMFRAME3 parent, D3DVALUE specularPower)
{
	LPDIRECT3DRMMESHBUILDER3 mesh;
	LPDIRECT3DRMMATERIAL2 mat;
	LPDIRECT3DRMMESH cubeMesh;

	//Create the mesh builder
	if (FAILED (rm->CreateMeshBuilder(&mesh)))
		goto error;

	if (FAILED(rm->CreateMaterial(specularPower, &mat)))
		goto error; 
	


	if (!SquareBuilder( 0 ,0, 0,  //Front face
						0, 1, 0,
						1, 1, 0,
						1, 0, 0,
						c1,       //Color
						mesh, mat))
	goto error;

	if (!SquareBuilder( 1 ,0, 0,     //Right Face
				  	    1, 1, 0,
						1, 1, 1,
						1, 0, 1,
						c2,  //Color
						mesh, mat))
	goto error;

	if (!SquareBuilder(1, 0, 1,    //BACK FACE
			  	       1, 1, 1,
					   0, 1, 1,
					   0, 0, 1,
					   c3,  //Color
					   mesh, mat))
	goto error;


	 if (!SquareBuilder(0, 0, 0,  //LEFT FACE
				  	    0, 0, 1,
						0, 1, 1,
						0, 1, 0,
						c4,  //Color
						mesh, mat))
	goto error;

	if (!SquareBuilder(0, 1, 0,  //TOP FACE
				  	   0, 1, 1,
					   1, 1, 1,
				   	   1, 1, 0,
					   c5,  //Color
						mesh, mat))
	goto error;

	if (!SquareBuilder(1, 0, 0,  //BOTTOM FACE
				  	   1, 0, 1,
					   0, 0, 1,
				   	   0, 0, 0,
					   c6,  //Color
						mesh, mat))
	goto error;
  
	if (FAILED(rm->CreateFrame(parent, frame)))
		goto error;

	
	if (FAILED(mesh->CreateMesh(&cubeMesh)))
		goto error;

	if (FAILED((*frame)->AddVisual(cubeMesh)))
	    goto error;


	RELEASE(mesh);
	RELEASE(cubeMesh);
	RELEASE(mat);
	return TRUE;

error:
	RELEASE(mesh);
	RELEASE(cubeMesh);
	RELEASE(mat);
	DISPLAYMSG("Error creating block!");
	return FALSE;
	
}


BOOL RubiksCube::SquareBuilder(D3DVALUE x1, D3DVALUE y1, D3DVALUE z1,
								D3DVALUE x2, D3DVALUE y2, D3DVALUE z2,
								D3DVALUE x3, D3DVALUE y3, D3DVALUE z3,
								D3DVALUE x4, D3DVALUE y4, D3DVALUE z4,
								D3DCOLOR color,
				                LPDIRECT3DRMMESHBUILDER3 meshb,
								LPDIRECT3DRMMATERIAL2 mat)
{
	LPDIRECT3DRMFACE2 cube_face;


	//create the mesh builder
	if (FAILED (meshb->CreateFace(&cube_face)))
		goto generic_error;

	if (FAILED (cube_face->SetColor(color)))
		goto generic_error;

	if (FAILED (cube_face->SetMaterial(mat)))
		goto generic_error;

	if (FAILED (cube_face->AddVertex((D3DVALUE)x1, (D3DVALUE)y1, (D3DVALUE)z1)))
		goto generic_error;
	if (FAILED (cube_face->AddVertex((D3DVALUE)x2, (D3DVALUE)y2, (D3DVALUE)z2)))
		goto generic_error;
	if (FAILED (cube_face->AddVertex((D3DVALUE)x3, (D3DVALUE)y3, (D3DVALUE)z3)))
		goto generic_error;
	if (FAILED (cube_face->AddVertex((D3DVALUE)x4, (D3DVALUE)y4, (D3DVALUE)z4)))
		goto generic_error;


	RELEASE(cube_face);
	return TRUE;

generic_error: 

	DISPLAYMSG("Error creating square in RubiksBlock.cpp!");
	RELEASE(cube_face);
	return FALSE;

}


void RubiksCube::createShadow(LPDIRECT3DRMFRAME3 parent, LPDIRECT3DRM3 rm)
{
	LPDIRECT3DRMMESHBUILDER3 meshb;
	LPDIRECT3DRMMESH mesh;
	LPDIRECT3DRMFACE2 face;

	shadow = NULL;

	//Create the mesh builder
	if (FAILED (rm->CreateMeshBuilder(&meshb)))
	{
		DISPLAYMSG("Unable to create mesh_builder");
		goto error;	
	}

	//create the mesh builder
	if (FAILED (meshb->CreateFace(&face)))
	{
		DISPLAYMSG("Unable to create face");
		goto error;	
	}

	if (FAILED (face->SetColor(D3DRGBA(0.0f, 0.0f, 0.0f, 127.0f))))
		goto error;	

	if (FAILED (face->AddVertex(-1.075f, 0.1f, -1.075f)))
		goto error;	
	if (FAILED (face->AddVertex(-1.075f, 0.1f, 1.075f)))
		goto error;	
	if (FAILED (face->AddVertex(1.075f,  0.1f, 1.075f)))
		goto error;	
	if (FAILED (face->AddVertex(1.075f,  0.1f, -1.075f)))
		goto error;	

	
	if (FAILED(meshb->CreateMesh(&mesh)))
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;	
	}

	if (FAILED(rm->CreateFrame(parent, &shadow)))
	{
		DISPLAYMSG("Unable to create shadow frame");
		goto error;	
	}

	if (FAILED(shadow->AddVisual(mesh)))
	{
		DISPLAYMSG("Unable to add mesh to shadow frame");
		goto error;	
	}

	RELEASE(meshb);
	RELEASE(mesh);
	RELEASE(face);
	return;

error:
	DISPLAYMSG("Error creating shadow!");
	RELEASE(meshb);
	RELEASE(mesh);
	RELEASE(face);
}

BOOL RubiksCube::UpdateListener()
{
	// Update the orientation vector of the listener
	vFront.x = cos( info.head );
	vFront.z = -1.0*sin( info.head );

	if(pSoundSet)
	{
		pSoundSet->SetListenerPosition( info.pos );
		pSoundSet->SetListenerOrientation( vFront, vTop );
	}
	return TRUE;
}



void RubiksCube::PickupPiece(PieceType piece) 
{
	int index;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		centerCube[RED_SQUARE] = true;
		index = GetCubeIndex(RED_SQUARE);
		// Play sound for picking up icon
		DoSound( PICKUP_PIECE );
		break;
	case BLUE_RUBIKS_PIECE:
		centerCube[BLUE_SQUARE] = true;
		index = GetCubeIndex(BLUE_SQUARE);
		// Play sound for picking up icon
		DoSound( PICKUP_PIECE );
		break;
	case GREEN_RUBIKS_PIECE:
		centerCube[GREEN_SQUARE] = true;
		index = GetCubeIndex(GREEN_SQUARE);
		// Play sound for picking up icon
		DoSound( PICKUP_PIECE );
		break;
	case YELLOW_RUBIKS_PIECE:
		centerCube[YELLOW_SQUARE] = true;
		index = GetCubeIndex(YELLOW_SQUARE);
		// Play sound for picking up icon
		DoSound( PICKUP_PIECE );
		break;
	case WHITE_RUBIKS_PIECE:
		centerCube[WHITE_SQUARE] = true;
		index = GetCubeIndex(WHITE_SQUARE);
		// Play sound for picking up icon
		DoSound( PICKUP_PIECE );
		break;
	case ORANGE_RUBIKS_PIECE:
		centerCube[ORANGE_SQUARE] = true;
		index = GetCubeIndex(ORANGE_SQUARE);
		// Play sound for picking up icon
		DoSound( PICKUP_PIECE );
		break;
	case MISSILE:
		currentWeapon = Tornado;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case RAM_OBJECT:
		currentWeapon = Ram;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case TORNADO_OBJECT:
		currentWeapon = Tornado;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case ARCING_OBJECT:
		currentWeapon = ArcingMortar;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case MULTI_SHOT:
		multi_shot = true;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	case RICOCHETE:
		ricochete = true;
		// Play sound for picking up icon
		DoSound( PICKUP_ICON );
		return;
		break;
	default:
		// This wasn't a Rubiks piece so don't do nuthin!
		return;
		break;
	}
	

	//Add the frame to the cube for the block we just picked up
	self_frame->AddChild(blocks[index]);
}

void RubiksCube::DropPiece(PieceType piece) 
{
	int index;

	switch(piece) 
	{
	case RED_RUBIKS_PIECE:
		centerCube[RED_SQUARE] = false;
		index = GetCubeIndex(RED_SQUARE);
		break;
	case BLUE_RUBIKS_PIECE:
		centerCube[BLUE_SQUARE] = false;
		index = GetCubeIndex(BLUE_SQUARE);
		break;
	case GREEN_RUBIKS_PIECE:
		centerCube[GREEN_SQUARE] = false;
		index = GetCubeIndex(GREEN_SQUARE);
		break;
	case YELLOW_RUBIKS_PIECE:
		centerCube[YELLOW_SQUARE] = false;
		index = GetCubeIndex(YELLOW_SQUARE);
		break;
	case WHITE_RUBIKS_PIECE:
		centerCube[WHITE_SQUARE] = false;
		index = GetCubeIndex(WHITE_SQUARE);
		break;
	case ORANGE_RUBIKS_PIECE:
		centerCube[ORANGE_SQUARE] = false;
		index = GetCubeIndex(ORANGE_SQUARE);
		break;
	default:
		// This wasn't a Rubiks piece so don't do nuthin!
		return;
		break;
	}
	// Play drop piece sound
	DoSound( DROP_PIECE );
	
	// Drop a power-up also if you lost a piece
	if(multi_shot)
		multi_shot = FALSE;
	else if(ricochete)
		ricochete = FALSE;

	//Remove the frame from the cube 
	self_frame->DeleteChild(blocks[index]);
}

void RubiksCube::DoSound( SOUND_STATE eState )
{
	// If this rubiks cube is the hero, then update the listener's position
	if( bHero )
		UpdateListener();
	
	switch( eState )
	{
	case ANIM:	// We have a sound associated with an enum

		switch( info.anim )
		{
		case HOPPING:
			// Play bounce sound when cube begins to jump
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now
			}
			break;

		case BACKWARDS:
			// Play bounce sound when cube begins to jump
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now
			}
			break;

		case FIRING:
			// Play firing sound when cube begins to fire
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_BANG, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BANG, info.pos, info.pos ); // Orientation just set to pos for now
			}
			break;

		case ACTION2:
			// Play tornado sound when cube begins to tornado attack
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				switch( GetCurrentWeapon() )
				{
				  case Tornado:
					if( lpdsb == NULL )
					{
						if( bHero )
							pSoundSet->Play( HERO_SCREAM, info.pos, info.pos, &lpdsb ); // Orientation just set to pos for now  	
						else
							pSoundSet->Play( SCREAM, info.pos, info.pos, &lpdsb ); // Orientation just set to pos for now
					}
					break;

				case ArcingMortar:
					if( bHero )
						pSoundSet->Play( HERO_CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
					else
						pSoundSet->Play( CANNON, info.pos, info.pos ); // Orientation just set to pos for now
					break;

				case Ram:
					if( bHero )
						pSoundSet->Play( HERO_CLANK, info.pos, info.pos ); // Orientation just set to pos for now  	
					else
						pSoundSet->Play( CLANK, info.pos, info.pos ); // Orientation just set to pos for now
					break;
				}			
			}
			break;

		case ENDACTION2:
			if( info.framecount == ANIMATION_STEP_SIZE )
			{
			  if( GetCurrentWeapon() == Tornado )
			  {
				if( lpdsb )
				{
				  lpdsb->Stop();
				  RELEASE( lpdsb );

				  lpdsb = NULL;
				}
			  }

			}
			
			break;


		case COLLIDEWITHPROJECTILE:
			// Play collision with projectile sound
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( CANNON, info.pos, info.pos ); // Orientation just set to pos for now  	
			}
			break;
		
		case COLLIDEWITHPLAYER:
			// Play collision sound when cube collides with other player
			if( info.framecount == ANIMATION_STEP_SIZE ) 
			{
				if( bHero )
					pSoundSet->Play( HERO_SBOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
				else
					pSoundSet->Play( BOUNCE, info.pos, info.pos ); // Orientation just set to pos for now  	
			}
			break;

		default:
			break;
		}
		break;

	// The rest of the sounds are not animation sounds...
	case PICKUP_ICON:
		if( bHero )
			pSoundSet->Play( HERO_GREATBABY, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( GREATBABY, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	case DROP_ICON:
		if( bHero )
			pSoundSet->Play( HERO_LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	case PICKUP_PIECE:
		if( bHero )
			pSoundSet->Play( HERO_WIN, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( WIN, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	case DROP_PIECE:
		if( bHero )
			pSoundSet->Play( HERO_LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		else
			pSoundSet->Play( LOSE, info.pos, info.pos ); // Orientation just set to pos for now  	
		break;

	default:
		break;
	}


	return;

}


