/**
  *
  * RubiksCube.h  -- Defines a class that draws a Rubiks Cube in
  *                  Direct3D.
  *
  * Author: Kevin Audleman
  *
  */

#ifndef  __RUBIKS_CUBE__
#define  __RUBIKS_CUBE__


#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <ddraw.h>

#include <stdio.h>
#include <math.h>
#include <assert.h>

#include "rubiksInfo.h"
#include "ClientPlayer.h"
#include "RubiksSoundSet.h"
#include "SoundTypes.h"

#ifndef RELEASE
#define RELEASE(x) if (x != NULL) { x->Release(); x = NULL; } 
#endif

// Macro to display a message box containing the given string. 
#ifndef DISPLAYMSG
#define DISPLAYMSG(x) MessageBox(NULL, x, "Rubiks Cube", MB_OK);
#endif




class RubiksCube  : public CClientPlayer {

public:
	virtual void DoSound( SOUND_STATE eState = ANIM );
	BOOL UpdateListener();
	RubiksCube(LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3 parent, BOOL centerPieces[6], 
		       char playername[], SoundSet *pSound = NULL, BOOL isHero = FALSE);
	~RubiksCube();

	//Sets our information based on the message we receive
	void ReceiveMessage(rubiks_info *message);

	//Call this function every time the Player app wants
	//to refresh this cube. 
	void Update();
	
	// return if we're a player or not.
	BOOL IsPlayer();

	virtual void PickupPiece(PieceType piece);
	virtual void DropPiece(PieceType piece);

protected:
	LPDIRECTSOUND3DBUFFER lpdsb3d;
	LPDIRECTSOUNDBUFFER lpdsb;
	LPDIRECT3DRMFRAME3 lpPlayerIconFrame;
///	void getPlayerIcon( LPDIRECT3DRM3 rm, LPLPDIRECT3DRMFRAME3 frame, LPDIRECT3DRMFRAME3 parent );
	//All of the information for this block. This is the message that
	//will be passed in between the server and player.
	//rubiks_info *info;

	D3DVECTOR vFront;
	D3DVECTOR vTop;
	SoundSet * pSoundSet;

	//Movement functions
	BOOL getHoppingFrame(bool forwards);
	BOOL getStandingFrame();

	BOOL getBeginFiringFrame();
	BOOL getFiringFrame();
	BOOL getEndFiringFrame();

	BOOL getTornadoFrame(int type);
	BOOL getHitByProjectileFrame();

	BOOL getBeginRamFrame();
	BOOL getRamFrame();
	BOOL getEndRamFrame();

	//Builds the hopping animation ANIMATION nodes
	void createHoppingAnimation();
	void createFiringAnimation();

	//Builds a frame with a single block in it.
	BOOL getBlockFrame(D3DCOLOR c1, D3DCOLOR c2, D3DCOLOR c3,
					   D3DCOLOR c4, D3DCOLOR c5, D3DCOLOR c6,
					   LPDIRECT3DRM3 rm, LPDIRECT3DRMFRAME3* frame,
					   LPDIRECT3DRMFRAME3 parent, D3DVALUE specularPower);

	//Builds a face.
	BOOL SquareBuilder( D3DVALUE x1, D3DVALUE y1, D3DVALUE z1,
						D3DVALUE x2, D3DVALUE y2, D3DVALUE z2,
						D3DVALUE x3, D3DVALUE y3, D3DVALUE z3,
						D3DVALUE x4, D3DVALUE y4, D3DVALUE z4,
						D3DCOLOR color, LPDIRECT3DRMMESHBUILDER3 meshb,
						LPDIRECT3DRMMATERIAL2 mat);

	void createShadow(LPDIRECT3DRMFRAME3 parent, LPDIRECT3DRM3 rm);

	/************ Variables *****************/

	//Frames
	LPDIRECT3DRMFRAME3 parent_frame;
	LPDIRECT3DRMFRAME3 *blocks;
	LPDIRECT3DRMFRAME3 shadow;

	//Hopping animations
	LPDIRECT3DRMANIMATION2 block_rotate, stack_move[3];  
	LPDIRECT3DRMANIMATION2 cannon;

	

	BOOL modified;  //Set when ReceiveMessage is called 
	D3DVALUE  specularPower;
	// moved up to ClientPlayer: char name[10];

};



#endif