#ifndef _RUBIKSSOUNDSET_H_
#define _RUBIKSSOUNDSET_H_

#include <mmsystem.h>
#include <dsound.h>
#include <d3dtypes.h>
#include <list>

#include "wave.h"
#include "debug.h"
#include "SoundTypes.h"

// Macro to display a message box containing the given string. 
#ifndef DISPLAYMSG
//#define DISPLAYMSG(x) MessageBox(NULL, x, "Rubiks ICON", MB_OK);
#define DISPLAYMSG(x) OutputDebugString(x)
#endif

// Macro to release an object. 
#ifndef	RELEASE
#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;} 
#endif

typedef enum _SOUND_STATE {
	ANIM,
	PICKUP_ICON,
	DROP_ICON,
	PICKUP_PIECE,
	DROP_PIECE
} SOUND_STATE;


typedef enum _BUFFER_TYPE {
	STATIC,		
	STREAMING,
	STATIC_3D,
	STREAMING_3D
} BUFFER_TYPE;

typedef struct _BUFFERINFO {
	LPDIRECTSOUNDBUFFER lpdsb;
	LPDIRECTSOUND3DBUFFER lpdsb3d;
} BUFFERINFO, *PBUFFERINFO;

typedef std::list<PBUFFERINFO> BUFFERLIST;
 
class SoundSet 
{
	public:
		BOOL Play( SOUND eSound, D3DVECTOR vPos, D3DVECTOR dOrient, LPLPDIRECTSOUNDBUFFER lplpdsb );
		BOOL Play( SOUND eSound );
		
		PBUFFERINFO pBufferInfo;
		
		SoundSet( HWND hwnd, GUID *pguid );
		SoundSet();
		~SoundSet();
	
		BOOL Play( SOUND eSound, D3DVECTOR vPos, D3DVECTOR dOrient );	// Play given sound 
		BOOL Play( SOUND eSound, D3DVECTOR vPos, D3DVECTOR dOrient, LPLPDIRECTSOUNDBUFFER lplpdsb, LPLPDIRECTSOUND3DBUFFER lplpdsb3d );

		BOOL CleanUpBuffers( );	// Releases all sound buffers and 3D buffers
		BOOL StopBuffer( LPDIRECTSOUNDBUFFER lpdsb ); // Makes the buffer stop playing

		BOOL SetListenerOrientation( D3DVECTOR vFront, D3DVECTOR vTop );
		BOOL SetListenerVelocity( D3DVECTOR vVelocity );
		BOOL SetListenerPosition( D3DVECTOR vPos );
		void ModifyListenerRolloff( D3DVALUE nRolloff );
	
	private:		
		BOOL Init3DListener();						// Create and init 3D listener
		BOOL InitDSound( HWND hwnd, GUID *pguid );	// Initialize Direct Sound
		BOOL CreateBuffer( BUFFER_TYPE eBufType );	// Create sound buffer of correct type
		BOOL SetUpBuffer( char *pcSoundFileName, BUFFER_TYPE eBufType );

	private:
		void ReleaseBufferInfo( PBUFFERINFO pBufInfo );
		D3DVECTOR m_vOrientation;
		D3DVECTOR m_vPos;			// Position of a 3DBuffer;
		
		LPDIRECTSOUND				lpds;
		DSBUFFERDESC                dsbdesc;
		LPDIRECTSOUNDBUFFER         lpdsb;
		LPDIRECTSOUNDBUFFER         lpdsbPrimary;
		LPDIRECTSOUND3DBUFFER		lpdsb3d;
		LPDIRECTSOUND3DLISTENER		lp3dListener;
		WAVEFORMATEX                *pwfx;
		HMMIO                       hmmio;
		MMCKINFO                    mmckinfoData, mmckinfoParent;

		BUFFERLIST					BufferList;
	
};
#endif //_RUBIKSSOUNDSET_H_