//-----------------------------------------------------------------------------
// File: client_objects.cpp
//
//
// 4-1-1999:	Grant
//-----------------------------------------------------------------------------

#include "client_objects.h"
#include "RubicsDirect3D.h"
#include "RubicsObjects.h"

// This is the implemenation file for our 2 Client Object Classes

/***************************************************************************
 *
 * CClientObjects (The movable ones)
 *
 **************************************************************************/

/* GetFrame
 */

void CClientObjects::DoSound( SOUND_STATE eState ) 
{
	
}

BOOL CClientObjects::GetFrame( LPDIRECT3DRMFRAME3 *pFrame )
{
	*pFrame = self_frame;
	//We're making another pointer to this frame, so increment the
	//reference count.
	if(self_frame)
		self_frame->AddRef();
	return TRUE;
}

/* GetPosition
 */
rubiks_info* CClientObjects::GetInfo()
{
	return &info;
}

/* IsPlayer
 */
BOOL CClientObjects::IsPlayer()
{
	return FALSE;
}

BOOL CClientObjects::IsRubiksPiece()
{
	return FALSE;
}

BOOL CClientObjects::IsProjectile()
{
	return FALSE;
}

PieceType CClientObjects::GetType() {

	return pType;

}
/* Function to respond to a message. All objects will
   receive the same message (a rubiks_info struct), but
   some of them may ignore certain variables. */
void CClientObjects::ReceiveMessage(rubiks_info* info) {

}
		
/* Function called every time the object should update
   its frame. This will probably be called in RenderFrame
   shortly after a ReceiveMessage call was made. */
void CClientObjects::Update() {
}

/***************************************************************************
 *
 * CClientObjectsNonMovable
 *
 **************************************************************************/
CClientObjectsNonMovable::CClientObjectsNonMovable() {

}

CClientObjectsNonMovable::CClientObjectsNonMovable(LPDIRECT3DRM3 lpD3DRM, LPDIRECT3DRMFRAME3 Parent, 
												   Position pos, float orientation, float scale, LPVOID dxfile, SoundSet *pSS )
{
	LPDIRECT3DRMMESHBUILDER3 meshBuilder;
	LPDIRECT3DRMMESH		 mesh;
	
	self_frame = NULL;
	pSoundSet = pSS;

	// Create the Mesh Builder for the Wagon
	if (FAILED( lpD3DRM->CreateMeshBuilder( &meshBuilder ) ) )
	{
		DISPLAYMSG("Failed to Create Mesh Builder");
		goto error;
	}

	if( dxfile != NULL) {		// Check to see if we have an X File to load
		// Load the X file
		if( FAILED( meshBuilder->Load((LPVOID) dxfile ,NULL,D3DRMLOAD_FROMFILE, NULL,NULL) ) )
		{
			DISPLAYMSG("Failed to load .X file");
			goto error;
		}
	}
	else goto error;

	// Create Mesh to be used for the Wagon
	if (FAILED( meshBuilder->CreateMesh( &mesh ) ) )
	{
		DISPLAYMSG("Unable to create mesh");
		goto error;
	}
	
	// Create frame that the outside world will see
	if (FAILED(lpD3DRM->CreateFrame( Parent, &self_frame ) ) )
	{
		DISPLAYMSG("Unable to create self frame");
		goto error;
	}

	if( FAILED( self_frame->AddVisual( mesh ) ) )
	{
		DISPLAYMSG( "Failed to add mesh builder to frame" );
		goto error;
	}

	if (FAILED(self_frame->SetPosition(Parent, pos.x, pos.y, pos.z)))
				DISPLAYMSG("Error setting Position on Non-Movable Object");

	if (FAILED(self_frame->SetOrientation(Parent, (cos(orientation)), 
										  0,
										  (sin(orientation)), 0, 1, 0)))
				DISPLAYMSG("Error setting Orientation on Non-Movable Object");

	this->ScaleMe(scale, scale, scale);

error:
	RELEASE( meshBuilder );
	RELEASE( mesh );
 }

CClientObjectsNonMovable::~CClientObjectsNonMovable() 
{
	RELEASE( self_frame );
}

 
// GetFrame
BOOL CClientObjectsNonMovable::GetFrame( LPDIRECT3DRMFRAME3 *pFrame )
{
	*pFrame = self_frame;
	//We're making another pointer to this frame, so increment the
	//reference count.
	if(self_frame)
		self_frame->AddRef();
	return TRUE;
}

CClientObjectsNonMovable::ScaleMe(float fx, float fy, float fz )
{
	self_frame->AddScale(D3DRMCOMBINE_BEFORE, fx, fy, fz);;
}






