#ifndef __CLIENTOBJECTS_H__
#define __CLIENTOBJECTS_H__

/**************************************************************************
 *
 * This file defines 2 classes.
 * CClientObjects all our movable objects 
 * &
 * CClientObjectsNonMovable all our non-movable objects.
 *
 * Author: Grant Jorde
 */

#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <ddraw.h>
#include <stdio.h>
#include <math.h>

#include "colors.h"
#include "rubiksinfo.h"
#include "messages.h"
#include "SoundTypes.h"
#include "RubiksSoundSet.h"

#ifndef RELEASE
#define RELEASE(x) if (x != NULL) { x->Release(); x = NULL; } 
#endif

// Macro to display a message box containing the given string. 
#ifndef DISPLAYMSG
//#define DISPLAYMSG(x) MessageBox(NULL, x, "Client Object", MB_OK);
#define DISPLAYMSG(x) OutputDebugString(x)
#endif

class CClientObjects {

	public:
		// Returns a frame that contains the Object
		virtual BOOL GetFrame( LPDIRECT3DRMFRAME3 *pFrame );

		/* Function to respond to a message. All objects will
		   receive the same message (a rubiks_info struct), but
		   some of them may ignore certain variables. */
		//virtual void ReceiveMessage(rubiks_info* info);
		virtual void ReceiveMessage(rubiks_info* info);
		
		/* Function called every time the object should update
		   its frame. This will probably be called in RenderFrame
		   shortly after a ReceiveMessage call was made. */
		//virtual void Update();
		virtual void Update();

		//This function performs the sound
		virtual void DoSound( SOUND_STATE eState = ANIM );
		
		// functions Hasinur added
		virtual BOOL IsPlayer();
		virtual BOOL IsRubiksPiece();
		virtual BOOL IsProjectile(); // but Dave added this one...
		PieceType GetType();

		rubiks_info* GetInfo();


	protected:
		LPDIRECT3DRMFRAME3 self_frame;
		//All of the information for this block, like positions. This is the message that
		//will be passed in between the server and player.
		rubiks_info info;
		PieceType pType;
};


/*****************************************************************************************/



class CClientObjectsNonMovable {

	public:
		CClientObjectsNonMovable();
		CClientObjectsNonMovable( LPDIRECT3DRM3 lpD3DRM, LPDIRECT3DRMFRAME3 Parent, Position pos, float orientation, float scale, LPVOID dxfile, SoundSet *pSS );
		~CClientObjectsNonMovable( );
		
		// Scale the 3DRM frame
		ScaleMe(float fx, float fy, float fz);

		// Returns a frame that contains the Object
		virtual BOOL GetFrame( LPDIRECT3DRMFRAME3 *pFrame );

	protected:
		SoundSet * pSoundSet;
		LPDIRECT3DRMFRAME3 self_frame;
};

#endif // __CLIENTOBJECTS_H__
