
#include <windows.h>
#include <windowsx.h>
#include <wtypes.h>
#include <cguid.h>

#include <dplay.h>
#include "devices.h"
#include "resource.h"
#include "RubicsDirect3D.h"

HRESULT ChooseDeviceUsingDialog(HINSTANCE hInstance, LPGUID * lplpGuid)
{
	// ask user for connection settings
	if (DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DEVICEDIALOG),
					   NULL, (DLGPROC) DeviceWndProc, (LPARAM) lplpGuid))
	{
		return (DP_OK);
	}
	else
	{
		return (DPERR_USERCANCEL);
	}
}





HRESULT ChooseDrawDeviceUsingDialog(HINSTANCE hInstance, LPGUID * lplpGuid)
{
	// ask user for connection settings
	if (DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DEVICEDIALOG),
					   NULL, (DLGPROC) DrawDeviceWndProc, (LPARAM) lplpGuid))
	{
		return (DP_OK);
	}
	else
	{
		return (DPERR_USERCANCEL);
	}
}


BOOL CALLBACK DeviceWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	
	HRESULT	hr;
	static LPDIRECTDRAW lpDD;
    static LPDIRECT3D lpD3D;
	static LPGUID * lplpGuid;
    // unreferenced! int i;
	int iIndex;

	switch(uMsg)
	{
	case WM_INITDIALOG:
	// save the connection info pointer
		lplpGuid = (LPGUID *) lParam;
		/*
		 * Create a DirectDraw object and query for the Direct3D interface 
		 * to use to enumerate the drivers.
		 */
		hr = DirectDrawCreate(NULL, &lpDD, NULL);
		if (hr != DD_OK) {
			DISPLAYMSG("Creation of DirectDraw HEL failed.\n%s");
			return FALSE;
		}
		hr = lpDD->QueryInterface(IID_IDirect3D, (void**) &lpD3D);
		if (hr != DD_OK) {
			DISPLAYMSG("Creation of Direct3D interface failed.\n%s");
			lpDD->Release();
			return FALSE;
		}
		
		// Enumerate the drivers. 
		hr = lpD3D->EnumDevices(enumDeviceFunc, hWnd);
		if (FAILED(hr)) {
			DISPLAYMSG("Enumeration of drivers failed.\n%s");
			return FALSE;
		}
		RELEASE(lpD3D);
		RELEASE(lpDD);
		return true;		
    
	/* unreferenced! SETUP_FAILURE:
		MessageBox(NULL, "This application requires DirectX 5 or later.", NULL, MB_OK);
		EndDialog(hWnd, FALSE);
		return true;*/		

	case WM_DESTROY:
		// delete information stored along with the lists
		DeleteDeviceList(hWnd);
		return true;		

	
	case WM_COMMAND:
	switch(LOWORD(wParam))
	{		
		case IDOK:
				
			iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0);
			if (iIndex == CB_ERR)
				goto OK_FAILURE;

			iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_GETITEMDATA,
								(WPARAM) iIndex, (LPARAM) 0);
			if ((iIndex == CB_ERR) || (iIndex == 0))
				goto OK_FAILURE;

			*lplpGuid = (_GUID *) GlobalAllocPtr(GHND, sizeof(GUID));
			
			memcpy(*lplpGuid, ((LPGUID) iIndex), sizeof(GUID));
			
			// dismiss dialog if we succeeded in joining
			//DeleteDeviceList(hWnd);
			EndDialog(hWnd, TRUE);
		OK_FAILURE:
			return true;
	}

	break;
	}

	// Allow for default processing
	return FALSE;
}

BOOL CALLBACK DrawDeviceWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	
	// unreferenced! HRESULT	hr;
    
	static LPGUID * lplpGuid;
    // unreferenced! int i;
	int iIndex;

	switch(uMsg)
	{
	case WM_INITDIALOG:
	// save the connection info pointer
		lplpGuid = (LPGUID *) lParam;
		/*
		 * Create a DirectDraw object and query for the Direct3D interface 
		 * to use to enumerate the drivers.
		 */
		
		// Enumerate the drivers. 
		DirectDrawEnumerate(enumDrawDeviceFunc, hWnd);
		iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_GETCOUNT ,
								(WPARAM) 0, (LPARAM) 0);		
		if(iIndex < 2) {
			*lplpGuid = NULL;
			EndDialog(hWnd, TRUE);
		}
		return true;		
    
	/* Unreferenced! SETUP_FAILURE:
		MessageBox(NULL, "This application requires DirectX 5 or later.", NULL, MB_OK);
		EndDialog(hWnd, FALSE);
		return true;*/		

	case WM_DESTROY:
		// delete information stored along with the lists
		DeleteDeviceList(hWnd);
		return true;		

	
	case WM_COMMAND:
	switch(LOWORD(wParam))
	{		
		case IDOK:
				
			iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0);
			if (iIndex == CB_ERR)
				goto OK_FAILURE;

			iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_GETITEMDATA,
								(WPARAM) iIndex, (LPARAM) 0);
			if ((iIndex == CB_ERR))
				goto OK_FAILURE;

			if(iIndex != 0) {
				*lplpGuid = (_GUID *) GlobalAllocPtr(GHND, sizeof(GUID));
				memcpy(*lplpGuid, ((LPGUID) iIndex), sizeof(GUID));
			} else *lplpGuid = NULL;
			
			// dismiss dialog if we succeeded in joining
			EndDialog(hWnd, TRUE);
		OK_FAILURE:
			return true;
	}

	break;
	}

	// Allow for default processing
	return FALSE;
}




/************************************************************************/
/*
 * enumDeviceFunc
 * Callback function which records each usable D3D driver's name and 
 * GUID. It selects the preferred driver on the system by choosing
 * hardware drivers over software drivers, and color lights over 
 * monochrome lights. It sets info.CurrDriver to indicate the 
 * preferred driver.
 */

HRESULT WINAPI enumDeviceFunc(LPGUID lpGuid, 
                                     LPSTR lpDeviceDescription, 
                                     LPSTR lpDeviceName,
                                     LPD3DDEVICEDESC lpHWDesc, 
                                     LPD3DDEVICEDESC lpHELDesc, 
                                     LPVOID lpContext)
{    
	HWND			hWnd = (HWND) lpContext;
	LRESULT 		iIndex;
	
	LPGUID lpDeviceGUID;
	 /*
     * Record this driver's GUID and name.
     */
    
	iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_ADDSTRING, 0, 
								(LPARAM) lpDeviceName);
	if (iIndex == CB_ERR)
		goto FAILURE;

	// make space for connection shortcut
	lpDeviceGUID = (_GUID *) GlobalAllocPtr(GHND, sizeof(GUID));
	
	// store pointer to connection shortcut in combo box
	memcpy(lpDeviceGUID, lpGuid, sizeof(GUID));
	SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_SETITEMDATA, (WPARAM) iIndex, 
									(LPARAM) lpDeviceGUID);
FAILURE:
    return (D3DENUMRET_OK);
}


/************************************************************************/
/*
 * enumDrawDeviceFunc
 * Callback function which records each usable D3D driver's name and 
 * GUID. It selects the preferred driver on the system by choosing
 * hardware drivers over software drivers, and color lights over 
 * monochrome lights. It sets info.CurrDriver to indicate the 
 * preferred driver.
 */

BOOL WINAPI enumDrawDeviceFunc(LPGUID lpGuid, 
                                     LPSTR lpDriveDescription, 
                                     LPSTR lpDriverName, 
                                     LPVOID lpContext)
{    
	HWND			hWnd = (HWND) lpContext;
	LRESULT 		iIndex;
	
	LPGUID lpDeviceGUID;
	 /*
     * Record this driver's GUID and name.
     */
    
	iIndex = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_ADDSTRING, 0, 
								(LPARAM) lpDriverName);
	if (iIndex == CB_ERR)
		goto FAILURE;

	// make space for connection shortcut
	lpDeviceGUID = (_GUID *) GlobalAllocPtr(GHND, sizeof(GUID));
	
	// store pointer to connection shortcut in combo box
	if(lpGuid)
		memcpy(lpDeviceGUID, lpGuid, sizeof(GUID));
	else lpDeviceGUID = lpGuid;
	SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_SETITEMDATA, (WPARAM) iIndex, 
									(LPARAM) lpDeviceGUID);
FAILURE:
    return (DDENUMRET_OK);
}


void DeleteDeviceList(HWND hWnd)
{
	WPARAM  i;
	LONG	lpData;
	
	// destroy the GUID's stored with each session name
	i = 0;
	while (TRUE)
	{
		// get data pointer stored with item
		lpData = SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_GETITEMDATA,
									(WPARAM) i, (LPARAM) 0);
		if (lpData == CB_ERR)			// error getting data
			break;

		if (lpData == 0) {				// no data to delete
			i += 1;			
			continue;
		}

		GlobalFreePtr((LPVOID) lpData);
		i += 1;
	}

	// delete all items in list
	SendDlgItemMessage(hWnd, IDC_DEVICECOMBO, CB_RESETCONTENT,
								(WPARAM) 0, (LPARAM) 0);
}
