#include <windows.h>
#include <windowsx.h>
#include <dmusicc.h>
#include <dmusici.h>


//step 2 create performance
#include "dmusic.h"

#define MUSIC1 "Smbtheme.mid"
#define MUSIC2 "rocky.mid"
#define MUSIC3 "rubiks.mid"
#define MUSIC4 "kngtridr.mid"
#define MUSIC5 "ff2overworld2.mid"

#define NUM_MUSIC_FILES  5

/* 
	Convert from multibyte format to Unicode using the following macro:
*/
#ifndef	MULTI_TO_WIDE
#define MULTI_TO_WIDE( x,y )  MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, y, -1, x, _MAX_PATH );

#endif

DMusic::DMusic(LPCSTR dir) {
	g_pMIDIseg = NULL;
	currentIndex = 0;
	if (FAILED(CoInitialize(NULL)))
	{    // Terminate the application.
		goto error;
	}   // Else full speed ahead!

	 g_pPerf= CreatePerformance();
	if (g_pPerf == NULL){
    // Failure -- performance not created
		goto error;
	}

	if (FAILED(g_pPerf->Init(NULL, NULL, NULL)))
	{
    // Failure -- performance not initialized
		goto error;
	}

	
	if (FAILED(g_pPerf->AddPort(NULL))){    
		// Failure -- port not initialized
		goto error;
	}

	g_pLoader = CreateLoader();
	if (g_pLoader == NULL){
    // Failure -- loader not created
		goto error;
	}
	if(SetDirectory(dir)== FALSE){

		goto error;
	}

	return;
error:
	FreeDirectMusic();

}

// destructor
DMusic::~DMusic() {

	FreeDirectMusic();
}

BOOL
DMusic::SetDirectory(LPCSTR dir){
	

	if (dir == NULL) {	
		char szDir[_MAX_PATH];
		//WCHAR wszDir[_MAX_PATH];
 
		if(_getcwd( szDir, _MAX_PATH ) == NULL)
		{
			return FALSE;
		}

		MULTI_TO_WIDE(wszDir, szDir);
	}
	else MULTI_TO_WIDE(wszDir, dir);

    HRESULT hr = g_pLoader->SetSearchDirectory(GUID_DirectMusicAllTypes,
        wszDir, FALSE);
    if (FAILED(hr)) 
    {
        return FALSE;
    }

	return TRUE;



}


// loads the midi file

void 
DMusic::LoadMidi(LPCSTR  name) {
	
	WCHAR wszDir[_MAX_PATH];
	MULTI_TO_WIDE(wszDir, name);
	
	if (g_pMIDIseg){ 
		
		g_pMIDIseg->Release();    
		g_pMIDIseg = NULL;
	} 
	
	if (g_pLoader)
	{    
		 g_pMIDIseg = LoadMIDISegment(wszDir);
		 if(g_pMIDIseg){
			g_pMIDIseg->SetParam(GUID_StandardMIDIFile,
            -1, 0, 0, (void*)g_pPerf);

			g_pMIDIseg->SetParam(GUID_Download, -1, 0, 0, (void*)g_pPerf); 

		 }
	}


}

// create the performance
IDirectMusicPerformance* 
DMusic::CreatePerformance(void){
    
	IDirectMusicPerformance* pPerf;     
	
	if (FAILED(CoCreateInstance(
            CLSID_DirectMusicPerformance,
            NULL,
            CLSCTX_INPROC, 
            IID_IDirectMusicPerformance,
            (void**)&pPerf        )))   
	{        
		pPerf = NULL;    
	}
    return pPerf;
}
// step 3 create the loader

IDirectMusicLoader* 
DMusic::CreateLoader(void){    
	
	IDirectMusicLoader* pLoader; 
    
	if (FAILED(CoCreateInstance(            
		CLSID_DirectMusicLoader,
            NULL,            
			CLSCTX_INPROC,             
			IID_IDirectMusicLoader,
            (void**)&pLoader        )))  
	{        
		pLoader = NULL;   
	}
    return pLoader;
}

// step 4 load a midi file
IDirectMusicSegment* 
DMusic::LoadMIDISegment( WCHAR* wszMidiFileName )
{    
	DMUS_OBJECTDESC ObjDesc; 
    IDirectMusicSegment* pSegment = NULL;

	// set the directory to current working directory
	 
 
	
	// set the object properties
    ObjDesc.guidClass = CLSID_DirectMusicSegment;
    ObjDesc.dwSize = sizeof(DMUS_OBJECTDESC);
    wcscpy( ObjDesc.wszFileName, wszMidiFileName );
    ObjDesc.dwValidData = DMUS_OBJ_CLASS | DMUS_OBJ_FILENAME;
	if(g_pLoader)
		g_pLoader->GetObject(&ObjDesc,
            IID_IDirectMusicSegment, (void**) &pSegment); 
	
    return pSegment;
 
} // End of LoadSegment()


// play the midi file
void 
DMusic::PlayMidi(LPCSTR fileName,BOOL repeat) {
	if(!fileName)
	  LoadMidi(MUSIC1);
	else
		LoadMidi(fileName);
	// GJ unreferenced: MUSIC_TIME length;
	if (g_pMIDIseg){

		// play the file in a loop if repeat is true
		if(repeat){
			g_pMIDIseg->SetRepeats(-1);
		}

		g_pPerf->PlaySegment(g_pMIDIseg, 0, 0, &g_pSegState);
	}
}

// call this function to stop the midi

void 
DMusic::StopMidi() {


	g_pPerf->Stop(NULL, NULL, 0, 0);

}

void
DMusic::NextMidi() {
	currentIndex = (currentIndex+1)% NUM_MUSIC_FILES;
	
	switch(currentIndex) {
	case 0:
		PlayMidi(MUSIC1,TRUE);
		break;
	case 1:
		PlayMidi(MUSIC2,TRUE);
		break;
	case 2:
		PlayMidi(MUSIC3,TRUE);
		break;
	case 3:
		PlayMidi(MUSIC4,TRUE);
		break;
	case 4:
		PlayMidi(MUSIC5,TRUE);
		break;
	default:
		break;

	}
}

// this function is called the destructor
HRESULT 
DMusic::FreeDirectMusic(){
    // If there is any music playing, stop it. This is 
    // not really necessary, because the music will stop when
    // the instruments are unloaded or the performance is    // closed down.
	if (g_pPerf)
		g_pPerf->Stop( NULL, NULL, 0, 0 ); 
    
	// Unload instruments  this will cause silence.
    // CloseDown unloads all instruments, so this call is also not 
    // strictly necessary.
    if(g_pMIDIseg) {
		g_pMIDIseg->SetParam(GUID_Unload, -1, 0, 0, (void*)g_pPerf); 
    
		// Release the segment.    
		g_pMIDIseg->Release(); 
		g_pMIDIseg = NULL;
	}
	if (g_pPerf) {
		// CloseDown and Release the performance object.   
		g_pPerf->CloseDown();
    
		g_pPerf->Release();     // Release the loader object.
		g_pPerf = NULL;
	}
    if(g_pLoader){
		g_pLoader->Release();
		g_pLoader = NULL;
	}
	// Release COM.    
	CoUninitialize(); 
    
	return S_OK;
}



















































