#ifndef _OVERLAY_H_
#define _OVERLAY_H_

#include <time.h>
#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <ddraw.h>
#include <stdio.h>
#include <map>
#include <windowsx.h>
#include <wtypes.h>
#include "colors.h"
#include "RubicsMain.h"
#include "Messages.h"
#include "rubiksInfo.h"
#include "ClientPlayer.h"


#ifndef	RELEASE
#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;} 
#endif
//typedef map <long, CClientObjects*, less<long> > ObjectMap;

// TYPE OF bitmaps we have
enum BTYPE{BAR, CUBE,CUBEICON,BOUNDARY,RADAR,MARBLE,BOX,HEALTHBAR,FRAME,WEAPON1,NUMBER};

class Overlay {

public:
	// constructor
	Overlay (IDirectDraw4* pd, LPDIRECTDRAWSURFACE4 lpddsPrimary, HWND hw);
	// destructor
	~Overlay();
	
	void DrawAllPlayersStat(ObjectMap &mapPieces,PieceID myID);
	void DrawMyStat(ObjectMap &mapPieces,PieceID myID);
	void DrawRadar(PieceID myID, ObjectMap &mapPieces);
	
	// prints the ending screen -- losers or winners
	void DisplayEndingOverlay( int myTeam);
	void InitEndingOverlay(PieceID id,int team, PieceType piece,BOOL won);
	// name1 - victim
	// name2 -- attacker
	void DisplayMessage();
	void SetMessageIndex(char* name1,char*name2,int index);

	// if true draw the ending overlay
	BOOL drawEndingOverlay;
private:

	void RestoreAll();
	// draws it on the surface
	void DoBlt(BTYPE b, int frameNumber, int x, int y, BOOL rotate);
	// loads the bitmap
	IDirectDrawSurface4*  LoadBitmap(LPCSTR szBitmap);
	HRESULT ReLoadBitmap(IDirectDrawSurface4 * pdds, LPCSTR szBitmap);
	// copies the bitmap to a surface
	int  CopyBitmap(IDirectDrawSurface4 *surface,HBITMAP hbm,int x, int y,int dx, int dy);

	// prints message to  a paticular coordinate
	int  PrintMessage(char* msg, int x, int y,COLORREF col, BOOL bg);

	int GetCubeFrame(PieceType type, int index);

	// creates a font
	void SetFont(int size,DWORD weight);
	// draws the player into the rader
	void PutObjectsInRadar(BTYPE type,int frame, int x, int y,BOOL blink, int team);
	// tell which color not to copy to the surface
	HRESULT SetColorKey(IDirectDrawSurface4 *surface,COLORREF rgb);
	DWORD ColorMatch( IDirectDrawSurface4 *surface,COLORREF rgb);
	// calculates the position for players in the radar
	BOOL  CalcPos(struct pos_* hisPos,struct pos_ *myPos,D3DVALUE head);
	void  DrawWeapon(CClientPlayer* player, int &x, int & y);
	void FreeObjects();
	int                  freeIndex[10];
	LPDIRECTDRAWSURFACE4 ovSurface;
	HWND                 ovhWnd;
   

	// direct draw surface for our bitamp
    IDirectDrawSurface4    *pdds;

	// direct draw surface for winning screen bitmap
	IDirectDrawSurface4    *endScreen;

	// surfaces for the weapon
	IDirectDrawSurface4	   *tornado,*ram,*threeAndRico, *ricochet, *mortar,*hoaming;
	IDirectDrawSurface4    *oneShot,*oneShotChecker,*threeRicoChecker;
	IDirectDrawSurface4    *threeShot,*threeShotChecker,*mortarChecker,*evil,*reallyEvil;
	//IDirectDrawSurface4

	
	// a pointer to our original direct draw object
	IDirectDraw4           *pdd;

	// font object
	HFONT              textFont;
	int                width;
	int                marbleIndex;  // to display different marbles
	int                playerX,playerY ; // this client's positions
	int                fontSize;
	BOOL               turn;
	CubeWeapon          currentWeapon;
	int                 health;
	RECT                endScreenRect,ramRect,threeAndRicoRect,threeRicoCheckerRect, ricochetRect,hoamingRect;
	RECT				oneShotRect,threeShotRect,tornadoRect,oneCheckerRect,threeCheckerRect,mortarRect,
						mortarCheckerRect,evilRect,reallyEvilRect;

	// store time when setMessageIndex is called
	time_t			    prevTime;
	
};

#endif