/**
  * Author: Kevin Audleman
  */

/** rubiks_info contains all of the information regarding the location,
  * bearing, speed, animation frame, and animation type for a rubiks
  * cube. Used as the message to pass between the server and player
  */

#ifndef __RUBIKS_INFO__
#define __RUBIKS_INFO__

//Some constants for framecounts for animations. 
#define TURNING_RATE 0.04f
#define MAX_HOPPING 30

#define MAX_BEGINFIRING 10
#define MAX_FIRING      30
#define MAX_ENDFIRING   10

#define MAX_BEGINTORNADO 20
#define MAX_TORNADO 100
#define MAX_ENDTORNADO 20

#define MAX_BEGINSTOMP 20
#define MAX_STOMP 20
#define MAX_ENDSTOMP 10

#define MAX_COLLIDEWITHPROJECTILE 80
#define MAX_COLLIDEWITHPLAYER     80

#define ANIMATION_STEP_SIZE 3

//The types of weapons that a RubiksCube can have
enum CubeWeapon {Stomp, Tornado, Ram, ArcingMortar};

//Different types of animations. These should be general enough for 
//all player types.
enum AnimationType {STANDING, HOPPING, BACKWARDS, 
					BEGINFIRING, FIRING, ENDFIRING,
					BEGINACTION2, ACTION2, ENDACTION2,
					COLLIDEWITHPLAYER, COLLIDEWITHPROJECTILE};

typedef struct r_info
{
	D3DVALUE      head;         //Heading in radians. Straight down the x-axis is 0.0
	D3DVECTOR     pos;          //Location
	D3DVALUE      framecount;  	//Which frame we're on
	AnimationType anim;         //Which animation we're doing.
	int			  hitpoints;    //Value from 0 to 150


	//copy constructor
	void copy(r_info& r){
		r.head = this->head; 
		r.pos = this->pos;
		r.framecount = this->framecount;
		r.anim = this->anim;
		r.hitpoints = this->hitpoints;
	}

} rubiks_info;


#endif