/*==========================================================================
 *
 *  Copyright (C) Drunken Mouse
 *
 *  File:       Server.cpp
 *  Content:	The main computational functions are here
 *
 ***************************************************************************/
#include <time.h>
#include <crtdbg.h>
#include <math.h>
#include <stdlib.h>
#include "Server.h"
#include "dpchat.h"
#include "ServerObject.h"
#include "ServerMovingObject.h"
#include "ServerWeapon.h"
#include "ServerCube.h"
#include "ServerConnect.h"
#include "ServerViewer.h"
#include "ServerIcon.h"
#include "RubicsDirect3D.h"
#include "Boardjects.h"


/* Bounding Box Format:
 = {Xmin, Ymin, Zmin, XMax, Ymax, Zmax}
 Don't forget to shift Y-coordinate up for our axis. See examples.
 A couple other global bounding boxes are in Server.h
 */
const D3DRMBOX TOWER_BOX = {-10.0f,0.0f,-6.0f, 10.0f,20.0f,6.0f};
const D3DRMBOX PEZ_BOX = {-1.0f,0.0f,-1.0f, 1.0f,4.0f,1.0f};
const D3DRMBOX BALLOON_BOX = {-3.0f,11.0f,-3.0f, 3.0f,16.8f,3.0f};
const D3DRMBOX CLOCK_BOX = {-3.0f,0.0f,-1.5f, 3.0f,5.0f,1.5f};

const D3DRMBOX CHAIR_BOX = {-4.0f,0.8f,-5.5f, 4.4f,2.2f,4.5f};
const D3DRMBOX CHAIR_BOX2 = {3.7f,2.2f,-5.5f, 4.4f,13.2f,4.5f};
const D3DRMBOX CHAIR_BOXL1 = {-4.0f,-8.5f,-5.5f, -3.0f,2.2f,-4.5f};
const D3DRMBOX CHAIR_BOXL2 = {-4.0f,-8.5f,3.5f, -3.0f,2.2f,4.5f};
const D3DRMBOX CHAIR_BOXL3 = {3.4f,-8.5f,-5.5f, 4.4f,2.2f,-4.5f};
const D3DRMBOX CHAIR_BOXL4 = {3.4f,-8.5f,3.5f, 4.4f,2.2f,4.5f};

const D3DRMBOX ENGINE_BOX = {-5.0f,-5.0f,-15.0f, 1.5f,1.0f,5.0f};
const D3DRMBOX ENGINE_BOX2 = {-5.1f,1.0f,-2.0f, 1.6f,3.2f,2.0f};
const D3DRMBOX ENGINE_BOX3 = {-4.0f,1.0f,-0.8f, 0.5f,4.4f,0.8f};
const D3DRMBOX CRAYON_BOX = {-0.9f,-1.0f,-17.0f, 0.9f,1.2f,8.0f};
const D3DRMBOX ALPHA_BOX = {-8.5f,0.0f,-8.5f, 8.0f,16.0f,8.0f};
const D3DRMBOX DOMINO_BOX = {-2.0f,0.0f,-12.5f, 2.0f,8.0f,8.0f};
const D3DRMBOX DECK_BOX = {-26.0f,0.0f,-26.0f, 8.0f,4.0f,8.0f};
const D3DRMBOX MISSILE_BOX = {-1.0f,0.0f,-1.8f, 1.0f,2.0f,1.8f};
const D3DRMBOX WALL_BOX = {-1.0f, -1.0f, -(BOARD_X/2.0f), 21.0f, 20*WALL_HEIGHT, (BOARD_X/2.0f)};

#define RESET_TIMER 256
#pragma warning(disable:4786)

MAP_OBJECTS Objects;
MAP_OBJECTS ObjectsToDelete;
MAP_MOVEABLE_OBJECTS MoveableObjects;
MAP_OBJECTS DeletedIcons;


MAP_OBJECTS WallObjects;
MAP_OBJECTS PlayerObjects;
MAP_OBJECTS ProjectileObjects;

MAP_OBJECTS::iterator gpit; //global player index for camera guy
BOOL game_over;
int end_game_count;

int num_players; //global used to position the players

#define MOVEABLE_OBJECT_TYPE 0x1000

extern LPDPLAYINFO lpDPInfo;


//#define MAX_VELOCITY 50
//#define MIN_VELOCITY -50

void UpdatePieces();
void ResetGame();
void InsertObject(CServerObject * lpObject);
void DeleteObjects();
void RemoveObjectFromGame(LPServerObject object);

// Our global board class
CBoardjects Board;

void InitServer() {
	//SetTimer(NULL, NULL, 33, TimerProc);
	Objects.clear();
	WallObjects.clear();
	PlayerObjects.clear();
	gpit = PlayerObjects.begin();
	ProjectileObjects.clear();
	ObjectsToDelete.clear();
	MoveableObjects.clear();
	DeletedIcons.clear();
	game_over = FALSE;
	end_game_count = 0;
	num_players = 0;
	InitBoard();
	srand(time(NULL));
}

/*VOID CALLBACK TimerProc(
  HWND hwnd,     // handle of window for timer messages
  UINT uMsg,     // WM_TIMER message
  UINT idEvent,  // timer identifier
  DWORD dwTime   // current system time
  ) {
	//If the game has ended, wait awhile & then reset the server.
	if(game_over){
		end_game_count++;
		if(end_game_count >= RESET_TIMER){
			ResetGame();
			end_game_count = 0;
			return;
		}
	}
	UpdatePieces();
	BoardCastState();
}*/

void UpdateGame()
 {
	//If the game has ended, wait awhile & then reset the server.
	if(game_over){
		end_game_count++;
		if(end_game_count >= RESET_TIMER){
			ResetGame();
			end_game_count = 0;
			return;
		}
	}
	UpdatePieces();
	BoardCastState();
}


void BoardCastState() {
	MAP_MOVEABLE_OBJECTS::iterator it;	
	int size = sizeof(SVRMSG_SET_STATE) + sizeof (APP_PIECE)*MoveableObjects.size();
	LPSVRMSG_SET_STATE lpMessage = (LPSVRMSG_SET_STATE) malloc(size);
	
	lpMessage->dwType = DPSVRMSG_SET_STATE;
	lpMessage->dwSize = size;
	lpMessage->NumPiece = MoveableObjects.size();
	int i = 0;
	for(it = MoveableObjects.begin(); it != MoveableObjects.end(); it++){
		lpMessage->objects[i].piece =it->second->GetID();
		it->second->GetMessage(&(lpMessage->objects[i].info));
		i++;
	}
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										0, lpMessage, size);
	_ASSERT(!FAILED(hr));
	return;
}

void ChangeDirection(D3DVALUE *velocity, D3DVALUE *orientation, Action action) {
	*velocity = action.velocity;
	*orientation=  fmod(*orientation+action.theta, 6.28);
}

void UpdatePieces() {
	MAP_MOVEABLE_OBJECTS::iterator it;
	MAP_OBJECTS::iterator it_icon;
	Position pos;
	pos.x = 0; pos.y = 0; pos.z = 0;

	for(it = MoveableObjects.begin(); it != MoveableObjects.end(); it++){	
		it->second->Update();

		// Test to see if our projectile's timer is up, if so it destroys internally!
		if(it->second->IsProjectile())
			((CServerWeapon*)it->second)->TimerDestroy();
	}

	//Regenerate Weapon Icon power_ups if they're ready
	for(it_icon = DeletedIcons.begin(); it_icon != DeletedIcons.end(); it_icon++){	
		if(it_icon->second->IsIcon()){
			if( ((CServerIcon*)it_icon->second)->TimerRegen() ) {
				AddObject(it_icon->second->Type, it_icon->second->GetBoundingBox(), it_icon->second->GetPosition(), 0.1f, NULL);

				// Now delete the old icon
				delete (it_icon->second);
				DeletedIcons.erase(it_icon);
				break;
			}
		}
	}

	//Delete all objects this go round
	DeleteObjects();
}


void UpdateDirections() {
	MAP_MOVEABLE_OBJECTS::iterator it;
	for(it = MoveableObjects.begin(); it != MoveableObjects.end(); it++){	
		ChangeDirection(&((*it).second)->velocity, &((*it).second)->orientation, ((*it).second)->action);
	}
}


bool Collision(LPServerObject cube, LPServerObject* other_object,LPD3DVECTOR normal,MAP_OBJECTS::iterator &it,bool reset,objectType type) {
	MAP_OBJECTS *obj;
	if(type == WALL_OBJECT)
	{
		obj = &WallObjects;
	}else if(type == PLAYER_OBJECT)
	{
		obj = &PlayerObjects;

	}else
	{
		obj = &ProjectileObjects;

	}
	if(reset)
	{
		it = obj->begin();
	}
	for(; it != obj->end(); it++){	
		if(cube->GetID() != (*it).first) {
			if(collide(cube, (*it).second,normal)) {
				*other_object = (*it).second;
				it++;
				return true;
			}
		}
	}
	*other_object = NULL;
	return false;
}

void BroadcastMsg(DPMSG_GENERIC * lpMessage, DWORD size) {
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										DPSEND_GUARANTEED, lpMessage, size);
	_ASSERT(!FAILED(hr));
	return;
}

bool IsAddOK(PieceID &piece, PieceType type) {
	return true;
}

float RandomAngle () {
	return (float) (rand() % 180) / 3.14;
}

Position RandomPos() {
	Position pos;
	pos.x = (rand() % (BOARD_X/2) );
	pos.z = (rand() % (BOARD_Y/2) );
	pos.y = (rand() % (WALL_HEIGHT/2) );;
	return pos;
}

LPServerObject NewPiece(PieceType type) {
	switch(type) {
	case RUBICS_CUBE:
		num_players++;
		return (CServerObject *)new CServerCube(num_players);
	case CONNECT_FOUR:
		num_players++;
		return (CServerObject *)new CServerConnect(num_players);
	case VIEWER:
		return (CServerObject *)new CServerViewer;
	default:
		return new CServerObject;
	}
}


void AddPiece(SVRMSG_ADDPIECE * AddPieceMsg, PieceID &piece, D3DRMBOX &box, PieceType type, int team, char playername[]) {
	MAP_OBJECTS::iterator it;	
	_ASSERT (Objects.end()==Objects.find(piece));
	LPServerObject lpObject = NewPiece(type);
	//If we're not a player then we didn't set the box in the constructor
	if(!lpObject->IsPlayer())
		lpObject->SetBox(box);

	lpObject->ID = piece;
	lpObject->Type = type;
	lpObject->velocity = 0;
	lpObject->orientation = RandomAngle();
	lpObject->SetPos(RandomPos());
	lpObject->action.velocity = 0;
	lpObject->action.theta = 0;
	// default name is '**********'
	for(int j=0; j<10; j++)
		lpObject->name[j] = '*';

	InsertObject(lpObject);
	AddPieceMsg->dwType = DPSVRMSG_ADDPIECE;
	AddPieceMsg->idPiece = piece;
	AddPieceMsg->pos = lpObject->GetPosition();
	AddPieceMsg->box = lpObject->GetBoundingBox();
	AddPieceMsg->scale = 1.0f;
	AddPieceMsg->typePiece = type;
	if(lpObject->IsPlayer()){
		//Send center square info
		for(j=0; j<6; j++)
			AddPieceMsg->centerSquare[j] = ((CServerPlayer*)lpObject)->centerSquare[j];
		//Get & Send player name
		for(j=0; j<10; j++){
			lpObject->name[j] = playername[j];
			AddPieceMsg->name[j] = playername[j];
		}
		//Get & Send player Team
		((CServerPlayer*)lpObject)->team = team;
		AddPieceMsg->team = ((CServerPlayer*)lpObject)->GetTeam();
	}
	AddPieceMsg->theta_xz = lpObject->orientation;
}

void SendCurrentObjectsMsg(DPID to_id) {
	MAP_OBJECTS::iterator it;
	SVRMSG_ADDPIECE AddPieceMsg;
	for(it = Objects.begin(); it != Objects.end(); it++) {	
		AddPieceMsg.dwType = DPSVRMSG_ADDPIECE;
		AddPieceMsg.idPiece = (*it).first;
		AddPieceMsg.pos = ((*it).second)->GetPosition();
		AddPieceMsg.box = ((*it).second)->GetBoundingBox();
		AddPieceMsg.typePiece = ((*it).second)->Type;
		AddPieceMsg.scale = ((*it).second)->scale;
		AddPieceMsg.theta_xz = (float)((*it).second)->orientation;
		// If we're a player, send our centerSquare, Name, & team info
		if(((*it).second)->IsPlayer()){
			for(int j=0; j<6; j++)
				AddPieceMsg.centerSquare[j] = ((CServerPlayer*)((*it).second))->centerSquare[j];
			for(j=0; j<10; j++)
				AddPieceMsg.name[j] = ((CServerPlayer*)((*it).second))->name[j];
			AddPieceMsg.team = ((CServerPlayer*)((*it).second))->GetTeam();
		}
		HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, to_id,
										DPSEND_GUARANTEED, &AddPieceMsg, sizeof(SVRMSG_ADDPIECE));
		_ASSERT(!FAILED(hr));
	}	
}

void SendPickupPieceMsg(PieceID id, PieceType type) {
	SVRMSG_PICKEDUPPIECE Msg;
	MAP_OBJECTS::iterator it;

	Msg.dwType = DPSVRMSG_PICKEDUPPIECE;
	Msg.idPiece = id;
	Msg.color = type;
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
									DPSEND_GUARANTEED, &Msg, sizeof(SVRMSG_PICKEDUPPIECE));
	_ASSERT(!FAILED(hr));

}

void SendDropPieceMsg(PieceID id, PieceType type, CServerObject* object, PieceType weapon) {
	SVRMSG_DROPPIECE Msg;
	MAP_OBJECTS::iterator it;
	int rand_msg;

	Msg.dwType = DPSVRMSG_DROPPIECE;
	Msg.idPiece = id;
	Msg.color = type;

	switch(weapon){
	case CUBE_PROJECTILE:
		rand_msg = rand() % 5;
		switch(rand_msg){
		case 0:  
			Msg.msg_index = 2;
			break;
		case 1:  
			Msg.msg_index = 5;
			break;
		case 2:  
			Msg.msg_index = 6;
			break;
		case 3:  
			Msg.msg_index = 8;
			break;
		case 4:  
			Msg.msg_index = 9;
			break;
		default:  
			Msg.msg_index = 11;
			break; 
		}
		break;
	case RAM_OBJECT:
		Msg.msg_index = 7;
		break;
	case RUBICS_CUBE:
		if( ((CServerCube*)object)->GetCurrentWeapon() == Ram)
			Msg.msg_index = 3;
		else if( ((CServerCube*)object)->GetCurrentWeapon() == Tornado && 
					((CServerCube*)object)->Type == RUBICS_CUBE )
			Msg.msg_index = 4;
		else
			Msg.msg_index = 11;
		break;
	case PEZ:
		Msg.msg_index = 10;
		break;
	default:
		Msg.msg_index = 11;
		break;
	}

	for(int i=0; i<10; i++){
		if(object != NULL && object->name != NULL){
			if(weapon == CUBE_PROJECTILE){
				Msg.attacker_name[i] = (object->GetOwner())->name[i];
			}else{
				Msg.attacker_name[i] = object->name[i];
			}
		}
		else{
			Msg.attacker_name[i] = 'a';
			Msg.attacker_name[9] = '\0';
		}
	}

	it=PlayerObjects.find(id);
	if(it!=PlayerObjects.end()) {
		for(i=0; i<10; i++){
			if(it->second !=NULL && it->second->name != NULL)
				Msg.loser_name[i] = it->second->name[i];
			else{
				Msg.loser_name[i] = 'a';
			    Msg.attacker_name[9] = '\0';
			}
		}
	}else{
		for(i=0; i<10; i++){
			Msg.loser_name[i] = 'n';
			Msg.attacker_name[9] = '\0';
		}
	}

	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
									DPSEND_GUARANTEED, &Msg, sizeof(SVRMSG_DROPPIECE));
	_ASSERT(!FAILED(hr));
}


void NewPlayerAdded(DPID id, char* playername) {
	static short NextPlayer = 1;
	SVRMSG_ADDPLAYER AddPlayerMsg;
	int index = 0;
	
	AddPlayerMsg.dwType = DPSVRMSG_ADDPLAYER;
	AddPlayerMsg.PlayerID = NextPlayer;
	while(playername != NULL && *playername != '/0' && index < 10){
		AddPlayerMsg.name[index] = *playername;
		playername ++;
		index ++;
	}

	HRESULT hr = lpDPInfo->lpDirectPlay4A->SetPlayerData(id,&NextPlayer ,sizeof(short), DPSET_LOCAL);
	_ASSERT(!FAILED(hr));
	
	hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, id,
										DPSEND_GUARANTEED, &AddPlayerMsg, sizeof(SVRMSG_ADDPLAYER));
	_ASSERT(!FAILED(hr));	
	NextPlayer++;
	SendCurrentObjectsMsg(id);
}

PieceType GetProjectile(CServerObject *owner) {
	switch(owner->Type) {
	case RUBICS_CUBE:
		return CUBE_PROJECTILE;
	case CONNECT_FOUR:
		return CONNECT_PROJECTILE;
	default:
		return CUBE_PROJECTILE;
	}
}


void CreateProjectile(Position pos, D3DVALUE orient_angle, BOOL ricochete, CServerObject *owner){
	SVRMSG_ADDPIECE AddPieceMsg;
	LPServerObject lpObject;

	//Add the projectile
	if(ricochete)
		lpObject = Board.InsertBoardject(RICOCHETE_PROJ, MISSILE_BOX, pos, orient_angle, 1.0f, owner);
	else
		lpObject = Board.InsertBoardject(CUBE_PROJECTILE, MISSILE_BOX, pos, orient_angle, 1.0f, owner);
	InsertObject(lpObject);
	
	// Send network message to add a new projectile
	AddPieceMsg.dwType = DPSVRMSG_ADDPIECE;
	AddPieceMsg.idPiece = lpObject->GetID();
	AddPieceMsg.pos = pos;
	AddPieceMsg.box = MISSILE_BOX;
	AddPieceMsg.scale = 1.0f;
	AddPieceMsg.typePiece = GetProjectile(owner);//CUBE_PROJECTILE
	AddPieceMsg.theta_xz = orient_angle;
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										DPSEND_GUARANTEED, &AddPieceMsg, sizeof(SVRMSG_ADDPIECE));
	_ASSERT(!FAILED(hr));
	return;
}

void DeletePlayerPiece(short Player){
	PieceID piece;
	MAP_OBJECTS::iterator it;
	MAP_MOVEABLE_OBJECTS::iterator it1;
	
	// Delete the players piece & reference from the total objects data structure
	piece = PIECE_ID(Player, 1);
	it=Objects.find(piece);
	//_ASSERT(it!=Objects.end());
	if(it!=Objects.end() ){
		if( (it->second)->IsPlayer()){
			for (int i = 0; i < 6; i++)
			{
				((CServerCube*) (it->second))->DropPiece( ((CServerObject*)it->second), PEZ );
			}
		}
		RemoveObject(it->second);
	}
}

//This function adds the object to the appropriate maps.
void InsertObject(CServerObject * lpObject) {
	if(lpObject) {
		Objects.insert(MAP_OBJECTS::value_type(lpObject->GetID(), lpObject));
		if(lpObject->IsMoveable()) {
			MoveableObjects.insert(MAP_MOVEABLE_OBJECTS::value_type(lpObject->GetID(), (LPServerMovingObject) lpObject));
		}
		if(lpObject->IsWall()) {
			WallObjects.insert(MAP_OBJECTS::value_type(lpObject->GetID(), (LPServerObject) lpObject));
		}
		if(lpObject->IsPlayer()) {
			PlayerObjects.insert(MAP_OBJECTS::value_type(lpObject->GetID(), (LPServerObject) lpObject));
			gpit = PlayerObjects.begin();
		}
		if(!lpObject->IsWall() && !lpObject->IsPlayer()) {
			ProjectileObjects.insert(MAP_OBJECTS::value_type(lpObject->GetID(), (LPServerObject) lpObject));
		}

	}
}

void InitBoard()
{
	Position pos;
	LPServerObject lpObject;

	// Add WALL 1
	pos.x = (float)BOARD_X/2.0f; pos.y = 0.0; pos.z = 0.0;
	lpObject = Board.InsertBoardject(WALL, WALL_BOX, pos, 0.0f);
	InsertObject(lpObject);
	
	// Add WALL 2
	pos.x = (float)-BOARD_X/2.0f; pos.y = 0.0; pos.z = 0.0;
	lpObject = Board.InsertBoardject(WALL, WALL_BOX, pos, (float)PI);
	InsertObject(lpObject);
	
	// Add WALL 3
	pos.x = 0.0; pos.y = 0.0; pos.z = (float)BOARD_Y/2.0f;
	lpObject = Board.InsertBoardject(WALL, WALL_BOX, pos, (float)PI/2.0f);
	InsertObject(lpObject);
	
	// Add WALL 4
	pos.x = 0.0; pos.y = 0.0; pos.z = (float)-BOARD_Y/2.0f;
	lpObject = Board.InsertBoardject(WALL, WALL_BOX, pos, 3.0f*(float)PI/2.0f);
	InsertObject(lpObject);
	
	// Add Red Rubiks Piece
	pos.x = -100.0; pos.y = 19.0; pos.z = -100.0;
	lpObject = Board.InsertBoardject(RED_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add 2nd Red Rubiks Piece
	pos.x = 100.0; pos.y = 84.0; pos.z = -65.0;
	lpObject = Board.InsertBoardject(RED_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Green Rubiks Piece
	pos.x = 98.0; pos.y = 42.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(GREEN_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add 2nd Green Rubiks Piece
	pos.x = 35.0; pos.y = 84.0; pos.z = -46.0;
	lpObject = Board.InsertBoardject(GREEN_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Blue Rubiks Piece
	pos.x = -100.0; pos.y = 9.0; pos.z = 100.0;
	lpObject = Board.InsertBoardject(BLUE_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add 2nd Blue Rubiks Piece
	pos.x = -230.0; pos.y = 7.0; pos.z = -230.0;
	lpObject = Board.InsertBoardject(BLUE_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add White Rubiks Piece
	pos.x = -230.0; pos.y = 7.0; pos.z = 230.0;
	lpObject = Board.InsertBoardject(WHITE_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add 2nd White Rubiks Piece
	pos.x = -6.0; pos.y = 38.0; pos.z = 16.0;
	lpObject = Board.InsertBoardject(WHITE_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Yellow Rubiks Piece
	pos.x = 230.0; pos.y = 7.0; pos.z = -230.0;
	lpObject = Board.InsertBoardject(YELLOW_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add 2nd Yellow Rubiks Piece
	pos.x = 100.0; pos.y = 19.0; pos.z = 100.0;
	lpObject = Board.InsertBoardject(YELLOW_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Orange Rubiks Piece
	pos.x = 230.0; pos.y = 7.0; pos.z = 230.0;
	lpObject = Board.InsertBoardject(ORANGE_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add 2nd Orange Rubiks Piece
	pos.x = 100.0; pos.y = 19.0; pos.z = 100.0;
	lpObject = Board.InsertBoardject(ORANGE_RUBIKS_PIECE, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add A Alphabet Block
	pos.x = -6.0; pos.y = 16.0; pos.z = 16.0;
	lpObject = Board.InsertBoardject(ALPHA_BLOCK_A, ALPHA_BOX, pos, -45.0f);
	InsertObject(lpObject);

	// Add B Alphabet Block
	pos.x = 9.0; pos.y = 0.0; pos.z = 19.0;
	lpObject = Board.InsertBoardject(ALPHA_BLOCK_B, ALPHA_BOX, pos, 45.0f);
	InsertObject(lpObject);

	// Add C Alphabet Block
	pos.x = -18.0; pos.y = 0.0; pos.z = 13.0;
	lpObject = Board.InsertBoardject(ALPHA_BLOCK_C, ALPHA_BOX, pos, 30.0f);
	InsertObject(lpObject);

	//Add mini A Alphabet Block
	pos.x = -100.0; pos.y = 0.0; pos.z = 100.0;
	lpObject = Board.InsertBoardject(ALPHA_BLOCK_A, ALPHA_BOX, pos, -45.0f, 0.5f);
	InsertObject(lpObject);
	
	//Add mini B Alphabet Block
	pos.x = 100.0; pos.y = 0.0; pos.z = 100.0;
	lpObject = Board.InsertBoardject(ALPHA_BLOCK_B, ALPHA_BOX, pos, 45.0f, 0.5f);
	InsertObject(lpObject);

	// Add mini C Alphabet Block
	pos.x = -100.0; pos.y = 0.0; pos.z = -100.0;
	lpObject = Board.InsertBoardject(ALPHA_BLOCK_C, ALPHA_BOX, pos, 30.0f, 0.5f);
	InsertObject(lpObject);

	// Add mini DIE
	pos.x = -140; pos.y = 0.0; pos.z = 68.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, 7*(float)PI/3.0f, 0.5f);
	InsertObject(lpObject);

	// Add big DOMINO
	pos.x = 75.0; pos.y = 0.0; pos.z = -75.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/6.0f, 2.0f);
	InsertObject(lpObject);

	//Add Engine
	pos.x = -200.0; pos.y = 7.0; pos.z = 200.0;
	lpObject = Board.InsertBoardject(ENGINE, ENGINE_BOX, pos, 3*(float)PI/4.0f, 3.0f);
	InsertObject(lpObject);
	//Add an invisible box for engine smoke stack
	pos.z += 4.5 * cos(3*(float)PI/4.0f); pos.x += 4.5*(-sin(3*(float)PI/4.0f));
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, ENGINE_BOX3, pos, 3*(float)PI/4.0f, 3.0f);
	InsertObject(lpObject);
	//Add an invisible box for engine cab
	pos.z -= 18.0 * cos(3*(float)PI/4.0f); pos.x -= 18.0 *(-sin(3*(float)PI/4.0f));
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, ENGINE_BOX2, pos, 3*(float)PI/4.0f, 3.0f);
	InsertObject(lpObject);
	
	//Add Crayon
	pos.x = 65.0; pos.y = 4.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, 0.0f, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 68.5; pos.y = 10.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 72.0; pos.y = 16.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, 0.0f, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 75.5; pos.y = 22.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 79.0; pos.y = 28.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, 0.0f, 2.3f);
	InsertObject(lpObject);

	// Add Blue DECK
	pos.x = 99.0; pos.y = 26.0; pos.z = 46.0;
	lpObject = Board.InsertBoardject(DECK, DECK_BOX, pos, 0.0f, 2.0f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 98.0; pos.y = 30.0; pos.z = 30.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 98.0; pos.y = 36.0; pos.z = 26.5;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, 3.0f*(float)PI/2.0f, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 98.0; pos.y = 42.0; pos.z = 23.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 98.0; pos.y = 48.0; pos.z = 19.5;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, 3.0f*(float)PI/2.0f, 2.3f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 98.0; pos.y = 54.0; pos.z = 16.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.3f);
	InsertObject(lpObject);

	// Add RED DECK
	pos.x = 105.0; pos.y = 51.0; pos.z = -2.0;
	lpObject = Board.InsertBoardject(DECK, DECK_BOX, pos, 0.0f, 2.0f);
	InsertObject(lpObject);

	/*Add Crayon
	pos.x = 110.0; pos.y = 59.0; pos.z = 19.5;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.0f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 110.0; pos.y = 67.0; pos.z = 23.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.0f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 110.0; pos.y = 75.0; pos.z = 26.5;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.0f);
	InsertObject(lpObject);

	//Add Crayon
	pos.x = 110.0; pos.y = 83.0; pos.z = 30.0;
	lpObject = Board.InsertBoardject(CRAYON, CRAYON_BOX, pos, (float)PI/2.0f, 2.0f);
	InsertObject(lpObject);*/

	// Add GREEN DECK
	pos.x = 95.0; pos.y = 49.0; pos.z = -65.0;
	lpObject = Board.InsertBoardject(DECK, DECK_BOX, pos, 0.0f, 3.0f);
	InsertObject(lpObject);

	// Add YELLOW DECK
	pos.x = 20.0; pos.y = 47.0; pos.z = -46.0;
	lpObject = Board.InsertBoardject(DECK, DECK_BOX, pos, 0.0f, 3.0f);
	InsertObject(lpObject);

	// Add Chair
	pos.x = 200.0; pos.y = 10.5; pos.z = 200.0;
	lpObject = Board.InsertBoardject(CHAIR, CHAIR_BOX, pos, 1.0f, 1.5f);
	InsertObject(lpObject);
	//Add an invisible box for chair back
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, CHAIR_BOX2, pos, 1.0f, 1.5f);
	InsertObject(lpObject);
	//Add an invisible box for chair leg1
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, CHAIR_BOXL1, pos, 1.0f, 1.5f);
	InsertObject(lpObject);
	//Add an invisible box for chair leg2
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, CHAIR_BOXL2, pos, 1.0f, 1.5f);
	InsertObject(lpObject);
	//Add an invisible box for chair leg3
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, CHAIR_BOXL3, pos, 1.0f, 1.5f);
	InsertObject(lpObject);
	//Add an invisible box for chair leg4
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, CHAIR_BOXL4, pos, 1.0f, 1.5f);
	InsertObject(lpObject);

	// Add DIE
	pos.x = 214.0; pos.y = 0.0; pos.z = -214.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, (float)PI/4.0f, 0.75f);
	InsertObject(lpObject);
	// Add DIE
	pos.x = 206.0; pos.y = 12.0; pos.z = -206.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, 3*(float)PI/4.0f, 0.75f);
	InsertObject(lpObject);
	// Add DIE
	pos.x = 200.0; pos.y = 24.0; pos.z = -200.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, (float)PI/2.0f, 0.75f);
	InsertObject(lpObject);
	// Add DIE
	pos.x = 194; pos.y = 12.0; pos.z = -194.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, 5*(float)PI/4.0f, 0.75f);
	InsertObject(lpObject);
	// Add DIE
	pos.x = 188; pos.y = 0.0; pos.z = -188.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, 7*(float)PI/4.0f, 0.75f);
	InsertObject(lpObject);

	// Add DIE
	pos.x = 200; pos.y = 0.0; pos.z = -148.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, 7*(float)PI/5.0f, 1.0f);
	InsertObject(lpObject);
	// Add DIE
	pos.x = 148; pos.y = 0.0; pos.z = -200.0;
	lpObject = Board.InsertBoardject(DICE, ALPHA_BOX, pos, 7*(float)PI/3.0f, 1.0f);
	InsertObject(lpObject);

	// Add DOMINO
	pos.x = -244.0; pos.y = 0.0; pos.z = -240.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -244.0; pos.y = 14.0; pos.z = -240.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -214.0; pos.y = 0.0; pos.z = -240.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -214.0; pos.y = 14.0; pos.z = -240.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.5f);
	InsertObject(lpObject);
	*/
	// Add DOMINO
	pos.x = -240.0; pos.y = 0.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -240.0; pos.y = 14.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -210.0; pos.y = 0.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -210.0; pos.y = 14.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -184.0; pos.y = 0.0; pos.z = -240.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -184.0; pos.y = 14.0; pos.z = -240.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -184.0; pos.y = 0.0; pos.z = -210.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -184.0; pos.y = 14.0; pos.z = -210.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -220.0; pos.y = 0.0; pos.z = -169.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -220.0; pos.y = 14.0; pos.z = -169.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -190.0; pos.y = 0.0; pos.z = -169.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -190.0; pos.y = 14.0; pos.z = -169.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -164.0; pos.y = 0.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -164.0; pos.y = 14.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -164.0; pos.y = 0.0; pos.z = -170.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);
	/* Add DOMINO
	pos.x = -164.0; pos.y = 14.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.5f);
	InsertObject(lpObject);
	*/

	// Add DOMINO
	pos.x = -164.0; pos.y = 0.0; pos.z = -140.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);

	// Add DOMINO
	pos.x = -134.0; pos.y = 0.0; pos.z = -204.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, 0.0f, 1.0f);
	InsertObject(lpObject);


	// Add DOMINO
	pos.x = -194.0; pos.y = 0.0; pos.z = -144.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.0f);
	InsertObject(lpObject);

	// Add DOMINO
	pos.x = -134.0; pos.y = 0.0; pos.z = -134.0;
	lpObject = Board.InsertBoardject(DOMINO, DOMINO_BOX, pos, (float)PI/2.0f, 1.0f);
	InsertObject(lpObject);


	/* Add Balloons
	pos.x = 100.0; pos.y = 92.0; pos.z = -45.0;
	lpObject = Board.InsertBoardject(BALLOON, BALLOON_BOX, pos, 0.0f);
	InsertObject(lpObject);
	
	// Add LEGO_GUY
	pos.x = -55.0; pos.y = 0.0; pos.z = 40.0;
	lpObject = Board.InsertBoardject(LEGO_GUY, CLOCK_BOX, pos, 2.5f);
	InsertObject(lpObject);

	// Add DART_BOARD
	pos.x = -55.0; pos.y = 15.0; pos.z = -60.0;
	lpObject = Board.InsertBoardject(DART_BOARD, CLOCK_BOX, pos, 2.5f);
	InsertObject(lpObject);

	// Add TABLE
	pos.x = 75.0; pos.y = 10.0; pos.z = 10.0;
	lpObject = Board.InsertBoardject(POOL_TABLE, CLOCK_BOX, pos, 2.5f);
	InsertObject(lpObject);

	// Add AIR_PLANE
	pos.x = 55.0; pos.y = 50.0; pos.z = 40.0;
	lpObject = Board.InsertBoardject(AIR_PLANE, CLOCK_BOX, pos, 2.5f);
	InsertObject(lpObject);

	// Add Pez Dispenser
	pos.x = -80.0; pos.y = 0.0; pos.z = -48.0;
	lpObject = Board.InsertBoardject(PEZ, PEZ_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Balloons
	pos.x = 35.0; pos.y = 0.0; pos.z = 100.0;
	lpObject = Board.InsertBoardject(BALLOON, BALLOON_BOX, pos, 2.5f);
	InsertObject(lpObject);
	
	// Add More Balloons
	pos.x = 30.0; pos.y = 0.0; pos.z = 104.0;
	lpObject = Board.InsertBoardject(BALLOON, BALLOON_BOX, pos, 1.9f);
	InsertObject(lpObject);
	*/

	// Add Multishot
	pos.x = 6.0; pos.y = 16.0; pos.z = 19.0;
	lpObject = Board.InsertBoardject(MULTI_SHOT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Multishot
	pos.x = 98.0; pos.y = 51.0; pos.z = 22.0;
	lpObject = Board.InsertBoardject(MULTI_SHOT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Ricochete
	pos.x = -218.0; pos.y = 0.0; pos.z = -18.0;
	lpObject = Board.InsertBoardject(RICOCHETE, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add Ricochete
	pos.x = -72.0; pos.y = 0.0; pos.z = 163.0;
	lpObject = Board.InsertBoardject(RICOCHETE, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add RAM_OBJECT
	pos.x = 50.0; pos.y = 0.0; pos.z = 200.0;
	lpObject = Board.InsertBoardject(RAM_OBJECT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add RAM_OBJECT
	pos.x = -90.0; pos.y = 0.0; pos.z = -78.0;
	lpObject = Board.InsertBoardject(RAM_OBJECT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);
	
	// Add TORNADO_OBJECT
	pos.x = -220.0; pos.y = 0.0; pos.z = 220.0;
	lpObject = Board.InsertBoardject(TORNADO_OBJECT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add TORNADO_OBJECT
	pos.x = 25.0; pos.y = 51.0; pos.z = -46.0;
	lpObject = Board.InsertBoardject(TORNADO_OBJECT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add ARCING_OBJECT
	pos.x = 200.0; pos.y = 0.0; pos.z = 200.0;
	lpObject = Board.InsertBoardject(ARCING_OBJECT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add ARCING_OBJECT
	pos.x = 200.0; pos.y = 0.0; pos.z = -200.0;
	lpObject = Board.InsertBoardject(ARCING_OBJECT, MISSILE_BOX, pos, 0);
	InsertObject(lpObject);

	// Add an invisible box as a test
	/*pos.x = 0.0; pos.y = 0.0; pos.z = 0.0;
	lpObject = Board.InsertBoardject(INVISIBLE_BOX, RUBIKS_PIECE_BOX, pos, 0);
	InsertObject(lpObject);*/

	return;
}

//Removes the object from the board
void RemoveObject(LPServerObject object) {
	MAP_OBJECTS::iterator it;
	MAP_MOVEABLE_OBJECTS::iterator it1;
	SVRMSG_REMOVEPIECE RemovePieceMsg;
	
	// Delete the players piece & reference from the total objects data structure
	it=Objects.find(object->GetID());
	_ASSERT(it!=Objects.end());
	if(it==Objects.end())
		return;
	Objects.erase(it);

	// Delete the players reference from the movable objects data structure
	it1=MoveableObjects.find(object->GetID());
	if(it1!=MoveableObjects.end()) {
		MoveableObjects.erase(it1);
	}

	it=PlayerObjects.find(object->GetID());
	if(it!=PlayerObjects.end()) {
		PlayerObjects.erase(it);
		gpit = PlayerObjects.begin();
	}

	it=WallObjects.find(object->GetID());
	if(it!=WallObjects.end()) {
		if(object->IsIcon()){
			DeletedIcons.insert(MAP_OBJECTS::value_type(object->GetID(), object));
		}
		WallObjects.erase(it);
	}

	it=ProjectileObjects.find(object->GetID());
	if(it!=ProjectileObjects.end()) {
		ProjectileObjects.erase(it);
	}

	// Send the client a message to remove the player
	RemovePieceMsg.dwType = DPSVRMSG_REMOVEPIECE;
	RemovePieceMsg.idPiece = object->GetID();
	RemovePieceMsg.typePiece = object->Type;
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										DPSEND_GUARANTEED, &RemovePieceMsg, sizeof(SVRMSG_REMOVEPIECE));
	_ASSERT(!FAILED(hr));
	if(!object->IsIcon())
		//delete ((CServerCube*)object);
		if(object->Type == CUBE_PROJECTILE || object->Type == RICOCHETE_PROJ)
			delete( (CServerWeapon*)object);
		else
			delete (object);
}

void MakeMove(PieceID & piece, Action & action) {
	MAP_MOVEABLE_OBJECTS::iterator it;	
	it=MoveableObjects.find(piece);
	//_ASSERT(it!=MoveableObjects.end());
	if(it != MoveableObjects.end())
		((*it).second)->ReceiveAction(action);
	return;
}


//This call will add the object will remove the object from the game and mark it for deletion.
void RemoveObjectFromGame(LPServerObject object) {
	if(object) {
		ObjectsToDelete.insert(MAP_OBJECTS::value_type(object->GetID(), object));
	}
}

//This call deletes all the current objects marked for deletion.
void DeleteObjects(){
	MAP_OBJECTS::iterator it;
	//MAP_MOVEABLE_OBJECTS::iterator it1;
	
	for(it = ObjectsToDelete.begin(); it != ObjectsToDelete.end(); it++){
		RemoveObject (it->second);
	}
	ObjectsToDelete.clear();
}


LPServerObject  AddObject(PieceType type, D3DRMBOX box,Position pos, float theta, LPServerObject owner)
{
	SVRMSG_ADDPIECE AddPieceMsg;
	LPServerObject lpObject;

	lpObject = Board.InsertBoardject(type, box, pos, theta, 1.0f, owner);
	InsertObject(lpObject);

	// Send network message to add a new projectile
	AddPieceMsg.dwType = DPSVRMSG_ADDPIECE;
	AddPieceMsg.idPiece = lpObject->GetID();
	AddPieceMsg.pos = pos;
	AddPieceMsg.box = box;
	AddPieceMsg.typePiece = type;
	AddPieceMsg.scale = 1.0f;
	AddPieceMsg.theta_xz = theta;
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										DPSEND_GUARANTEED, &AddPieceMsg, sizeof(SVRMSG_ADDPIECE));
	_ASSERT(!FAILED(hr));
	return lpObject;
}

Position GetNextCameraPos(float* orient)
{
	Position pos;
	pos.x = 0; pos.y = 0; pos.z = 30;
	*orient = -(float)PI/2.0f;

	//If no one's in the map then return
	if(PlayerObjects.empty())
		return (pos);

	gpit++;

	if(gpit == PlayerObjects.end())
		gpit = PlayerObjects.begin();

	*orient = ((CServerMovingObject*)gpit->second)->info.head;

	return ( ((CServerMovingObject*)gpit->second)->info.pos );
}

void DoWinningSeq(LPServerObject player){
	game_over = TRUE;
	SVRMSG_WINNER WinningMsg;

	WinningMsg.dwType = DPSVRMSG_WINNER;
	WinningMsg.idPiece = player->GetID();
	WinningMsg.team = ((CServerPlayer*)player)->GetTeam();
	WinningMsg.typePiece = player->Type;
	
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										DPSEND_GUARANTEED, &WinningMsg, sizeof(SVRMSG_WINNER));
	_ASSERT(!FAILED(hr));

}

void ResetGame(){
	MAP_OBJECTS::iterator it;
	SVRMSG_RESETGAME ResetMsg;
	LPServerObject to_delete;
	
	//This actually dealocates all memory, then I clear maps
	for(it = Objects.begin(); it != Objects.end(); it++){
		to_delete = it->second;
		delete (to_delete);
	}

	Objects.clear();
	WallObjects.clear();
	PlayerObjects.clear();
	gpit = PlayerObjects.begin();
	ProjectileObjects.clear();
	ObjectsToDelete.clear();
	MoveableObjects.clear();
	DeletedIcons.clear();
	game_over = FALSE;
	num_players = 0;
	
	InitBoard();

	ResetMsg.dwType = DPSVRMSG_RESETGAME;
	HRESULT hr = lpDPInfo->lpDirectPlay4A->Send(lpDPInfo->dpidPlayer, DPID_ALLPLAYERS,
										DPSEND_GUARANTEED, &ResetMsg, sizeof(SVRMSG_RESETGAME));
	_ASSERT(!FAILED(hr));
}

LPServerObject GetObjectFromID(PieceID id) {
	MAP_OBJECTS::iterator it;	
	it=Objects.find(id);
	//_ASSERT(it!=Objects.end());
	if(it!=Objects.end())
		return it->second;
	else return NULL;
}

MAP_OBJECTS* GetPlayersMap(){
	return &PlayerObjects;
}