// ServerConnect.cpp: implementation of the CServerConnect class.
//
//////////////////////////////////////////////////////////////////////

#include "ServerConnect.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServerConnect::CServerConnect()
{
	info.head = 0.0f;

	info.pos.x = 200.0f;
	info.pos.y = 300.0f;
	info.pos.z = 200.0f;

	info.framecount = 0.0f;
	info.anim = STANDING;

	for (int i = 0; i < 6; i++)
		centerSquare[i] = false;
	velocity = 0.0f;
	action.velocity = 0.0f;
	action.theta =  0.0f;

	//Setup the bounding box.  Fixed by GJ 5/20
	box.min.x = -1.575f;
	box.min.y = 0.0f;
	box.min.z = -1.575f;
	box.max.x = 1.575f;
	box.max.y = 4.80f;
	box.max.z = 1.575f;

	currentWeapon = Stomp;
	RamObject = NULL;
}

CServerConnect::CServerConnect(int num_players)
{
	info.head = (float)PI/4.0f;

	info.pos.x = 200.0f;
	info.pos.y = 300.0f;
	info.pos.z = 200.0f;
	
	//Start the players in the four corners
	switch(num_players % 4){
	case 0:
		info.head = 3*(float)PI/4.0f;
		info.pos.x *= 1.0f;
		info.pos.z *= 1.0f;
		break;
	case 1:
		info.head = (float)PI/4.0f;
		info.pos.x *= -1.0f;
		info.pos.z *= 1.0f;
		break;
	case 2:
		info.head = 5*(float)PI/4.0f;
		info.pos.x *= 1.0f;
		info.pos.z *= -1.0f;
		break;
	case 3:
		info.head = 7*(float)PI/4.0f;
		info.pos.x *= -1.0f;
		info.pos.z *= -1.0f;
		break;
	default:
		break;
	}

	info.framecount = 0.0f;
	info.anim = STANDING;

	for (int i = 0; i < 6; i++)
		centerSquare[i] = false;
	velocity = 0.0f;
	action.velocity = 0.0f;
	action.theta =  0.0f;

	//Setup the bounding box.  Fixed by GJ 5/20
	box.min.x = -1.575f;
	box.min.y = 0.0f;
	box.min.z = -1.575f;
	box.max.x = 1.575f;
	box.max.y = 4.80f;
	box.max.z = 1.575f;

	currentWeapon = Stomp;
	RamObject = NULL;
}

CServerConnect::~CServerConnect()
{

}

bool CServerConnect::DoTornado() {
{
	D3DVALUE  vel = 2.3f;
	
	info.pos.x += vel * (D3DVALUE)cos(info.head);
	info.pos.z -= vel * (D3DVALUE)sin(info.head);

	
	if(RamObject) {
		D3DVALUE  ram_distance = (box.max.x+RamObject->GetBoundingBox().max.x);
		RamObject->SetPosition(info.pos.x + ram_distance* D3DVAL(cos(info.head)), info.pos.y,info.pos.z + ram_distance * -D3DVAL(sin(info.head)));
		RamObject->SetOrientation(info.head);
	}

	info.framecount += ANIMATION_STEP_SIZE;
	if (info.framecount > MAX_TORNADO) 
	{
		info.framecount = 0;
		return true;
	}
	return false;
}

}

bool CServerConnect::DoBeginTornado()
{
	D3DVALUE  vel = 1.4f;
	D3DRMBOX RamBox;
	D3DVECTOR RamPos;
	//RamBox.min.z = -1.575f;
	RamBox.min.x = -0.5f;
	RamBox.min.y = 0.0f;
	RamBox.min.z = -1.575f;
	RamBox.max.x = 0.5f;
	RamBox.max.y = 4.80f;
	RamBox.max.z = 1.575f;
	
	RamPos.x = box.max.x+RamBox.max.x;
	RamPos.y = 0;
	RamPos.z = 0;

	if(!RamObject){
		RamObject = (CServerRam*) AddObject(RAM_BLOCK, RamBox, RamPos, info.head, this);
		for(int i = 0; i <10; i++)
			RamObject->name[i] = name[i];
	}

	info.pos.x += vel * (D3DVALUE)cos(info.head);
	info.pos.z -= vel * (D3DVALUE)sin(info.head);

	//Hurricane is a bad thing to touch!
	//dontTouch = true;
	D3DVALUE  ram_distance = (box.max.x+RamObject->GetBoundingBox().max.x);
	RamObject->SetPosition(info.pos.x + ram_distance * D3DVAL(cos(info.head)), ram_distance,info.pos.z + RamPos.x * D3DVAL(sin(info.head)));
	RamObject->SetOrientation(info.head);
	
	info.framecount += ANIMATION_STEP_SIZE;
	if (info.framecount > MAX_BEGINTORNADO) 
	{
		info.framecount = 0;
		return true;
	}
	return false;
}


bool CServerConnect::DoEndTornado()
{
	D3DVALUE  vel = 1.0f;
	if(RamObject) {
		RemoveObjectFromGame(RamObject);
		RamObject = NULL;
	}

	info.pos.x += vel * (D3DVALUE)cos(info.head);
	info.pos.z -= vel * (D3DVALUE)sin(info.head);

	info.framecount += ANIMATION_STEP_SIZE;
	if (info.framecount > MAX_ENDTORNADO) 
	{
		info.framecount = 0;
		return true;
	}
	return false;
}

bool CServerConnect::DoBeginAction2()
{
	switch (currentWeapon)
	{
		case Stomp:
		case Ram:
			return DoBeginTornado();
			break;
		case Tornado:
			return DoBeginFiring();
			break;
		case ArcingMortar:
			//this is temporary
			//if(info.framecount == 0)
			//	AddObject(CONECT_MORTAR_PROJ, box ,info.pos, info.head, this);
			return DoBeginFiring();
			break;
		default:
			info.anim = STANDING;
			break;
	}
	//Should never reach here
	//assert(0);
	return false;
}

bool CServerConnect::DoAction2()
{
	switch (currentWeapon)
	{
		case Ram:
		case Stomp:
			return DoTornado();
			break;
		case ArcingMortar:
			//this is temporary
			if(info.framecount == 0)
				AddObject(CONECT_MORTAR_PROJ, box ,info.pos, info.head, this);
			return DoFiring();
			break;
		case Tornado:
			if(info.framecount == 0)
				AddObject(HOMING_PROJ, box ,info.pos, info.head, this);
			return DoFiring();
		default:
			info.anim = STANDING;
			break;
	}
	//Should never reach here
	//assert(0);
	return false;
}

bool CServerConnect::DoEndAction2()
{
	switch (currentWeapon)
	{
		case Ram:
		case Stomp:
			return DoEndTornado();
			break;
		case Tornado:
			return DoEndFiring();
		case ArcingMortar:
			return DoEndFiring();
		default:
			break;
	}
	//Should never reach here
	//assert(0);
	return false;
}
