/** ServerCube.h
  * 
  * Class to control a Rubiks Cube player on the server. Handles
  * moving, animation (well, telling the player app how it should 
  * animate), and collision with other objects. 
  *
  * Author: Kevin Audleman
  */
#ifndef __SERVER_CUBE__
#define __SERVER_CUBE__


#include <windows.h>   // Standard Windows header file 
#include <direct.h>    // DirectDraw definitions 
#include <d3drmwin.h>  // D3DRM definitions 
#include <ddraw.h>
#include "server.h"

#include "math.h"
#include "assert.h"
#include "rubiksInfo.h"
#include "ServerPlayer.h"
#include "RubicsDirect3D.h"


#define PI 3.1415926535




/************* ServerCube class ******************/
class CServerCube : public CServerPlayer
{

public:
	CServerCube();
	CServerCube(int num_players);
	~CServerCube();
	virtual bool PickUp(PieceType type);
	//Pick a random piece to drop
	virtual void DropPiece(CServerObject* object, PieceType weapon);
	CubeWeapon GetCurrentWeapon(){ return currentWeapon; }
	virtual D3DRMBOX GetBoundingBox();

protected:
	/** 
	 *	Movement functions:
	 *
	 * Returns FALSE if we're in the middle of the animation, 
	 * Returns TRUE when the final frame has been hit.
	 */
	virtual bool DoForwardHop();
	virtual bool DoBackwardHop();
	
	virtual bool DoBeginFiring();
	virtual bool DoFiring();
	virtual bool DoEndFiring();


	virtual bool DoBeginAction2();
	virtual bool DoAction2();				
	virtual bool DoEndAction2();			    

	virtual bool DoBeginTornado();
	virtual bool DoTornado();
	virtual bool DoEndTornado();

	virtual bool DoBeginStomp();
	virtual bool DoStomp();
	virtual bool DoEndStomp();

	virtual bool DoCollideWithProjectile();
	virtual bool DoCollideWithPlayer();
	virtual void TurnRight();
	virtual void TurnLeft();

	
	CubeWeapon currentWeapon;

};

#endif