// ServerHoming.cpp: implementation of the CServerHoming class.
//
//////////////////////////////////////////////////////////////////////

#include "ServerHoming.h"
#include "Server.h"
#include <math.h>
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#define PI 3.1415926535f
#define DEFAULT_DAMAGE 50
#define HOMING_DAMAGE 30
#define TARGET_DISTANCE 150
#define TARGET_WIDTH 40

CServerHoming::CServerHoming() : CServerWeapon()
{


	TargetPiece();
}


CServerHoming::CServerHoming(Position p, D3DRMBOX b) : CServerWeapon(p, b)
{

	TargetPiece();
}
CServerHoming::CServerHoming(long PieceID, Position p,D3DRMBOX b) : CServerWeapon(PieceID, p, b){

	TargetPiece();
}

CServerHoming::CServerHoming(long PieceID, Position p,D3DRMBOX b, CServerObject * own) : CServerWeapon(PieceID, p, b, CUBE_PROJECTILE, own) {

	TargetPiece();
}

CServerHoming::~CServerHoming()
{

}


void CServerHoming::TargetPiece() {
	MAP_OBJECTS::iterator wall_it;
	LPServerObject other_object = NULL;
	//D3DVECTOR normal;
	bool reset = true;
	Position pos;
	
	if(owner) {
		info.head = owner->GetOrientation();

		MAP_OBJECTS::iterator it;
		MAP_OBJECTS *obj;
		obj = GetPlayersMap();
		it = obj->begin();
		float distance;
		float best_distance = 10000;  //best distance is infinity
		float angle;
		for(; it != obj->end(); it++)
		{	
			if(owner != (*it).second) 
			{
				pos = (*it).second->GetPosition();
				distance = sqrt((pos.x - info.pos.x) * (pos.x - info.pos.x) + (pos.y - info.pos.y) * (pos.y - info.pos.y) + (pos.z - info.pos.z) * (pos.z - info.pos.z));
				angle = ((pos.x - info.pos.x) * cos(info.head) - (pos.z - info.pos.z) * sin(info.head)) / distance;
				if(angle > 0.7)
				{
					if(distance < best_distance)
					{
						target = (*it).first;
						best_distance = distance;
					}
				}
			}
		}
	}
}
void CServerHoming::Update() {
	//info.head = fmod(info.head + delta_theta, 2*PI);	
	Position pos;
	LPServerObject target_object;
	
	info.pos.x += velocity * D3DVAL(cos(info.head));
	info.pos.z -= velocity * D3DVAL(sin(info.head));

	if(velocity && (target_object = GetObjectFromID(target))) {
		pos = target_object->GetPosition();
		
		float dx = (pos.x - info.pos.x);
		float dz = (pos.z - info.pos.z);
		float angle = atan2(-dz,dx);

		while(info.head < angle - PI)
			info.head += 2.0f * PI;
		while(info.head > angle + PI)
			info.head -= 2.0f * PI;

		if(info.head < angle)
		{
			info.head += 0.025f;
		}else
		{
			info.head -= 0.025f;
		}
		if(pos.y + 1.6 < info.pos.y)
		{
			info.pos.y -= 0.5;
		}else if (pos.y + 1.0 > info.pos.y)
		{
			info.pos.y += 0.5;
		}


		D3DVECTOR normal;
		rubiks_info temp_info;
		CServerObject * other_object;

		//Make a copy of our info (we may need to erase it in case of a collision)
		info.copy(temp_info);
		
		//Check for a collision 
	
		MAP_OBJECTS::iterator wall_it;
		bool reset = true;
		while (Collision(this, &other_object,&normal,wall_it,reset,WALL_OBJECT))
		{
			reset = false;
			CollideWith(other_object);
			other_object->CollideWith(this);
		}
	}
	
}

bool CServerHoming::CollideWith(CServerObject * object) {
	rubiks_info temp_info(info);
	if(object != owner ) {
		if(object) {
			if(object->IsPlayer()) {
				((CServerPlayer *)object)->DoDamage(HOMING_DAMAGE, this, CUBE_PROJECTILE);
			}
			if(object->IsWall() && projectile_type==RICOCHETE_PROJ) {

				info.head = NewHeading(collide_normal);		
				info.pos.x += velocity * D3DVAL(cos(info.head));
				info.pos.z -= velocity * D3DVAL(sin(info.head));
				if(collide(this,object,NULL))
				{
					info.head = temp_info.head;
					float temp = collide_normal.x;
					collide_normal.x = collide_normal.z;
					collide_normal.z = -temp;
					info.head = NewHeading(collide_normal);		
					info.pos.x = temp_info.pos.x + velocity * D3DVAL(cos(info.head));
					info.pos.z = temp_info.pos.z - velocity * D3DVAL(sin(info.head));
				}
				if(collide(this,object,NULL))
				{
					info.head = temp_info.head + PI;
					info.pos.x = temp_info.pos.x + velocity * D3DVAL(cos(info.head));
					info.pos.z = temp_info.pos.z - velocity * D3DVAL(sin(info.head));
				}

			}else{
				RemoveObjectFromGame(this);
			}
		} else {
			//We hit the floor so remove the object
			RemoveObjectFromGame(this);
		}
	}
	return false;
}

D3DRMBOX CServerHoming::GetBoundingBox() {
	/*
	if(targeting) {
		return target_box;
	} else */
	return box;
}

Position CServerHoming::GetPosition() {
	/*
	Position target_pos;
	if(targeting) {
		target_pos = info.pos;
		target_pos.x += TARGET_DISTANCE/2.0f * D3DVAL(cos(info.head));
		target_pos.z -= TARGET_DISTANCE/2.0f * D3DVAL(sin(info.head));
		return target_pos;
	} else 
	*/
	return info.pos;
}
