// ServerMortar.cpp: implementation of the CServerMortar class.
//
//////////////////////////////////////////////////////////////////////

#include "ServerMortar.h"
#include "Server.h"
#include <math.h>


#define MAX_MORTAR_ANIM 50.0f
#define MAX_SCALE 10.0f

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServerMortar::CServerMortar() : CServerWeapon()
{

	yVelocity = 4.0f;
	velocity = 2.0f;
	

	/* Make up the bounding box from here */
	box.min.x = -2.5f;
	box.min.y =  0.0f;
	box.min.z = -2.5f;
	box.max.x =  2.5f;
	box.max.y =  5.0f;
	box.max.z =  2.5f;
}


CServerMortar::CServerMortar(Position p, D3DRMBOX b) : CServerWeapon(p, b)
{


	yVelocity = 4.0f;
	velocity = 2.0f;
	

	/* Make up the bounding box from here */
	box.min.x = -2.5f;
	box.min.y =  0.0f;
	box.min.z = -2.5f;
	box.max.x =  2.5f;
	box.max.y =  5.0f;
	box.max.z =  2.5f;
}
CServerMortar::CServerMortar(long PieceID, Position p,D3DRMBOX b) : 
CServerWeapon(PieceID, p, b)
{

	yVelocity = 4.0f;
	velocity = 2.0f;
	

	/* Make up the bounding box from here */
	box.min.x = -2.5f;
	box.min.y =  0.0f;
	box.min.z = -2.5f;
	box.max.x =  2.5f;
	box.max.y =  5.0f;
	box.max.z =  2.5f;
}

CServerMortar::CServerMortar(long PieceID, Position p,D3DRMBOX b, CServerObject * own) :
CServerWeapon(PieceID, p, b, CUBE_PROJECTILE, own) 
{

	yVelocity = 4.0f;
	velocity = 2.0f;
	

	/* Make up the bounding box from here */
	box.min.x = -2.5f;
	box.min.y =  0.0f;
	box.min.z = -2.5f;
	box.max.x =  2.5f;
	box.max.y =  5.0f;
	box.max.z =  2.5f;
}

CServerMortar::CServerMortar(long PieceID, Position p,D3DRMBOX b, 
							 PieceType t,  CServerObject * own) :
CServerWeapon(PieceID, p, b, CUBE_PROJECTILE, own) 
{
	Type = t;
	yVelocity = 4.0f;
	velocity = 2.0f;

	/* Make up the bounding box from here */
	box.min.x = -2.5f;
	box.min.y =  0.0f;
	box.min.z = -2.5f;
	box.max.x =  2.5f;
	box.max.y =  5.0f;
	box.max.z =  2.5f;
}


CServerMortar::~CServerMortar()
{

}


void CServerMortar::Update() {
	bool onGround = false;
	// unreferenced:  rubiks_info temp_info;
	CServerObject * other_object;
	D3DVECTOR normal;
	// unreferenced:  Position pos;
	// unreferenced:  Position vel;
	// unreferenced:  float dot;
	
	info.framecount++;
	if(info.framecount > MAX_MORTAR_ANIM) {
		info.framecount = MAX_MORTAR_ANIM;
	}

	//GRAVITY
	yVelocity -= 0.4f; 

	if (yVelocity < 0.0f && abs(yVelocity) > (box.max.y - box.min.y))
		yVelocity = -1.0f * (box.max.y - box.min.y);

	info.pos.y += yVelocity;

	info.pos.x += velocity * D3DVAL(cos(info.head));
	info.pos.z -= velocity * D3DVAL(sin(info.head));

	/* Change in x velocity due to wind resistance */
	if (velocity > 0.0) velocity -= 0.05;
	
	MAP_OBJECTS::iterator wall_it;
	//Check for a collision 
	if (Collision(this, &other_object,&normal,wall_it, true, WALL_OBJECT)
		|| info.pos.y < 0.0f)
	{				
		RemoveObjectFromGame(this);
	}
}

D3DRMBOX CServerMortar::GetBoundingBox() {

	
	return box;
/*	D3DRMBOX temp_box;
	float scale_factor;
	if(info.framecount  <= 1)
		scale_factor = 1.0f;
	else
		scale_factor = (info.framecount / MAX_MORTAR_ANIM) *MAX_SCALE;

	scale_factor = 5.0f;
	temp_box.max.x = box.max.x * (scale_factor);
	temp_box.max.y = box.max.y * (scale_factor);
	temp_box.max.z = box.max.z * (scale_factor);
	temp_box.min.x = box.min.x * (scale_factor);
	temp_box.min.y = box.min.y; // * (scale_factor);
	temp_box.min.z = box.min.z * (scale_factor);
	return temp_box;
	*/
}
