
#include "messages.h"
#include "ServerPlayer.h"
#include "rubiksInfo.h"

/*CServerPlayer is the super class for all the player subclasses*/
CServerPlayer::CServerPlayer()
{

	info.head = 0.0f;
	info.pos.x = 0.0f;
	info.pos.y = 0.0f;
	info.pos.z = 0.0f;
	info.framecount = 0.0f;
	info.anim = STANDING;
	info.hitpoints = 150;

	for (int i = 0; i < 6; i++)
		centerSquare[i] = false;

	CannonPower1 = FALSE;
	CannonPower2 = FALSE;

	velocity = 0.0f;
	action.velocity = 0.0f;
	action.theta =  0.0f;

	yVelocity = 0.0f;
	onGround = false;
	totalDamage = 0;
	

	dontTouch = false;
	gravity = true;
}

void CServerPlayer::ReceiveAction(Action &a)
{
	memcpy(&action, &a, sizeof(Action));
}

void CServerPlayer::GetMessage(rubiks_info* buffer)
{
	memcpy(buffer, &info, sizeof(rubiks_info));
}


bool CServerPlayer::CollideWith(CServerObject * object)
{
	info.anim = COLLIDEWITHPLAYER;
	dontTouch = false;

	DoDamage(80, object, RUBICS_CUBE);
	info.framecount = 0;
	GetCollideHeadingAndVelocity(((CServerMovingObject*)object)->info.head);
	collideVel *= 2.0f;
	//Fly into the air.
	yVelocity = 2.4f;

	return false;
}




/** The main action sequence. This function takes in the 
  * current message and a list of all the objects on the map,
  * and does the following things:
  * 1) Moves the cube based on the move action
  * 2) Checks for collisions. 
  */

void CServerPlayer::Update()
{
	
	rubiks_info temp_info;
	CServerObject * other_object;
	D3DVECTOR normal;
	Position pos, projectile_pos;
	Position velocity;
	float dot;
	float my_temp;

	pos.x = info.pos.x;
	pos.y = info.pos.y;
	pos.z = info.pos.z;

	
	if (info.anim != COLLIDEWITHPLAYER && info.anim != COLLIDEWITHPROJECTILE)
	{
		/**** Decide how (and if) to handle a new message *******************/
		if (info.anim == STANDING || info.anim == HOPPING || info.anim == BACKWARDS)
		{
			AnimationType old = info.anim;
	
			/* The order of these statements provide a priority for the
			 * different actions. Don't mess it up!*/
			if (action.action1 == TRUE)       info.anim = BEGINFIRING;
			else if (action.action2 == TRUE)  info.anim = BEGINACTION2;
			else if (action.velocity > 0)     info.anim = HOPPING;
			else if (action.velocity < 0)     info.anim = BACKWARDS;
			else                              info.anim = STANDING;

		
			if (info.anim != old) info.framecount = 0;
		}

		/******** Jumping code ****************************/
		if (action.jump == TRUE && onGround == true 
			&& info.anim != COLLIDEWITHPROJECTILE 
			&& info.anim != COLLIDEWITHPLAYER)
		{
			onGround = false;
			yVelocity = 3.0f;
		}

		
	}
	
	//Make a copy of our info (we will need to erase our move it in case of a collision)
	info.copy(temp_info);
				


	/************* Turning *****************************/
		if (action.theta > 0.0f) TurnLeft();
		if (action.theta < 0.0f) TurnRight();
		
	/**************** Animation ************************/
	switch (info.anim)
	{		
		case STANDING:
			break;
		case HOPPING:
			if (DoForwardHop())	info.anim = STANDING;
			break;
		case BACKWARDS:
			if (DoBackwardHop()) info.anim = STANDING;
			break;

		case BEGINFIRING:
			if (DoBeginFiring())
			{
				info.anim = FIRING;
				projectile_pos = info.pos;
				projectile_pos.y += 2.0f;
				CreateProjectile(projectile_pos, info.head, CannonPower2, this);
				if(CannonPower1)
				{
					//create multiple projectiles
					CreateProjectile(projectile_pos, (info.head + 3.14f/4.0f), CannonPower2, this);
					CreateProjectile(projectile_pos, (info.head - 3.14f/4.0f), CannonPower2, this);
				}

			}
			break;

		case FIRING:
			if (DoFiring()) 
				if (action.action1 == TRUE) 
				{
				  projectile_pos = info.pos;
				  projectile_pos.y += 2.0f;

				  CreateProjectile(projectile_pos, info.head, CannonPower2, this);
				  /*if(CannonPower1)
					{	
						//create multiple projectiles
						CreateProjectile(projectile_pos, (info.head + 3.14f/4.0f), CannonPower2, this);
						CreateProjectile(projectile_pos, (info.head - 3.14f/4.0f), CannonPower2, this);
					}*/
				}
				else 
				  info.anim = ENDFIRING;
			break;

		case ENDFIRING:
			if (DoEndFiring()) info.anim = STANDING;
			break;

		case BEGINACTION2:
			if (DoBeginAction2()) info.anim = ACTION2;
			break;

		case ACTION2:
			if (DoAction2()) 
				if (action.action2 == FALSE) 
					info.anim = ENDACTION2;
			break;

		case ENDACTION2:
			if (DoEndAction2()) info.anim = STANDING;
			break;

		case COLLIDEWITHPROJECTILE:
			if (DoCollideWithProjectile()) info.anim = STANDING;
			break;
		case COLLIDEWITHPLAYER:
			if (DoCollideWithPlayer()) info.anim = STANDING;
			break;
		default:
			OutputDebugString("XXXXXXXXXXXXXXXXX:: Default case. What went wrong??");
	}


	/******** Gravity **********************************/
	if (gravity)
	{
		if (yVelocity > -3.6f) yVelocity -= 0.2f;
		
		info.pos.y += yVelocity;
		if (info.pos.y < 0.0f)
		{
			info.pos.y = 0.0f;
		    onGround = true;
		}
	}

	/********** Collision code from here on down ***********/
	/********** Collision code from here on down ***********/
	/********** Collision code from here on down ***********/

	/* If we're in a collision animation, then we're moving along the collideHead
	 * instead of the info.head. This breaks the collision code, so we'll 
	 * set info.head to collideHead and undo it afterwards.
	 */
	// unreferenced: D3DVALUE head;
	if (info.anim == COLLIDEWITHPROJECTILE || info.anim == COLLIDEWITHPLAYER)
	{
//		head = info.head;
//		info.head = collideHead;
	}

	//Determine velocity of object
	velocity.x = info.pos.x - temp_info.pos.x;
	velocity.z = info.pos.z - temp_info.pos.z;
	velocity.y = info.pos.y - temp_info.pos.y;
	
	int num_times = 0;
	bool collisions = true;
	bool reset;
	bool next_reset = true;

	MAP_OBJECTS::iterator proj_it;
	MAP_OBJECTS::iterator wall_it;
	MAP_OBJECTS::iterator player_it;
	//Check for a collision 
	while (collisions && num_times < 6)	
	{	
		reset = next_reset;
		next_reset = false;
		collisions = false;
		num_times++;
	
		
		/************************CASE 1 - Wall (or floor) *************************/
		if (Collision(this, &other_object,&normal,wall_it,reset,WALL_OBJECT))
		{				
			//Tell the other object we hit it.
			other_object->CollideWith(this);

			//we need to keep colliding with objects, and reset the iterators
			collisions = true;
			float my_temp;


			my_temp = info.pos.y;
			//Check if the collision was caused by gravity;
			info.pos.y = temp_info.pos.y;
			if(collide(this,other_object,NULL))
			{
				//make sure the normal is acceptable
				if(normal.y > 0.0f)
				{
					normal.y = 0.0f;
					normal.x = 1.0f;
				}
				info.pos.y = my_temp;
			//	temp_info.pos.x = info.pos.x;
			//	temp_info.pos.y = info.pos.y;
			//	temp_info.pos.z = info.pos.z; 
				//calculate the dot product, and the new bounced position
				dot = velocity.x * normal.x + velocity.y * normal.y + velocity.z * normal.z;
				info.pos.x -= 2 * dot * normal.x;
				info.pos.z -= 2 * dot * normal.z;
				info.pos.y -= 2 * dot * normal.y;
				//check if collision was on the corner of the object, and hit
				//multiple sides
				if(collide(this,other_object,NULL))
				{
					
					//Reset the position of the object
					info.pos.x = temp_info.pos.x;
					info.pos.y = temp_info.pos.y;
					info.pos.z = temp_info.pos.z; 
				
					my_temp = normal.x;
					normal.x = -normal.z;
					normal.z = my_temp;
					
			//		temp_info.pos.x = info.pos.x;
			//		temp_info.pos.y = info.pos.y;
			//		temp_info.pos.z = info.pos.z; 
					//calculate the dot product, and the new bounced position
					dot = velocity.x * normal.x + velocity.y * normal.y + velocity.z * normal.z;
					info.pos.x -= 2 * dot * normal.x;
					info.pos.z -= 2 * dot * normal.z;
					info.pos.y -= 2 * dot * normal.y;
				}

				if(collide(this,other_object,NULL))
				{
					
					//Reset the position of the object
					info.pos.x = temp_info.pos.x;
					info.pos.y = temp_info.pos.y;
					info.pos.z = temp_info.pos.z; 
					//Undo the rotation
					info.head = temp_info.head;
				}




			}
			else
			{
				//if the collision was due to falling set the position to the top
				//of the object;
				//If we're colliding with the ground, set the  boolean so we'll know
				if(velocity.y > 0.0)
				{
					info.pos.y = other_object->GetPosition().y + other_object->GetBoundingBox().min.y - this->GetBoundingBox().max.y - 0.01;
					
				}else
				{
					onGround = true;
					info.pos.y = other_object->GetBoundingBox().max.y 
							+ other_object->GetPosition().y + 0.01 + this->GetBoundingBox().min.y;
				}
						
			}
			//set to reset all iterators during next pass because position changed
			next_reset = true;
			//Determine velocity of object
			velocity.x = info.pos.x - temp_info.pos.x;
			velocity.z = info.pos.z - temp_info.pos.z;
			velocity.y = info.pos.y - temp_info.pos.y;
		}

		if (info.anim != COLLIDEWITHPLAYER && info.anim != COLLIDEWITHPROJECTILE)
		{
			/******************* CASE 2 - Projectile *******************/
			if(Collision(this, &other_object,&normal,proj_it,reset,PROJECTILE_OBJECT))
			{	
			  //we collided so we need to keep colliding with objects
				collisions = true;
				//Tell the other object we hit it.
				other_object->CollideWith(this);

			  if (other_object->IsProjectile() && other_object->GetOwner() != this) 
			  {
				info.anim = COLLIDEWITHPROJECTILE;
				info.framecount = 0;

				// Find our new heading and velocity  
				GetCollideHeadingAndVelocity(((CServerMovingObject*)other_object)->info.head);
			  }
			}
		}
			/********************* CASE 3 - Player  **********************/
			if(Collision(this, &other_object,&normal,player_it,reset,PLAYER_OBJECT))
			{
				collisions = true;

				//If the player is in an attack that makes his body bad to touch, get hit.
				//if (((CServerPlayer*)other_object)->dontTouch == true)
				if (dontTouch)	((CServerPlayer*)other_object)->CollideWith(this);
					/*
					info.anim = COLLIDEWITHPLAYER;
					info.framecount = 0;
					GetCollideHeadingAndVelocity(((CServerMovingObject*)other_object)->info.head);
					collideVel *= 2.0f;
					//Throw the guy in the air.
					yVelocity = 2.4f;
					*/
				
					/// BRGIN HUGE CHUNK OF COPIED CODE///
				
					my_temp = info.pos.y;
					//Check if the collision was caused by gravity;
					info.pos.y = temp_info.pos.y;
					if(collide(this,other_object,NULL))
					{
						//make sure the normal is acceptable
						if(normal.y > 0.0f)
						{
							normal.y = 0.0f;
							normal.x = 1.0f;
						}
						info.pos.y = my_temp;
						//calculate the dot product, and the new bounced position
						dot = velocity.x * normal.x + velocity.y * normal.y + velocity.z * normal.z;
						info.pos.x -= 2 * dot * normal.x;
						info.pos.z -= 2 * dot * normal.z;
						info.pos.y -= 2 * dot * normal.y;
						//check if collision was on the corner of the object, and hit
						//multiple sides
						if(collide(this,other_object,NULL))
						{
							
							//Reset the position of the object
							info.pos.x = temp_info.pos.x;
	 						info.pos.y = temp_info.pos.y;
							info.pos.z = temp_info.pos.z; 

							my_temp = normal.x;
							normal.x = -normal.z;
							normal.z = my_temp;
							
							//calculate the dot product, and the new bounced position
							dot = velocity.x * normal.x + velocity.y * normal.y + velocity.z * normal.z;
							info.pos.x -= 2 * dot * normal.x;
							info.pos.z -= 2 * dot * normal.z;
							info.pos.y -= 2 * dot * normal.y;
						}
						if(collide(this,other_object,NULL))
						{
							
							//Reset the position of the object
							info.pos.x = temp_info.pos.x;
							info.pos.y = temp_info.pos.y;
							info.pos.z = temp_info.pos.z; 
							//Undo the rotation
							info.head = temp_info.head;
						}


					}
					else
					{
						//if the collision was due to falling set the position to the top
						//of the object;
						//If we're colliding with the ground, set the  boolean so we'll know
						if(velocity.y > 0.0)
						{
							info.pos.y = other_object->GetPosition().y + other_object->GetBoundingBox().min.y - this->GetBoundingBox().max.y - 0.01;
							
						}else
						{
							onGround = true;
							info.pos.y = other_object->GetBoundingBox().max.y 
									+ other_object->GetPosition().y + 0.01 + this->GetBoundingBox().min.y;
						}
					}
				//set to reset all iterators during next pass because position changed
				next_reset = true;
				//Determine velocity of object
				velocity.x = info.pos.x - temp_info.pos.x;
				velocity.z = info.pos.z - temp_info.pos.z;
				velocity.y = info.pos.y - temp_info.pos.y;
			  //////////////END HUGE CHUNKA COPIED CODE//////////////////////////
			} 

	}

	if (Collision(this, &other_object,&normal,wall_it,true,WALL_OBJECT) || Collision(this, &other_object,&normal,wall_it,true,PLAYER_OBJECT))
	{
		//Reset the position of the object
		info.pos.x = temp_info.pos.x;
		info.pos.y = temp_info.pos.y;
		info.pos.z = temp_info.pos.z; 
		//Undo the rotation
		info.head = temp_info.head;
	}

	/* Undo the setting of the heading */
	if (info.anim == COLLIDEWITHPROJECTILE || info.anim == COLLIDEWITHPLAYER)
	{
//		info.head = head;
	}
}


//This function will record the damage done by a weapon.
void CServerPlayer::DoDamage(int damage, CServerObject* object, PieceType weapon) 
{
	info.hitpoints -= damage; 

	if (info.hitpoints > 0) return;

	DropPiece(object, weapon);
	info.hitpoints = 150;
}

/* Sets the collideHead and collideVel variables to the proper direction
 * to move in for the collision animations. Based on our current heading
 * and the heading of the object that hit us. 
 */
void CServerPlayer::GetCollideHeadingAndVelocity(D3DVALUE otherHead)
{
	if (otherHead - info.head < 1.57f) 
			{
				collideHead = (info.head + otherHead) / 2.0f;
				collideVel = 3.0f;
			}
			else if (otherHead - info.head < 3.14f)
			{
				collideHead = info.head + 3.14f;
				collideHead = (collideHead + otherHead)/2.0f;
				collideVel = 2.0f;
			}
			else //angle is greater than 3.14
			{
				otherHead = 6.28f - otherHead;

				if (otherHead - info.head < 1.57f) 
				{
					collideHead = (info.head + otherHead) / 2.0f;
					collideVel = 3.0f;
				}
				else //if (otherHead - info.head < 3.14f)
				{	
					collideHead = info.head + 3.14f;
					collideHead = (collideHead + otherHead)/2.0f;
					collideVel = 2.0f;
				}	
			
				collideHead = 6.28f - collideHead;
			}
}