#ifndef _SERVER_PLAYER_H_
#define _SERVER_PLAYER_H_

#include "ServerMovingObject.h"
#include <math.h>
#include "Server.h"
#include "RubicsDirect3D.h"

// Limits for to keep player on the board
#define MAX_X (BOARD_X+17)
// bad naming style here cause the board Y is really the Z axis
#define MAX_Z (BOARD_Y+17)

class CServerPlayer : public CServerMovingObject
{
public:

	CServerPlayer();
	


	/* What should this return? */
	virtual void Update();
	virtual bool PickUp(PieceType type) = 0;
	//Pick a random piece to drop
	virtual void DropPiece(CServerObject* object, PieceType weapon) = 0;

	virtual bool CollideWith(CServerObject * object);

	virtual void ReceiveAction(Action &a);
	virtual void GetMessage(rubiks_info *buffer);

public:
/*
Variables inherited from CMovingObject
	Action      action;
	rubiks_info info; 
*/

	bool IsPlayer() { return true; }
	bool IsWall() { return false; }
	int GetTeam() { return team; }
	
	virtual void DoDamage(int damage, CServerObject* object, PieceType weapon);

	void GetCollideHeadingAndVelocity(D3DVALUE otherHead);

	/** Movement functions:
	 *  
	 *  Returns FALSE if we're in the middle of the animation, 
	 *  Returns TRUE when the final frame has been hit.
	 */
	virtual bool DoForwardHop()              {return true;}
	virtual bool DoBackwardHop()             {return true;}

	virtual bool DoBeginFiring()             {return true;}
	virtual bool DoFiring()                  {return true;}
	virtual bool DoEndFiring()               {return true;}

	virtual bool DoBeginAction2()			 {return true;}
	virtual bool DoAction2()				 {return true;}
	virtual bool DoEndAction2()			     {return true;}

	virtual bool DoCollideWithProjectile()   {return true;}
	virtual bool DoCollideWithPlayer()       {return true;}
	virtual bool DoCollideWithWall()         {return true;}

	virtual void TurnRight() {}
	virtual void TurnLeft()  {}

public:

	//Orientation to follow when in CollideWithProjectile animation.
	//Use this variable so we don't have to change the actual orientation
	//of the player, which would move the camera all around when you're hit.
	D3DVALUE collideHead;
	//Ditto with the velocity
	D3DVALUE collideVel;

	//Tells whether or not touching us results in taking damage
	bool dontTouch;

	bool onGround, gravity;
	int totalDamage;
	BOOL centerSquare[6];
	//Team# of the player. An int for scalability but we'll only have 2 teams: 1 & 2.
	int team;
	
	//Powerups for the cannon
	BOOL CannonPower1;  //Multishot
	BOOL CannonPower2;  //Ricochete
	
};

#endif //_SERVER_PLAYER_H_