// SeverViewer.cpp: implementation of the CServerViewer class.
//
//////////////////////////////////////////////////////////////////////

#include "ServerViewer.h"
#include "Server.h"
#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServerViewer::~CServerViewer()
{

}

void CServerViewer::ReceiveAction(Action &a)
{
	memcpy(&action, &a, sizeof(Action));
}

void CServerViewer::Update() {
	const float PI = 3.1459f;
	//Orientation for the camera
	float orient;
	float theta = 0.0f;

	if(!action.strafe)
	//Turn Left or Right
	{
		if(action.theta == -DEFAULT_THETA)
			theta = TURNING_RATE;
		else if(action.theta < 0 && action.theta != -DEFAULT_THETA)
			theta = 2.5f*TURNING_RATE;
		else if(action.theta == DEFAULT_THETA)
			theta = -TURNING_RATE;
		else if(action.theta > 0 && action.theta != DEFAULT_THETA)
			theta = 2.5f*-TURNING_RATE;

		info.head = fmod(info.head + theta, 2*PI);	
		info.pos.x += action.velocity * D3DVAL(cos(info.head));
		info.pos.z -= action.velocity * D3DVAL(sin(info.head));
	//Strafe Left or Right
	}else{
		if(action.theta < 0){
			velocity = -1.0f;
			info.pos.x += velocity * (D3DVALUE)cos(info.head - 1.57f);
			info.pos.z -= velocity * (D3DVALUE)sin(info.head - 1.57f);
		}else if(action.theta > 0){
			velocity = -1.0f;
			info.pos.x += velocity * (D3DVALUE)cos(info.head + 1.57f);
			info.pos.z -= velocity * (D3DVALUE)sin(info.head + 1.57f);
		}
	}
	
	// Jump to next player's position & orientation
	if(action.action1 && !space_down){
		space_down = true;
		info.pos = GetNextCameraPos(&orient);
		info.head = orient;
	}else if(!action.action1 && space_down){
		space_down = false;
	}
	
	// Move Up
	if(action.jump) {
		info.pos.y += 0.5f;
	// Move Down but not < 0
	}else if(action.action2) {
		info.pos.y -= 0.5f;
		if(info.pos.y < 0)
			info.pos.y = 0;
	}
}

void CServerViewer::GetMessage(rubiks_info* buffer)
{
	memcpy(buffer, &info, sizeof(rubiks_info));
}

bool CServerViewer::IsMoveable() {
	return true;
}

Position CServerViewer::GetPosition() {
	return info.pos;
}

D3DVALUE CServerViewer::GetOrientation() {
	return info.head;
}

CServerViewer::CServerViewer() : CServerMovingObject()
{
	space_down = false;
}

CServerViewer::CServerViewer(long PieceID, Position npos,D3DRMBOX nbox, CServerObject * owner) : CServerMovingObject(PieceID, npos, nbox, owner) {
	space_down = false;
}

CServerViewer::CServerViewer(long PieceID, Position npos,D3DRMBOX nbox) : CServerMovingObject(PieceID, npos, nbox) {
	space_down = false;
}

CServerViewer::CServerViewer(Position npos,D3DRMBOX nbox) : CServerMovingObject(npos, nbox)
{
	space_down = false;
}
