// ServerWeapon.cpp: implementation of the CServerWeapon class.
//
//////////////////////////////////////////////////////////////////////

#include <math.h>
#include "ServerWeapon.h"
#include "ServerPlayer.h"
#include "RubicsDirect3D.h"

#include "rubiksInfo.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// Limits for to keep player on the board
#define MAX_X (BOARD_X+17)
// bad naming style here cause the board Y is really the Z axis
#define MAX_Z (BOARD_Y+17)

#define PI 3.1415926535
#define DEFAULT_DAMAGE 50

CServerWeapon::CServerWeapon()
{
	projectile_type = CUBE_PROJECTILE;
	info.head = 0.0f;
	
	info.pos.x = 0.0f;
	info.pos.y = 0.0f;
	info.pos.z = 0.0f;
	info.framecount = 0.0f;
	info.anim = STANDING;

	delta_theta = 0.0f;
	if(projectile_type == CUBE_PROJECTILE){
		velocity = 1.6f;
		counter = TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == MORTAR_PROJ || projectile_type == CONECT_MORTAR_PROJ) {
		velocity = 1.6f;
		counter = 2*TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == RAM_BLOCK){
		velocity = 1.6f;
		counter = -TIMER_LIFE;
	}else{
		velocity = 1.6f;
		counter = 3*TIMER_LIFE;  // destroy after 100 * 30ms
	}
}


CServerWeapon::CServerWeapon(Position p, D3DRMBOX b) : CServerMovingObject(p, b)
{
	projectile_type = CUBE_PROJECTILE;
	info.anim = STANDING;
	delta_theta = 0.0f;

	if(projectile_type == CUBE_PROJECTILE){
		velocity = 1.6f;
		counter = TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == MORTAR_PROJ || projectile_type == CONECT_MORTAR_PROJ) {
		velocity = 1.6f;
		counter = 2*TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == RAM_BLOCK){
		velocity = 1.6f;
		counter = -TIMER_LIFE;
	}else{
		velocity = 1.6f;
		counter = 3*TIMER_LIFE;  // destroy after 100 * 30ms
	}
}


CServerWeapon::CServerWeapon(long PieceID, Position p,D3DRMBOX b) : CServerMovingObject(PieceID, p, b){
	projectile_type = CUBE_PROJECTILE;
	info.anim = STANDING;
	delta_theta = 0.0f;

	if(projectile_type == CUBE_PROJECTILE){
		velocity = 1.6f;
		counter = TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == MORTAR_PROJ || projectile_type == CONECT_MORTAR_PROJ) {
		velocity = 1.6f;
		counter = 2*TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == RAM_BLOCK){
		velocity = 1.6f;
		counter = -TIMER_LIFE;
	}else{
		velocity = 1.6f;
		counter = 3*TIMER_LIFE;  // destroy after 100 * 30ms
	}
}

CServerWeapon::CServerWeapon(long PieceID, Position p,D3DRMBOX b, PieceType type, CServerObject * own) :CServerMovingObject(PieceID, p, b, own) {
	projectile_type = type;
	info.anim = STANDING;
	delta_theta = 0.0f;

	if(projectile_type == CUBE_PROJECTILE){
		velocity = 1.6f;
		counter = TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == MORTAR_PROJ || projectile_type == CONECT_MORTAR_PROJ) {
		velocity = 1.6f;
		counter = 2*TIMER_LIFE;  // destroy after 100 * 30ms
	} else if(projectile_type == RAM_BLOCK){
		velocity = 1.6f;
		counter = -TIMER_LIFE;
	}else{
		velocity = 1.6f;
		counter = 3*TIMER_LIFE;  // destroy after 100 * 30ms
	}
}




CServerWeapon::~CServerWeapon()
{

}

//This has been moved up to CServerMovingObject : Aaron
/*
void CServerWeapon::Update()
{
	//info.head = fmod(info.head + delta_theta, 2*PI);
	info.pos.x += velocity * D3DVAL(cos(info.head));
	info.pos.z -= velocity * D3DVAL(sin(info.head));

	D3DVECTOR normal;
	rubiks_info temp_info;
	CServerObject * other_object;

	//Make a copy of our info (we may need to erase it in case of a collision)
	info.copy(temp_info);
		
	//Check for a collision 
	
	MAP_OBJECTS::iterator wall_it;
	bool reset = true;
	while (Collision(this, &other_object,&normal,wall_it,reset,WALL_OBJECT))
	{
		reset = false;
		CollideWith(other_object);
		other_object->CollideWith(this);
	}
}
*/
void CServerWeapon::TimerDestroy()
{
	if(counter == (-TIMER_LIFE)){
		return;
	}else if(counter <= 0){
		RemoveObjectFromGame(this);
	}else{
		counter--;
	}
}

float CServerWeapon::NewHeading(D3DVECTOR normal){
	float hx,hz,dot,angle;
	hx = cos(info.head);
	hz = -sin(info.head);
	dot = hx * normal.x + hz * normal.z;
	hx = hx - 2 * dot * normal.x;
	hz = hz - 2 * dot * normal.z;
	angle = atan2(-hz,hx);
	while(angle < 0) angle += 2.0f * (float)PI;
	return angle;
}

bool CServerWeapon::CollideWith(CServerObject * object) {
	rubiks_info temp_info(info);
	if(object != owner ) {
		if(object) {
			if(object->IsPlayer()) {
				((CServerPlayer *)object)->DoDamage(DEFAULT_DAMAGE, this, CUBE_PROJECTILE);
			}
			if(object->IsWall() && projectile_type==RICOCHETE_PROJ) {

				info.head = NewHeading(collide_normal);		
				info.pos.x += velocity * D3DVAL(cos(info.head));
				info.pos.z -= velocity * D3DVAL(sin(info.head));
				if(collide(this,object,NULL))
				{
					info.head = temp_info.head;
					float temp = collide_normal.x;
					collide_normal.x = collide_normal.z;
					collide_normal.z = -temp;
					info.head = NewHeading(collide_normal);		
					info.pos.x = temp_info.pos.x + velocity * D3DVAL(cos(info.head));
					info.pos.z = temp_info.pos.z - velocity * D3DVAL(sin(info.head));
				}
				if(collide(this,object,NULL))
				{
					info.head = temp_info.head + PI;
					info.pos.x = temp_info.pos.x + velocity * D3DVAL(cos(info.head));
					info.pos.z = temp_info.pos.z - velocity * D3DVAL(sin(info.head));
				}

			}else{
				RemoveObjectFromGame(this);
			}
		} else {
			//We hit the floor so remove the object
			RemoveObjectFromGame(this);
		}
	}
	return false;
}

bool CServerWeapon::IsProjectile() {
	return true;
}